////////////////////////////////////////////////////////////////////////////////////////////////////
// Histograms
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <iomanip>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include <sstream>
#include <sys/stat.h>
#include "TChain.h"
#include <TApplication.h>
#include <TCanvas.h>
#include <TMultiGraph.h>
#include <TGraph.h>
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TLegend.h>
#include <TPaveText.h>
#include <TLorentzVector.h>
#include <TStyle.h>
#include <TTree.h>
#include <TTreeReader.h>
#include <TColor.h>
#define SP12 setprecision(12) << setw(21)
#define SW5 setw(9)

using namespace std;

const double PI = 3.1415926535897932385;

void set_style();

double Observable(int iObs, double *px, double *py, double *pz, double *E);

int main()
{
// Create output folder
    mkdir("histograms", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
//--------------------------------------------------------------------------------------------------
    set_style();
    TCanvas c("c", "c", 3000, 2000);
    const string htitle("");
    c.SetLeftMargin(0.14);
    c.SetRightMargin(0.1);
    c.SetTopMargin(0.12);
    c.SetBottomMargin(0.15);
//--------------------------------------------------------------------------------------------------
// User defined colors
    TColor kColor0(10000, 0, 0, 0);
    TColor kColor1(10001, 1, 0, 0);
    TColor kColor2(10002, 0, 1, 0);
    TColor kColor3(10003, 0, 0, 1);
//--------------------------------------------------------------------------------------------------
    double born_xs = 0.0;
    double loop_xs = 0.0;

    double weightborn[1], pxborn[3], pyborn[3], pzborn[3], Eborn[3];
    double weightloop[1], pxloop[3], pyloop[3], pzloop[3], Eloop[3];

    const int NOBS = 6;
    const int LNAME = 50;

    double ObsMin[NOBS],ObsMax[NOBS];
    double ObsBornMin[NOBS],ObsBornMax[NOBS];
    double ObsLoopMin[NOBS],ObsLoopMax[NOBS];
    double ObsRatioMin[NOBS],ObsRatioMax[NOBS];

    ofstream fObs[NOBS];
    char obsName[NOBS][LNAME];
    char xTitle[NOBS][LNAME];
    char yTitle[NOBS][LNAME];
    char nameRoot[NOBS][LNAME];
    char namePdf[NOBS][LNAME];
    char nameRootRatio[NOBS][LNAME];
    char namePdfRatio[NOBS][LNAME];
    char nameDat[NOBS][LNAME];

    const int    binN[NOBS] = { 100, 100, 100, 100,  100,  100};
    const double binL[NOBS] = {  0., -1., -1., -4.,   0.,   0.};
    const double binU[NOBS] = {180.,  1.,  1.,  4., 250., 250.};

    sprintf(obsName[0], "Theta3");
    sprintf(xTitle[0], "#theta(3) [deg.]");
    sprintf(yTitle[0], "d#sigma/d#theta(3) [fb/deg.]");
    sprintf(obsName[1], "Cost3");
    sprintf(xTitle[1], "cos#theta(3)");
    sprintf(yTitle[1], "d#sigma/dcos#theta(3) [fb]");
    sprintf(obsName[2], "Y3");
    sprintf(xTitle[2], "Y(3)");
    sprintf(yTitle[2], "d#sigma/dY(3) [fb]");
    sprintf(obsName[3], "Eta3");
    sprintf(xTitle[3], "#eta(3)");
    sprintf(yTitle[3], "d#sigma/d#eta(3) [fb]");
    sprintf(obsName[4], "Pt3");
    sprintf(xTitle[4], "P_{T}(3) [GeV]");
    sprintf(yTitle[4], "d#sigma/dP_{T}(3) [fb/GeV]");
    sprintf(obsName[5], "M34");
    sprintf(xTitle[5], "M(34) [GeV]");
    sprintf(yTitle[5], "d#sigma/dM(34) [fb/GeV]");

    double binW[NOBS];
    for (int i = 0; i < NOBS; i++) {
	binW[i] = (binU[i]-binL[i])/binN[i];
    }

    for (int i = 0; i < NOBS; i++) {
	sprintf(nameRoot[i], "histograms/%s.root", obsName[i]);
	sprintf(namePdf[i], "histograms/%s.pdf", obsName[i]);
	sprintf(nameRootRatio[i], "histograms/%sRatio.root", obsName[i]);
	sprintf(namePdfRatio[i], "histograms/%sRatio.pdf", obsName[i]);
	sprintf(nameDat[i], "histograms/%s.dat", obsName[i]);
    }
//--------------------------------------------------------------------------------------------------
    TFile *fborn = new TFile("events_LO.root");
    TTree *tborn = (TTree*)fborn->Get("events_born");
    tborn->SetBranchAddress("weight_born",&weightborn);
    tborn->SetBranchAddress("px_born",&pxborn);
    tborn->SetBranchAddress("py_born",&pyborn);
    tborn->SetBranchAddress("pz_born",&pzborn);
    tborn->SetBranchAddress("E_born",&Eborn);
//--------------------------------------------------------------------------------------------------
    TFile *floop = new TFile("events_NLO.root");
    TTree *tloop = (TTree*)floop->Get("events_tot");
    tloop->SetBranchAddress("weight_tot",&weightloop);
    tloop->SetBranchAddress("px_tot",&pxloop);
    tloop->SetBranchAddress("py_tot",&pyloop);
    tloop->SetBranchAddress("pz_tot",&pzloop);
    tloop->SetBranchAddress("E_tot",&Eloop);
//--------------------------------------------------------------------------------------------------
    ifstream f_born_xs;
    f_born_xs.open("events_LO.dat");
    f_born_xs >> born_xs;
    f_born_xs.close();
    born_xs *= 1.E3;
//--------------------------------------------------------------------------------------------------
    ifstream f_loop_xs;
    f_loop_xs.open("events_NLO.dat");
    f_loop_xs >> loop_xs;
    f_loop_xs.close();
    loop_xs *= 1.E3;
//--------------------------------------------------------------------------------------------------
    int nentriesborn = tborn->GetEntries();
    int nentriesloop = tloop->GetEntries();
//--------------------------------------------------------------------------------------------------
    TH1D *hBorn[NOBS], *hBornMin[NOBS], *hBornMax[NOBS];
    TH1D *hLoop[NOBS], *hLoopMin[NOBS], *hLoopMax[NOBS];
    TH1D *hRatio[NOBS], *hRatioMin[NOBS], *hRatioMax[NOBS];
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < NOBS; i++) {
	gStyle->SetHistLineColor(10000);
	gStyle->SetHistFillColor(10000);
	gStyle->SetMarkerColor(10000);
	gStyle->SetHistLineWidth(1);

	hBorn[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBorn[i]->Sumw2();
	hBornMin[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBornMin[i]->Sumw2();
	hBornMax[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hBornMax[i]->Sumw2();

	gStyle->SetHistLineColor(10001);
	gStyle->SetHistFillColor(10001);
	gStyle->SetMarkerColor(10001);
	gStyle->SetHistLineWidth(1);

	hLoop[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoop[i]->Sumw2();
	hLoopMin[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoopMin[i]->Sumw2();
	hLoopMax[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hLoopMax[i]->Sumw2();

	hRatio[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hRatio[i]->Sumw2();
	hRatioMin[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hRatioMin[i]->Sumw2();
	hRatioMax[i] = new TH1D("","",binN[i],binL[i],binU[i]);
	hRatioMax[i]->Sumw2();
    }
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < nentriesborn; i++) {
	tborn->GetEntry(i);
	for (int j = 0; j < NOBS; j++) {
	    hBorn[j]->Fill(Observable(j, pxborn, pyborn, pzborn, Eborn), weightborn[0]*born_xs/nentriesborn/binW[j]);
	}
    }

    for (int i = 0; i < nentriesloop; i++) {
	tloop->GetEntry(i);
	for (int j = 0; j < NOBS; j++) {
	    hLoop[j]->Fill(Observable(j, pxloop, pyloop, pzloop, Eloop), weightloop[0]*loop_xs/nentriesloop/binW[j]);
	}
    }

    for (int i = 0; i < NOBS; i++) {
	hRatio[i]->Divide(hLoop[i],hBorn[i],1.,1.);
    }

    for (int i = 0; i < NOBS; i++) {
	for (int j = 1; j <= binN[i]; j++) {
	    hBornMin[i]->SetBinContent(j, hBorn[i]->GetBinContent(j)-hBorn[i]->GetBinError(j));
	    hBornMax[i]->SetBinContent(j, hBorn[i]->GetBinContent(j)+hBorn[i]->GetBinError(j));
	    hLoopMin[i]->SetBinContent(j, hLoop[i]->GetBinContent(j)-hLoop[i]->GetBinError(j));
	    hLoopMax[i]->SetBinContent(j, hLoop[i]->GetBinContent(j)+hLoop[i]->GetBinError(j));
	    hRatioMin[i]->SetBinContent(j, hRatio[i]->GetBinContent(j)-hRatio[i]->GetBinError(j));
	    hRatioMax[i]->SetBinContent(j, hRatio[i]->GetBinContent(j)+hRatio[i]->GetBinError(j));
	}
	ObsBornMin[i] = hBornMin[i]->GetBinContent(hBornMin[i]->GetMinimumBin());
	ObsBornMax[i] = hBornMax[i]->GetBinContent(hBornMax[i]->GetMaximumBin());
	ObsLoopMin[i] = hLoopMin[i]->GetBinContent(hLoopMin[i]->GetMinimumBin());
	ObsLoopMax[i] = hLoopMax[i]->GetBinContent(hLoopMax[i]->GetMaximumBin());
	ObsMin[i] = min(ObsBornMin[i], ObsLoopMin[i]);
	ObsMax[i] = max(ObsBornMax[i], ObsLoopMax[i]);
	ObsRatioMin[i] = hRatioMin[i]->GetBinContent(hRatioMin[i]->GetMinimumBin());
	ObsRatioMax[i] = hRatioMax[i]->GetBinContent(hRatioMax[i]->GetMaximumBin());
    }
//--------------------------------------------------------------------------------------------------
    TLegend *legend = new TLegend(0.15, 0.9, 0.45, 1.0);
    legend->SetFillColor(0);
    legend->SetFillStyle(4000);
    legend->SetBorderSize(0);
    legend->AddEntry(hBorn[0], "Born", "L");
    legend->AddEntry(hLoop[0], "1-loop", "L");
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < NOBS; i++) {
	c.SetLogy(0);
	hBorn[i]->SetMinimum(ObsMin[i]);
	hBorn[i]->SetMaximum(ObsMax[i]);
	hBorn[i]->GetXaxis()->SetTitle(xTitle[i]);
	hBorn[i]->GetYaxis()->SetTitle(yTitle[i]);
	hBorn[i]->DrawCopy("");
	hBorn[i]->DrawCopy("hist same");
	hLoop[i]->DrawCopy("same ");
	hLoop[i]->DrawCopy("hist same");
	legend->Draw();
	c.Update();
	c.SaveAs(nameRoot[i]);
	c.Print(namePdf[i]);

	fObs[i].open(nameDat[i]);
	fObs[i] << "Born:" << endl;
	for (int j = 1; j <= binN[i]; j++) {
	    fObs[i] << SW5 << binL[i]+binW[i]*(j-0.5) << SP12 << hBorn[i]->GetBinContent(j) << SP12 << hBorn[i]->GetBinError(j) << endl;
	}
	fObs[i] << "1-loop:" << endl;
	for (int j = 1; j <= binN[i]; j++) {
	    fObs[i] << SW5 << binL[i]+binW[i]*(j-0.5) << SP12 << hLoop[i]->GetBinContent(j) << SP12 << hLoop[i]->GetBinError(j) << endl;
	}
	fObs[i].close();

	c.SetLogy(0);
	hRatio[i]->SetMinimum(ObsRatioMin[i]);
	hRatio[i]->SetMaximum(ObsRatioMax[i]);
	hRatio[i]->GetXaxis()->SetTitle(xTitle[i]);
	hRatio[i]->GetYaxis()->SetTitle("Ratio");
	hRatio[i]->DrawCopy("");
	hRatio[i]->DrawCopy("hist same");
	c.Update();
	c.SaveAs(nameRootRatio[i]);
	c.Print(namePdfRatio[i]);
    }

    for (int i = 0; i < NOBS; i++) {
	delete hBorn[i];
	delete hBornMin[i];
	delete hBornMax[i];
	delete hLoop[i];
	delete hLoopMin[i];
	delete hLoopMax[i];
	delete hRatio[i];
	delete hRatioMin[i];
	delete hRatioMax[i];
    }

    return 0;
}

void set_style()
{
    gStyle->SetCanvasColor(kWhite);
    gStyle->SetHistFillColor(kWhite);
    gStyle->SetHistLineColor(kBlack);
    gStyle->SetHistFillStyle(0);
    gStyle->SetFillStyle(4050);
    gStyle->SetMarkerColor(kBlack);
    gStyle->SetMarkerStyle(1);
    gStyle->SetHistLineStyle(1);
    gStyle->SetHistLineWidth(1);
    gStyle->SetPaperSize(20, 28);
    gStyle->SetPalette(1);
    gStyle->SetOptStat(0);
    gStyle->SetLabelFont(42,"X");
    gStyle->SetLabelSize(0.06,"X");
    gStyle->SetLabelOffset(0.006,"X");
    gStyle->SetTitleFont(42,"X");
    gStyle->SetTitleSize(0.06,"X");
    gStyle->SetTitleOffset(1.1,"X");
    gStyle->SetTitleColor(kBlack,"X");
    gStyle->SetLabelFont(42,"Y");
    gStyle->SetLabelSize(0.06,"Y");
    gStyle->SetLabelOffset(0.006,"Y");
    gStyle->SetTitleFont(42,"Y");
    gStyle->SetTitleSize(0.06,"Y");
    gStyle->SetTitleOffset(1.1,"Y");
    gStyle->SetTitleColor(kBlack,"Y");
    gStyle->SetTextFont(42);
}

double Observable(int iObs, double *px, double *py, double *pz, double *E) {
    double Obs;
    switch (iObs) {
    case 0: // Angle theta for 3rd particle
	Obs = 180/PI*acos(pz[0]/sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0]));
	break;
    case 1: // Cosine of theta for 3rd particle
	Obs = pz[0]/sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0]);
	break;
    case 2: // Rapidity of 3rd particle
	Obs = 0.5*log((E[0]+pz[0])/(E[0]-pz[0]));
	break;
    case 3: // Pseudorapidity of 3rd particle
	Obs = 0.5*log((sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0])+pz[0])/(sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0])-pz[0]));
	break;
    case 4: // Transverse momentum of 3rd particle
	Obs = sqrt(px[0]*px[0]+py[0]*py[0]);
	break;
    case 5: // Invariant mass of 3rd and 4th particles
	Obs = sqrt((E[0]+E[1])*(E[0]+E[1])-(px[0]+px[1])*(px[0]+px[1])-(py[0]+py[1])*(py[0]+py[1])-(pz[0]+pz[1])*(pz[0]+pz[1]));
	break;
    default:
	Obs = 0.;
    }
    return Obs;
};
