////////////////////////////////////////////////////////////////////////////////////////////////////
// Histograms
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <iostream>
#include <iomanip>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include <sstream>
#include <sys/stat.h>
#include "TChain.h"
#include <TApplication.h>
#include <TCanvas.h>
#include <TMultiGraph.h>
#include <TGraph.h>
#include <TFile.h>
#include <TH1D.h>
#include <TH2D.h>
#include <TLegend.h>
#include <TPaveText.h>
#include <TLorentzVector.h>
#include <TStyle.h>
#include <TTree.h>
#include <TTreeReader.h>
#include <TColor.h>
#define SP12 setprecision(12) << setw(21)
#define SW5 setw(9)

using namespace std;

const double PI = 3.1415926535897932385;

void set_style();

double Observable(int iObs, double *px, double *py, double *pz, double *E);

int main()
{
// Create output folder
    mkdir("histograms", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
//--------------------------------------------------------------------------------------------------
    set_style();
    TCanvas c("c", "c", 3000, 2000);
    const string htitle("");
    c.SetLeftMargin(0.14);
    c.SetRightMargin(0.1);
    c.SetTopMargin(0.12);
    c.SetBottomMargin(0.15);
//--------------------------------------------------------------------------------------------------
// User defined colors
    TColor kColor0(10000, 0, 0, 0);
    TColor kColor1(10001, 1, 0, 0);
    TColor kColor2(10002, 0, 1, 0);
    TColor kColor3(10003, 0, 0, 1);
//--------------------------------------------------------------------------------------------------
    double loop_xs = 0.0;
    const int LNAME = 50;
    double weightloop[1], pxloop[3], pyloop[3], pzloop[3], Eloop[3];

    char obsName[2][LNAME];
    char xTitle[LNAME];
    char yTitle[LNAME];
    char zTitle[LNAME];
    char nameRoot[LNAME];
    char namePdf[LNAME];

    const int    binN[2] = { 100,  100};
    const double binL[2] = {  0.,   0.};
    const double binU[2] = {180.,  10.};

    sprintf(obsName[0], "Theta3");
    sprintf(obsName[1], "E5");
    sprintf(xTitle, "#theta(3) [deg.]");
    sprintf(yTitle, "E(5) [GeV]");
    sprintf(zTitle, "d#sigma/d#theta(3)dE(5) [fb/deg./GeV]");
    sprintf(nameRoot, "histograms/%s%s.root",obsName[0],obsName[1]);
    sprintf(namePdf, "histograms/%s%s.pdf",obsName[0],obsName[1]);

    double binW[2];
    for (int i = 0; i < 1; i++) {
	binW[i] = (binU[i]-binL[i])/binN[i];
    }
//--------------------------------------------------------------------------------------------------
    TFile *floop = new TFile("events_NLO.root");
    TTree *tloop = (TTree*)floop->Get("events_tot");
    tloop->SetBranchAddress("weight_tot",&weightloop);
    tloop->SetBranchAddress("px_tot",&pxloop);
    tloop->SetBranchAddress("py_tot",&pyloop);
    tloop->SetBranchAddress("pz_tot",&pzloop);
    tloop->SetBranchAddress("E_tot",&Eloop);
//--------------------------------------------------------------------------------------------------
    ifstream f_loop_xs;
    f_loop_xs.open("events_NLO.dat");
    f_loop_xs >> loop_xs;
    f_loop_xs.close();
    loop_xs *= 1.E3;
//--------------------------------------------------------------------------------------------------
//    int nentriesloop = tloop->GetEntries();
    int nentriesloop = 1000000;
//--------------------------------------------------------------------------------------------------
    TH2D *hLoop;
//--------------------------------------------------------------------------------------------------
    gStyle->SetHistLineColor(10000);
    gStyle->SetHistFillColor(10000);
    gStyle->SetMarkerColor(10000);
    gStyle->SetHistLineWidth(1);
    hLoop = new TH2D("","",binN[0],binL[0],binU[0],binN[1],binL[1],binU[1]);
    hLoop->Sumw2();
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < nentriesloop; i++) {
	tloop->GetEntry(i);
	hLoop->Fill(Observable(0, pxloop, pyloop, pzloop, Eloop),
	    Observable(1, pxloop, pyloop, pzloop, Eloop), weightloop[0]*loop_xs/nentriesloop/binW[0]/binW[0]);
    }
//--------------------------------------------------------------------------------------------------
    c.SetLogx(0);
    c.SetLogy(0);
    c.SetLogz(1);
//    hLoop->SetMinimum(0.);
    hLoop->GetXaxis()->SetTitle(xTitle);
    hLoop->GetYaxis()->SetTitle(yTitle);
    hLoop->GetZaxis()->SetTitle(zTitle);
    hLoop->DrawCopy("lego2");
    c.Update();
    c.SaveAs(nameRoot);
    c.Print(namePdf);
    delete hLoop;

    return 0;
}

void set_style()
{
    gStyle->SetCanvasColor(kWhite);
    gStyle->SetHistFillColor(kWhite);
    gStyle->SetHistLineColor(kBlack);
    gStyle->SetHistFillStyle(0);
    gStyle->SetFillStyle(4050);
    gStyle->SetMarkerColor(kBlack);
    gStyle->SetMarkerStyle(1);
    gStyle->SetHistLineStyle(1);
    gStyle->SetHistLineWidth(1);
    gStyle->SetPaperSize(20, 28);
    gStyle->SetPalette(1);
    gStyle->SetOptStat(0);
    gStyle->SetLabelFont(42,"X");
    gStyle->SetLabelSize(0.06,"X");
    gStyle->SetLabelOffset(0.006,"X");
    gStyle->SetTitleFont(42,"X");
    gStyle->SetTitleSize(0.06,"X");
    gStyle->SetTitleOffset(1.1,"X");
    gStyle->SetTitleColor(kBlack,"X");
    gStyle->SetLabelFont(42,"Y");
    gStyle->SetLabelSize(0.06,"Y");
    gStyle->SetLabelOffset(0.006,"Y");
    gStyle->SetTitleFont(42,"Y");
    gStyle->SetTitleSize(0.06,"Y");
    gStyle->SetTitleOffset(1.1,"Y");
    gStyle->SetTitleColor(kBlack,"Y");
    gStyle->SetTextFont(42);
}

double Observable(int iObs, double *px, double *py, double *pz, double *E) {
    double Obs;
    switch (iObs) {
    case 0: // Angle theta for 3rd particle
	Obs = 180/PI*acos(pz[0]/sqrt(px[0]*px[0]+py[0]*py[0]+pz[0]*pz[0]));
	break;
    case 1: // Energy for 5th particle (photon)
	Obs = E[2];
	break;
    default:
	Obs = 0.;
    }
    return Obs;
};
