#ifndef LHEF_H
#define LHEF_H
#include <fstream>

namespace SANC {

class Info;

struct LHEinit {
    int idbmup[2]; // PDG id's of beam particles.
    double ebmup[2]; // Energy of beam particles given in GeV.
    int pdfgup[2]; // The author group for the PDF used for the beams according to the PDFLib specification.
    int pdfsup[2]; // The id number the PDF used for the beams according to the PDFLib specification.
    int idwtup; // Master switch indicating how the ME generator envisages the events weights should be interpreted according to the Les Houches accord.
    int nprup; // The number of different subprocesses in this file.
    double xsecup; // The cross sections for the different subprocesses in pb.
    double xerrup; // The statistical error in the cross sections for the different subprocesses in pb.
    double xmaxup; // The maximum event weights (in HEPEUP::XWGTUP) for different subprocesses.
    int lprup; // The subprocess code for the different subprocesses.
};

struct LHEevent {
    int nup; // The number of particle entries in the current event.
    int idprup; // The subprocess code for this event (as given in LPRUP).
    double xwgtup; // The weight for this event.
    double scalup; // The scale in GeV used in the calculation of the PDF's in this event.
    double aqedup; // The value of the QED coupling used in this event.
    double aqcdup; // The value of the QCD coupling used in this event.
    int idup[10]; // The PDG id's for the particle entries in this event.
    int istup[10]; // The status codes for the particle entries in this event.
    int mothup[10][2]; // Indices for the first and last mother for the particle entries in this event.
    int icolup[10][2]; // The colour-line indices (first(second) is (anti)colour) for the particle entries in this event.
    double pup[10][5]; // Lab frame momentum (Px, Py, Pz, E and M in GeV) for the particle entries in this event.
    double vtimup[10]; // Invariant lifetime (c*tau, distance from production to decay in mm) for the particle entries in this event.
    double spinup[10]; // Spin info for the particle entries in this event given as the cosine of the angle between the spin vector of a particle and the 3-momentum of the decaying particle, specified in the lab frame.
};

class LHEF {
public:
    LHEF(Info & _info);
    ~LHEF(){}
    void LHEInit(){};
    void printInit(std::ostream & output);
    void printEvent(std::ostream & output);
    void printFinish(std::ostream & output);
    LHEinit heprup;
    LHEevent hepeup;
private:
    Info & info;
    char dateNow[12];
    char timeNow[9];
};

}

#endif // LHEF_H
