#ifndef PROCESS_H
#define PROCESS_H

#include "Event.h"
#include <string>
#include <vector>

namespace SANC {

class Info;
class PDF;

class Process {
public:
  Process(Info &_info, PDF *_pdf);
  void Born(double *Z, double *BornQ);
  void Virt(double *Z, double *VirtQ);
  void Hard(double *Z, double *HardQ);
  void Brdq(double *Z, double *BrdqQ);
  void Ibaa(double *Z, double *IbaaQ);
  void Ibaq(double *Z, double *IbaqQ);
  void Ibs1(double *Z, double *Ibs1Q);
  void Ibs2(double *Z, double *Ibs2Q);
  void AFB(Vec4 *P[6], double *BornXS);
  void VariablesBORN(int nDim, double *Xarg, double *Z, double *Jac);
  void VariablesHARD(int nDim, double *Xarg, double *Z, double *Jac);
  void VariablesBRDQ(int nDim, double *Xarg, double *Z, double *Jac);
  void VariablesIBAA(int nDim, double *Xarg, double *Z, double *Jac);
  void VariablesIBAQ(int nDim, double *Xarg, double *Z, double *Jac);
  void VariablesIBS1(int nDim, double *Xarg, double *Z, double *Jac);
  void VariablesIBS2(int nDim, double *Xarg, double *Z, double *Jac);
  bool CutsPrelimBORN(double *Z);
  bool CutsPrelimHARD(double *Z);
  bool CutsPrelimIBAA(double *Z);
  bool CutsPrelimIBAQ(double *Z);
  bool CutsPrelimIBS1(double *Z);
  bool CutsPrelimIBS2(double *Z);
  void GetFourMomentaBORN(double *Z, Vec4 *P[6]);
  void GetFourMomentaHARD(double *Z, Vec4 *P[6]);
  void GetFourMomentaBRDQ(double *Z, Vec4 *P[6]);
  void GetFourMomentaIBAA(double *Z, Vec4 *P[6]);
  void GetFourMomentaIBAQ(double *Z, Vec4 *P[6]);
  void GetFourMomentaIBS1(double *Z, Vec4 *P[6]);
  void GetFourMomentaIBS2(double *Z, Vec4 *P[6]);
  bool Cuts(Vec4 *P[6]);
  void PdfConvBORN(double *Z, double *QQ);
  void PdfConvHARD(double *Z, double *QQ);
  void PdfConvBRDQ(double *Z, double *QQ);
  void PdfConvIBAA(double *Z, double *QQ);
  void PdfConvIBAQ(double *Z, double *QQ);
  void PdfConvIBS1(double *Z, double *QQ);
  void PdfConvIBS2(double *Z, double *QQ);
  int idProc;
  int kDimBorn, kDimVirt, kDimHard, kDimLL11, kDimLL22, kDimLL12;
  double Mf1, Mf2, Mf3, Mf4;
  double Mf1_2, Mf2_2, Mf3_2, Mf4_2;
  double m34min, m34max;
  double dm, sm;
  double S, sqS, E1, E2;
  double beta1, beta2;
  double scale;
  int pdfID;
  int icolup[10][2];
  std::vector<int> idup;
  std::vector<int> iQ;
  std::vector<double> SubProcFactor;
  std::vector<double> eQ2i;
  std::vector<double> mQ2i;
  std::vector<double> eQ2f;
  std::vector<double> mQ2f;
  std::string procName, arXiv;
  Event event;
  int nup; // Number of particles
protected:
  void init();
  Info &info;
  PDF *pdf;
};

} // namespace SANC

#endif // PROCESS_H
