#include <fstream>
#include <iostream>
#include <cmath>
#include "Integrand.h"
#include "Info.h"
#include "LHEF.h"
#include "TMath.h"
#include "ModulesInterface.h"
#include "Process_pp.h"
#include "Vec4.h"
#include "PartonDidtributionsLHAPDF.h"
using namespace std;
using namespace SANC;
using namespace SANCmodules_pp;
using namespace SANCmodulesVar;

Integrand_pp::Integrand_pp(SANC::Info &_info, SANC::Process & _proc, PDF * _pdf, int type):info(_info),proc(_proc),pdf(_pdf){
    m_Type=type;
    Mf1 = proc.Mf1;
    Mf2 = proc.Mf2;
    Mf3 = proc.Mf3;
    Mf4 = proc.Mf4;
    Mf1_2 = proc.Mf1_2;
    Mf2_2 = proc.Mf2_2;
    Mf3_2 = proc.Mf3_2;
    Mf4_2 = proc.Mf4_2;
    sm = proc.sm;
    dm = proc.dm;
    P_com[0] = &proc.event.P_com(0);
    P_com[1] = &proc.event.P_com(1);
    P_com[2] = &proc.event.P_com(2);
    P_com[3] = &proc.event.P_com(3);
    P_com[4] = &proc.event.P_com(4);
    P_com[5] = &proc.event.P_com(5);
    P_lab[0] = &proc.event.P_lab(0);
    P_lab[1] = &proc.event.P_lab(1);
    P_lab[2] = &proc.event.P_lab(2);
    P_lab[3] = &proc.event.P_lab(3);
    P_lab[4] = &proc.event.P_lab(4);
    P_lab[5] = &proc.event.P_lab(5);
    *P_com[0] = Vec4();
    *P_lab[0] = Vec4();
}

double Integrand_pp::Density(int nDim, double *Xarg){
    double rho;
    rho = 1e-100;
    if (m_Type == 10) rho = BornH(nDim, Xarg);
    else if (m_Type == 11) rho = VirtH(nDim, Xarg);
    else if (m_Type == 12) rho = HardH(nDim, Xarg);
    else if (m_Type == 13) rho = BrdqH(nDim, Xarg);
    else if (m_Type == 14) rho = IbaaH(nDim, Xarg);
    else if (m_Type == 15) rho = IbaqH(nDim, Xarg);
    else if (m_Type == 16) rho = Ibs1H(nDim, Xarg);
    else if (m_Type == 17) rho = Ibs2H(nDim, Xarg);
    else{
	cout << " TFDISTR::Density: !!!!!! WRONG m_Type= " << m_Type << endl;
    }
    return rho;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::BornH(int nDim, double *Xarg) {
    double BornXS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double BornQ(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesBORN(nDim,Xarg,Z,&Jac);
    if (proc.CutsPrelimBORN(Z)) return BornXS;
    proc.GetFourMomentaBORN(Z,P_lab);
    if (proc.Cuts(P_lab)) return BornXS;
    proc.PdfConvBORN(Z,&QQ);
    proc.Born(Z,&BornQ);
    BornXS = 10.*BornQ*QQ*Jac;
    proc.AFB(P_lab,&BornXS);
    if (BornXS < 0) proc.event.weightProc = -1.;
    BornXS = abs(BornXS);
//--------------------------------------------------------------------------------------------------
    if (BornXS != BornXS) {
	BornXS = 0.;
	cout << "Born NAN!" << endl;
    }
//--------------------------------------------------------------------------------------------------
    return BornXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::VirtH(int nDim, double *Xarg) {
    double VirtXS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double VirtQ(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesBORN(nDim,Xarg,Z,&Jac);
    if (proc.CutsPrelimBORN(Z)) return VirtXS;
    proc.GetFourMomentaBORN(Z,P_lab);
    if (proc.Cuts(P_lab)) return VirtXS;
    proc.PdfConvBORN(Z,&QQ);
    proc.Virt(Z,&VirtQ);
    VirtXS = 10.*VirtQ*QQ*Jac;
    proc.AFB(P_lab,&VirtXS);
    if (VirtXS < 0) proc.event.weightProc = -1.;
    VirtXS = abs(VirtXS);
//--------------------------------------------------------------------------------------------------
    if (VirtXS != VirtXS) {
	VirtXS = 0.;
	cout << "Virt NAN!" << endl;
    }
//--------------------------------------------------------------------------------------------------
    return VirtXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::HardH(int nDim, double *Xarg) {
    double HardXS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double HardQ(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesHARD(nDim,Xarg,Z,&Jac);
    if (proc.CutsPrelimHARD(Z)) return HardXS;
    proc.GetFourMomentaHARD(Z,P_lab);
    if (proc.Cuts(P_lab)) return HardXS;
    proc.PdfConvHARD(Z,&QQ);
    proc.Hard(Z,&HardQ);
    HardXS = 20.*HardQ*QQ*Jac;
    proc.AFB(P_lab,&HardXS);
    if (HardXS < 0) proc.event.weightProc = -1.;
    HardXS = abs(HardXS);
//--------------------------------------------------------------------------------------------------
    if (HardXS != HardXS) {
	HardXS = 0.;
	cout << "Hard NAN!" << endl;
    }
//--------------------------------------------------------------------------------------------------
    return HardXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::BrdqH(int nDim, double *Xarg) {
    double BrdqXS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double BrdqQ(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesBRDQ(nDim,Xarg,Z,&Jac);
    if (proc.CutsPrelimBORN(Z)) return BrdqXS;
    proc.GetFourMomentaBRDQ(Z,P_lab);
    if (proc.Cuts(P_lab)) return BrdqXS;
    proc.PdfConvBRDQ(Z,&QQ);
    proc.Brdq(Z,&BrdqQ);
    BrdqXS = 10.*BrdqQ*QQ*Jac;
    proc.AFB(P_lab,&BrdqXS);
    if (BrdqXS < 0) proc.event.weightProc = -1.;
    BrdqXS = abs(BrdqXS);
//--------------------------------------------------------------------------------------------------
    if (BrdqXS != BrdqXS) {
	BrdqXS = 0.;
	cout << "Brdq NAN!" << endl;
    }
//--------------------------------------------------------------------------------------------------
    return BrdqXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::IbaaH(int nDim, double *Xarg) {
    double IbaaXS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double Ibaa(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesIBAA(nDim,Xarg,Z,&Jac);
    if (proc.CutsPrelimIBAA(Z)) return IbaaXS;
    proc.GetFourMomentaIBAA(Z,P_lab);
    if (proc.Cuts(P_lab)) return IbaaXS;
    proc.PdfConvIBAA(Z,&QQ);
    proc.Ibaa(Z,&Ibaa);
    IbaaXS = Ibaa*QQ*Jac;
    proc.AFB(P_lab,&IbaaXS);
    if (IbaaXS < 0) proc.event.weightProc = -1.;
    IbaaXS = abs(IbaaXS);
//--------------------------------------------------------------------------------------------------
    if (IbaaXS != IbaaXS) {
	IbaaXS = 0.;
	cout << "Ibaa NAN!" << endl;
    }
//--------------------------------------------------------------------------------------------------
    return IbaaXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::IbaqH(int nDim, double *Xarg) {
    double IbaqXS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double IbaqQ(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesIBAQ(nDim,Xarg,Z,&Jac);
    if (info.Iqcd == 1) {
	masses_.mdn = info.Mdn/10.;
	masses_.mup = info.Mup/10.;
	r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	r2masses_.rupm2 = masses_.mup*masses_.mup;
    }
    if (proc.CutsPrelimIBAQ(Z)) {
	if (info.Iqcd == 1) {
	    masses_.mdn = info.Mdn;
	    masses_.mup = info.Mup;
	    r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	    r2masses_.rupm2 = masses_.mup*masses_.mup;
	}
	return IbaqXS;
    }
    proc.GetFourMomentaIBAQ(Z,P_lab);
    if (proc.Cuts(P_lab)) {
	if (info.Iqcd == 1) {
	    masses_.mdn = info.Mdn;
	    masses_.mup = info.Mup;
	    r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	    r2masses_.rupm2 = masses_.mup*masses_.mup;
	}
	return IbaqXS;
    }
    proc.PdfConvIBAQ(Z,&QQ);
    proc.Ibaq(Z,&IbaqQ);
    IbaqXS = 40.*IbaqQ*QQ*Jac;
    proc.AFB(P_lab,&IbaqXS);
    if (IbaqXS < 0) proc.event.weightProc = -1.;
    IbaqXS = abs(IbaqXS);
//--------------------------------------------------------------------------------------------------
    if (IbaqXS != IbaqXS) {
	IbaqXS = 0.;
	cout << "Ibaq NAN!" << endl;
    }
    if (info.Iqcd == 1) {
	masses_.mdn = info.Mdn;
	masses_.mup = info.Mup;
	r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	r2masses_.rupm2 = masses_.mup*masses_.mup;
    }
//--------------------------------------------------------------------------------------------------
    return IbaqXS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::Ibs1H(int nDim, double *Xarg) {
    double Ibs1XS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double Ibs1Q(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesIBS1(nDim,Xarg,Z,&Jac);
    if (info.Iqcd == 1) {
	masses_.mdn = info.Mdn/10.;
	masses_.mup = info.Mup/10.;
	r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	r2masses_.rupm2 = masses_.mup*masses_.mup;
    }
    if (proc.CutsPrelimIBS1(Z)) {
	if (info.Iqcd == 1) {
	    masses_.mdn = info.Mdn;
	    masses_.mup = info.Mup;
	    r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	    r2masses_.rupm2 = masses_.mup*masses_.mup;
	}
	return Ibs1XS;
    }
    proc.GetFourMomentaIBS1(Z,P_lab);
    if (proc.Cuts(P_lab)) {
	if (info.Iqcd == 1) {
	    masses_.mdn = info.Mdn;
	    masses_.mup = info.Mup;
	    r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	    r2masses_.rupm2 = masses_.mup*masses_.mup;
	}
	return Ibs1XS;
    }
    proc.PdfConvIBS1(Z,&QQ);
    proc.Ibs1(Z,&Ibs1Q);
    Ibs1XS = 40.*Ibs1Q*QQ*Jac;
    proc.AFB(P_lab,&Ibs1XS);
    if (Ibs1XS < 0) proc.event.weightProc = -1.;
    Ibs1XS = abs(Ibs1XS);
//--------------------------------------------------------------------------------------------------
    if (Ibs1XS != Ibs1XS) {
	Ibs1XS = 0.;
	cout << "Ibs1 NAN!" << endl;
    }
    if (info.Iqcd == 1) {
	masses_.mdn = info.Mdn;
	masses_.mup = info.Mup;
	r2masses_.rdnm2 = masses_.mdn*masses_.mdn;
	r2masses_.rupm2 = masses_.mup*masses_.mup;
    }
//--------------------------------------------------------------------------------------------------
    return Ibs1XS;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
double Integrand_pp::Ibs2H(int nDim, double *Xarg) {
    double Ibs2XS(0.);
    double Z[nDim];
    double Jac(1.);
    double QQ(0.);
    double Ibs2Q(0.);
    proc.event.weightProc = 1.;
//--------------------------------------------------------------------------------------------------
    proc.VariablesIBS2(nDim,Xarg,Z,&Jac);
    if (proc.CutsPrelimIBS2(Z)) return Ibs2XS;
    proc.GetFourMomentaIBS2(Z,P_lab);
    if (proc.Cuts(P_lab)) return Ibs2XS;
    proc.PdfConvIBS2(Z,&QQ);
    proc.Ibs2(Z,&Ibs2Q);
    Ibs2XS = 20.*Ibs2Q*QQ*Jac;
    proc.AFB(P_lab,&Ibs2XS);
    if (Ibs2XS < 0) proc.event.weightProc = -1.;
    Ibs2XS = abs(Ibs2XS);
//--------------------------------------------------------------------------------------------------
    if (Ibs2XS != Ibs2XS) {
	Ibs2XS = 0.;
	cout << "Ibs2 NAN!" << endl;
    }
//--------------------------------------------------------------------------------------------------
    return Ibs2XS;
}
