#include "Process_pp.h"
#include "General.h"
#include "Info.h"
#include "ModulesInterface.h"
#include "PartonDidtributionsLHAPDF.h"
#include "TMath.h"
using namespace SANC;
using namespace SANCmodulesVar;
using namespace SANCmodules_pp;

Process::Process(Info & _info, PDF * _pdf):info(_info),pdf(_pdf)
{
    pdfID = pdf->pdfID();
    idup.resize(6);
    idup[0] = 0;
    if (info.Ibeams==1){
        idup[1] = 2212;
        idup[2] = 2212;
    }
    if (info.Ibeams==2){
        idup[1] = 2212;
        idup[2] = -2212;
    }
    switch (info.Pid) {
    case 201:
	procName = "hh -> e+e-X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mel;
	Mf4 = info.Mel;
	idup[3] = 11;
	idup[4] = -11;
	iQ.assign({1,2,3,4,5,-5,-4,-3,-2,-1});
	SubProcFactor.assign({1.,1.,1.,1.,1.,1.,1.,1.,1.,1.});
	eQ2i.assign({1./9.,4./9,1./9.,4./9.,1./9.,1./9.,4./9.,1./9.,4./9,1./9.});
	mQ2i.assign({info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,
		     info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn});
	eQ2f.assign({1./9.,4./9,1./9.,4./9.,1./9.,1./9.,4./9.,1./9.,4./9,1./9.});
	mQ2f.assign({info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,
		     info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn});
	break;
    case 202:
	procName = "hh -> mu+mu-X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mmo;
	Mf4 = info.Mmo;
	idup[3] = 13;
	idup[4] = -13;
	iQ.assign({1,2,3,4,5,-5,-4,-3,-2,-1});
	SubProcFactor.assign({1.,1.,1.,1.,1.,1.,1.,1.,1.,1.});
	eQ2i.assign({1./9.,4./9,1./9.,4./9.,1./9.,1./9.,4./9.,1./9.,4./9,1./9.});
	mQ2i.assign({info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,
		     info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn});
	eQ2f.assign({1./9.,4./9,1./9.,4./9.,1./9.,1./9.,4./9.,1./9.,4./9,1./9.});
	mQ2f.assign({info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,
		     info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn});
	break;
    case 203:
	procName = "hh -> tau+tau-X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mta;
	Mf4 = info.Mta;
	idup[3] = 15;
	idup[4] = -15;
	iQ.assign({1,2,3,4,5,-5,-4,-3,-2,-1});
	SubProcFactor.assign({1.,1.,1.,1.,1.,1.,1.,1.,1.,1.});
	eQ2i.assign({1./9.,4./9,1./9.,4./9.,1./9.,1./9.,4./9.,1./9.,4./9,1./9.});
	mQ2i.assign({info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,
		     info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn});
	eQ2f.assign({1./9.,4./9,1./9.,4./9.,1./9.,1./9.,4./9.,1./9.,4./9,1./9.});
	mQ2f.assign({info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,
		     info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn,info.Mup*info.Mup,info.Mdn*info.Mdn});
	break;
    case 211:
	procName = "hh -> e- antinue X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mel;
	Mf4 = info.Men;
	idup[3] = 11;
	idup[4] = -12;
	iQ.assign({0,1,3,1,3,-4,-4,-2,-2,0});
	SubProcFactor.assign({0,info.Vud,info.Vus,info.Vcd,info.Vcs,info.Vcs,info.Vcd,info.Vus,info.Vud,0});
	eQ2i.assign({0.,1./9,1./9.,1./9.,1./9.,4./9.,4./9.,4./9.,4./9,0.});
	mQ2i.assign({0.,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,
		    info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,0.});
	eQ2f.assign({0.,4./9,4./9.,4./9.,4./9.,1./9.,1./9.,1./9.,1./9,0.});
	mQ2f.assign({0.,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,
		    info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,0.});
	break;
    case 212:
	procName = "hh -> mu- antinumu X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mmo;
	Mf4 = info.Mmn;
	idup[3] = 13;
	idup[4] = -14;
	iQ.assign({0,1,3,1,3,-4,-4,-2,-2,0});
	SubProcFactor.assign({0,info.Vud,info.Vus,info.Vcd,info.Vcs,info.Vcs,info.Vcd,info.Vus,info.Vud,0});
	eQ2i.assign({0.,1./9,1./9.,1./9.,1./9.,4./9.,4./9.,4./9.,4./9,0.});
	mQ2i.assign({0.,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,
		    info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,0.});
	eQ2f.assign({0.,4./9,4./9.,4./9.,4./9.,1./9.,1./9.,1./9.,1./9,0.});
	mQ2f.assign({0.,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,
		    info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,0.});
	break;
    case 213:
	procName = "hh -> tau- antinutau X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mta;
	Mf4 = info.Mtn;
	idup[3] = 15;
	idup[4] = -16;
	iQ.assign({0,1,3,1,3,-4,-4,-2,-2,0});
	SubProcFactor.assign({0,info.Vud,info.Vus,info.Vcd,info.Vcs,info.Vcs,info.Vcd,info.Vus,info.Vud,0});
	eQ2i.assign({0.,1./9,1./9.,1./9.,1./9.,4./9.,4./9.,4./9.,4./9,0.});
	mQ2i.assign({0.,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,
		    info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,0.});
	eQ2f.assign({0.,4./9,4./9.,4./9.,4./9.,1./9.,1./9.,1./9.,1./9,0.});
	mQ2f.assign({0.,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,
		    info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,0.});
	break;
    case 221:
	procName = "hh -> nue e+ X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Men;
	Mf4 = info.Mel;
	idup[3] = 12;
	idup[4] = -11;
	iQ.assign({0,2,2,4,4,-3,-1,-3,-1,0});
	SubProcFactor.assign({0,info.Vud,info.Vus,info.Vcd,info.Vcs,info.Vcs,info.Vcd,info.Vus,info.Vud,0});
	eQ2i.assign({0.,4./9,4./9.,4./9.,4./9.,1./9.,1./9.,1./9.,1./9,0.});
	mQ2i.assign({0.,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,
		    info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,0.});
	eQ2f.assign({0.,1./9,1./9.,1./9.,1./9.,4./9.,4./9.,4./9.,4./9,0.});
	mQ2f.assign({0.,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,
		    info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,0.});
	break;
    case 222:
	procName = "hh -> numu mu+ X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mmn;
	Mf4 = info.Mmo;
	idup[3] = 14;
	idup[4] = -13;
	iQ.assign({0,2,2,4,4,-3,-1,-3,-1,0});
	SubProcFactor.assign({0,info.Vud,info.Vus,info.Vcd,info.Vcs,info.Vcs,info.Vcd,info.Vus,info.Vud,0});
	eQ2i.assign({0.,4./9,4./9.,4./9.,4./9.,1./9.,1./9.,1./9.,1./9,0.});
	mQ2i.assign({0.,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,
		    info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,0.});
	eQ2f.assign({0.,1./9,1./9.,1./9.,1./9.,4./9.,4./9.,4./9.,4./9,0.});
	mQ2f.assign({0.,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,
		    info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,0.});
	break;
    case 223:
	procName = "hh -> nutau tau+ X";
	Mf1 = 0.;
	Mf2 = 0.;
	Mf3 = info.Mtn;
	Mf4 = info.Mta;
	idup[3] = 16;
	idup[4] = -15;
	iQ.assign({0,2,2,4,4,-3,-1,-3,-1,0});
	SubProcFactor.assign({0,info.Vud,info.Vus,info.Vcd,info.Vcs,info.Vcs,info.Vcd,info.Vus,info.Vud,0});
	eQ2i.assign({0.,4./9,4./9.,4./9.,4./9.,1./9.,1./9.,1./9.,1./9,0.});
	mQ2i.assign({0.,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,
		    info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,0.});
	eQ2f.assign({0.,1./9,1./9.,1./9.,1./9.,4./9.,4./9.,4./9.,4./9,0.});
	mQ2f.assign({0.,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,info.Mdn*info.Mdn,
		    info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,info.Mup*info.Mup,0.});
	break;
    default:
	std::cerr<<"Error: Unknown process "<<info.Pid<<std::endl;
	exit(1);
    }
    sqS = info.Ecm;
    E1 = sqS/2.;
    E2 = sqS/2.;    
    S = pow2(info.Ecm);
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    dm = (Mf1-Mf2)*(Mf1+Mf2)*(Mf4-Mf3)*(Mf4+Mf3);
    sm = Mf1_2+Mf2_2+Mf3_2+Mf4_2;
    if (info.m34cut) {
      m34min = std::max(info.m34min,Mf3+Mf4);
      m34max = std::min(info.m34max,info.Ecm);
    }
    else {
      m34min = Mf3+Mf4;
      m34max = info.Ecm;
    }
	event.particles.clear();
	for (auto const & id:  idup) {
		event.particles.push_back(Particle(id));
	}
	event.particles.push_back(Particle(22));
}

void Process::VariablesBORN(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double Factor(1.);
    double S;
//--------------------------------------------------------------------------------------------------
    Vl[0] = -1.;
    Vu[0] =  1.;
    if (info.Pid <= 203) {
	Vl[1] = atan((pow2(m34min)-info.MZ*info.MZ)/info.MZ/info.Wz);
	Vu[1] = atan((pow2(m34max)-info.MZ*info.MZ)/info.MZ/info.Wz);
    } else {
	Vl[1] = atan((pow2(m34min)-info.MW*info.MW)/info.MW/info.Ww);
	Vu[1] = atan((pow2(m34max)-info.MW*info.MW)/info.MW/info.Ww);
    }
    Vl[2] = -10.;
    Vu[2] = 10.;
    Vl[3] = 0.;
    Vu[3] = 2.*consts_.pi;
    Vl[4] = 0.;
    Vu[4] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }
//--------------------------------------------------------------------------------------------------
    if (info.Pid <= 203) {
	S = info.MZ*info.MZ + info.MZ*info.Wz*tan(V[1]);
	*Jac = 1./info.Ecm/info.Ecm/2./consts_.pi*((S-info.MZ*info.MZ)*(S-info.MZ*info.MZ)+info.MZ*info.MZ*info.Wz*info.Wz)/info.MZ/info.Wz;
    } else {
	S = info.MW*info.MW + info.MW*info.Ww*tan(V[1]);
	*Jac = 1./info.Ecm/info.Ecm/2./consts_.pi*((S-info.MW*info.MW)*(S-info.MW*info.MW)+info.MW*info.MW*info.Ww*info.Ww)/info.MW/info.Ww;
    }
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = V[0]; // cos(theta_3)
    Z[1] = S;    // S
    Z[2] = V[2]; // y_12
    Z[3] = V[3]; // phi_3
    Z[4] = V[4]; // subprocess
}

void Process::VariablesBRDQ(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double Factor(1.);
    double S;
//--------------------------------------------------------------------------------------------------
    Vl[0] = -1.;
    Vu[0] =  1.;
    if (info.Pid <= 203) {
	Vl[1] = atan((pow2(m34min)-info.MZ*info.MZ)/info.MZ/info.Wz);
	Vu[1] = atan((pow2(m34max)-info.MZ*info.MZ)/info.MZ/info.Wz);
    } else {
	Vl[1] = atan((pow2(m34min)-info.MW*info.MW)/info.MW/info.Ww);
	Vu[1] = atan((pow2(m34max)-info.MW*info.MW)/info.MW/info.Ww);
    }
    Vl[2] = -10.;
    Vu[2] = 10.;
    Vl[3] = 0.;
    Vu[3] = 2.*consts_.pi;
    Vl[4] = 0.;
    Vu[4] = 1.;
    Vl[5] = 0.;
    Vu[5] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }
//--------------------------------------------------------------------------------------------------
    if (info.Pid <= 203) {
	S = info.MZ*info.MZ + info.MZ*info.Wz*tan(V[1]);
	*Jac = 1./info.Ecm/info.Ecm/2./consts_.pi*((S-info.MZ*info.MZ)*(S-info.MZ*info.MZ)+info.MZ*info.MZ*info.Wz*info.Wz)/info.MZ/info.Wz;
    } else {
	S = info.MW*info.MW + info.MW*info.Ww*tan(V[1]);
	*Jac = 1./info.Ecm/info.Ecm/2./consts_.pi*((S-info.MW*info.MW)*(S-info.MW*info.MW)+info.MW*info.MW*info.Ww*info.Ww)/info.MW/info.Ww;
    }
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = V[0]; // cos(theta_3)
    Z[1] = S;    // S
    Z[2] = V[2]; // y_12
    Z[3] = V[3]; // phi_3
    Z[4] = V[4]; // z
    Z[5] = V[5]; // subprocess
}

void Process::VariablesHARD(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double Factor(1.);
    double S,Spr;
//--------------------------------------------------------------------------------------------------
    Vl[0] = 0.;
    Vu[0] = log(1./info.Ome*(1.-(Mf3+Mf4)*(Mf3+Mf4)/(info.Ecm*info.Ecm)));
    Vl[1] = 0.;
    Vu[1] = 2.*consts_.pi;
    Vl[2] = -1.;
    Vu[2] = 1.;
    Vl[3] = -1.;
    Vu[3] = 1.;
    if (info.Pid <= 203) {
	Vl[4] = atan((pow2(m34min)-info.MZ*info.MZ)/info.MZ/info.Wz);
	Vu[4] = atan((info.Ecm*info.Ecm-info.MZ*info.MZ)/info.MZ/info.Wz);
    } else {
	Vl[4] = atan((pow2(m34min)-info.MW*info.MW)/info.MW/info.Ww);
	Vu[4] = atan((info.Ecm*info.Ecm-info.MW*info.MW)/info.MW/info.Ww);
    }
    Vl[5] = -50.;
    Vu[5] = 50.;
    Vl[6] = 0.;
    Vu[6] = 2.*consts_.pi;
    Vl[7] = 0.;
    Vu[7] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }

//--------------------------------------------------------------------------------------------------
    if (info.Pid <= 203) {
	S = info.MZ*info.MZ + info.MZ*info.Wz*tan(V[4]);
	Spr= S*(1.-info.Ome*exp(V[0]));
	*Jac = 1./info.Ecm/info.Ecm/2./consts_.pi*((S-info.MZ*info.MZ)*(S-info.MZ*info.MZ)+info.MZ*info.MZ*info.Wz*info.Wz)/info.MZ/info.Wz*(S-Spr);
    } else {
	S = info.MW*info.MW + info.MW*info.Ww*tan(V[4]);
	Spr= S*(1.-info.Ome*exp(V[0]));
	*Jac = 1./info.Ecm/info.Ecm/2./consts_.pi*((S-info.MW*info.MW)*(S-info.MW*info.MW)+info.MW*info.MW*info.Ww*info.Ww)/info.MW/info.Ww*(S-Spr);
    }
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = Spr;  // S'
    Z[1] = V[1]; // phi_4
    Z[2] = V[2]; // cos(theta_4)
    Z[3] = V[3]; // cos(theta_5)
    Z[4] = S;    // S
    Z[5] = V[5]; // y_12
    Z[6] = V[6]; // phi_5
    Z[7] = V[7]; // subprocess
}

void Process::VariablesIBAA(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double Factor(1.);
    double sqS;
//--------------------------------------------------------------------------------------------------
    Vl[0] = -1.;
    Vu[0] = 1.;
    Vl[1] = m34min;
    Vu[1] = m34max;
    Vl[2] = -10.;
    Vu[2] = 10.;
    Vl[3] = 0.;
    Vu[3] = 2.*consts_.pi;
    Vl[4] = 0.;
    Vu[4] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }
    sqS = V[1];
    *Jac = 2.*sqS/info.Ecm/info.Ecm/2./consts_.pi;
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = V[0]; // cos(theta_3)
    Z[1] = V[1]; // sqS
    Z[2] = V[2]; // y_12
    Z[3] = V[3]; // phi_3
    Z[4] = V[4]; // subprocess
}

void Process::VariablesIBAQ(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double Factor(1.);
    double S,Spr,Cth5,Sth5;
//--------------------------------------------------------------------------------------------------
    if (info.Pid <= 203) {
	Vl[0] = atan((pow2(m34min)-info.MZ*info.MZ)/info.MZ/info.Wz);
	Vu[0] = atan((info.Ecm*info.Ecm-info.MZ*info.MZ)/info.MZ/info.Wz);
    } else {
	Vl[0] = atan((pow2(m34min)-info.MW*info.MW)/info.MW/info.Ww);
	Vu[0] = atan((info.Ecm*info.Ecm-info.MW*info.MW)/info.MW/info.Ww);
    }
    Vl[1] = 0.;
    Vu[1] = 2.*consts_.pi;
    Vl[2] = -1.;
    Vu[2] = 1.;
    Vl[3] = -50.;
    Vu[3] = 50.;
    if (info.Pid <= 203) {
	Vl[4] = -1./(info.MW*info.MZ + info.MZ*info.Wz*tan(Vl[0]+(Vu[0]-Vl[0])*Xarg[0]));
	Vu[4] = -1./info.Ecm/info.Ecm;
    } else {
	Vl[4] = -1./(info.MW*info.MW + info.MW*info.Ww*tan(Vl[0]+(Vu[0]-Vl[0])*Xarg[0]));
	Vu[4] = -1./info.Ecm/info.Ecm;
    }
    Vl[5] = -50.0;
    Vu[5] = 50.0;
    Vl[6] = 0.;
    Vu[6] = 2.*consts_.pi;
    Vl[7] = 0.;
    Vu[7] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }
    if (info.Pid <= 203) {
	Spr = info.MZ*info.MZ + info.MZ*info.Wz*tan(V[0]);
    } else {
	Spr = info.MW*info.MW + info.MW*info.Ww*tan(V[0]);
    }
    S = -1./V[4];
    Cth5 = (exp(2.*V[3])-1.)/(exp(2.*V[3])+1.);
    Sth5 = sqrt(1.-Cth5*Cth5);
    if (info.Pid <= 203) {
	*Jac = ((Spr-info.MZ*info.MZ)*(Spr-info.MZ*info.MZ)+info.MZ*info.MZ*info.Wz*info.Wz)*S*S/(info.Ecm)/(info.Ecm)/2./consts_.pi/info.MZ/info.Wz*Sth5*Sth5;
    } else {
	*Jac = ((Spr-info.MW*info.MW)*(Spr-info.MW*info.MW)+info.MW*info.MW*info.Ww*info.Ww)*S*S/(info.Ecm)/(info.Ecm)/2./consts_.pi/info.MW/info.Ww*Sth5*Sth5;
    }
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = Spr;  // Spr
    Z[1] = V[1]; // phi_4
    Z[2] = V[2]; // cos(theta_4)
    Z[3] = Cth5; // cos(theta_5)
    Z[4] = S;    // S
    Z[5] = V[5]; // y_12
    Z[6] = V[6]; // phi_5
    Z[7] = V[7]; // subprocess
}

void Process::VariablesIBS1(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double Factor(1.);
    double sqS,x3;
//--------------------------------------------------------------------------------------------------
    Vl[0] = -1.;
    Vu[0] = 1.;
    Vl[1] = m34min;
    Vu[1] = info.Ecm;
    Vl[2] = -10.;
    Vu[2] = 10.;
    Vl[3] = 0.;
    Vu[3] = 1.;
    Vl[4] = 0.;
    Vu[4] = 2.*consts_.pi;
    Vl[5] = 0.;
    Vu[5] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }
    sqS = V[1];
    x3 = V[3];
    *Jac = 2.*sqS/info.Ecm/info.Ecm/x3/2./consts_.pi;
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = V[0]; // cos(theta_3)
    Z[1] = sqS;  // sqS
    Z[2] = V[2]; // y_12
    Z[3] = V[3]; // x_3
    Z[4] = V[4]; // phi_3
    Z[5] = V[5]; // subprocess
}

void Process::VariablesIBS2(int nDim, double *Xarg, double *Z, double *Jac) {
    double Vl[nDim],Vu[nDim],V[nDim];
    double Factor(1.);
    double sqS,x3;
//--------------------------------------------------------------------------------------------------
    Vl[0] = -1.;
    Vu[0] = 1.;
    Vl[1] = m34min;
    Vu[1] = info.Ecm;
    Vl[2] = -10.;
    Vu[2] = 10.;
    Vl[3] = 0.;
    Vu[3] = 1.;
    Vl[4] = 0.;
    Vu[4] = 2.*consts_.pi;
    Vl[5] = 0.;
    Vu[5] = 1.;
    for (int i = 0; i < nDim; i++) {
	V[i] = Vl[i]+(Vu[i]-Vl[i])*Xarg[i];
	Factor *= Vu[i]-Vl[i];
    }
    sqS = V[1];
    x3 = V[3];
    *Jac = 2.*sqS/info.Ecm/info.Ecm/x3/2./consts_.pi;
    *Jac *= Factor;
//--------------------------------------------------------------------------------------------------
    Z[0] = V[0]; // cos(theta_3)
    Z[1] = sqS;  // sqS
    Z[2] = V[2]; // y_12
    Z[3] = V[3]; // x_3
    Z[4] = V[4]; // phi_3
    Z[5] = V[5]; // subprocess
}

bool Process::CutsPrelimBORN(double *Z) {
    bool cuts = false;
    double sqS,y,x1,x2;
    sqS = sqrt(Z[1]);
    y = Z[2];
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (sqS <= (Mf3+Mf4)) cuts = true;
    if (sqS <= sqrt(pdf->Q2min())) cuts = true;
    if (x1 <= info.xmin || x1 >= info.xmax) cuts = true;
    if (x2 <= info.xmin || x2 >= info.xmax) cuts = true;
    return cuts;
}

bool Process::CutsPrelimHARD(double *Z) {
    bool cuts = false;
    double S,Spr,y,sqS,x1,x2;
    Spr = Z[0];
    S = Z[4];
    y = Z[5];
    sqS = sqrt(S);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (Spr <= (Mf3+Mf4)*(Mf3+Mf4)) cuts = true;
    if (Spr >= S*(1.-info.Ome)) cuts = true;
    if (info.Iscale == 1 && S <= pdf->Q2min()) cuts = true;
    if (info.Iscale == 3 && Spr <= pdf->Q2min()) cuts = true;
    if (x1 <= info.xmin || x1 >= info.xmax) cuts = true;
    if (x2 <= info.xmin || x2 >= info.xmax) cuts = true;
    return cuts;
}

bool Process::CutsPrelimIBAA(double *Z) {
    bool cuts = false;
    double sqS,y,x1,x2;
    sqS = Z[1];
    y = Z[2];
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (sqS <= (Mf3+Mf4)) cuts = true;
    if (sqS <= sqrt(pdf->Q2min())) cuts = true;
    if (x1 <= info.xmin || x1 >= info.xmax) cuts = true;
    if (x2 <= info.xmin || x2 >= info.xmax) cuts = true;
    return cuts;
}

bool Process::CutsPrelimIBAQ(double *Z) {
    bool cuts = false;
    double S,Spr,y,x1,x2,sqS,sqSpr;
    Spr = Z[0];
    S = Z[4];
    y = Z[5];
    sqSpr = sqrt(Spr);
    sqS = sqrt(S);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (S <= (Mf3+Mf4)*(Mf3+Mf4)) cuts = true;
    if (Spr <= (Mf3+Mf4)*(Mf3+Mf4)) cuts = true;
    if (info.Iscale == 1 && S <= pdf->Q2min()) cuts = true;
    if (info.Iscale == 3 && Spr <= pdf->Q2min()) cuts = true;
    if (x1 <= info.xmin || x1 >= info.xmax) cuts = true;
    if (x2 <= info.xmin || x2 >= info.xmax) cuts = true;
    if (sqSpr >= sqS-masses_.mup) cuts = true;
    if (sqSpr >= sqS-masses_.mdn) cuts = true;
    return cuts;
}

bool Process::CutsPrelimIBS1(double *Z) {
    bool cuts = false;
    int qDir(1);
    double sqS,y,x1,x2,x3;
    sqS = Z[1];
    y = Z[2];
    x3 = Z[3];
    if (Z[5] < 0.5) qDir = 0;
    if (qDir == 0) {
	x1 = sqS/info.Ecm*exp(y)/x3;
	x2 = sqS/info.Ecm*exp(-y);
    } else {
	x1 = sqS/info.Ecm*exp(y);
	x2 = sqS/info.Ecm*exp(-y)/x3;
    }
    if (sqS <= (Mf3+Mf4)) cuts = true;
    if (sqS <= sqrt(pdf->Q2min())) cuts = true;
    if (x1 <= info.xmin || x1 >= info.xmax) cuts = true;
    if (x2 <= info.xmin || x2 >= info.xmax) cuts = true;
    return cuts;
}

bool Process::CutsPrelimIBS2(double *Z) {
    bool cuts = false;
    double sqS;
    sqS = Z[1];
    if (sqS <= (Mf3+Mf4)) cuts = true;
    if (sqS <= sqrt(pdf->Q2min())) cuts = true;
    return cuts;
}

void Process::GetFourMomentaBORN(double *Z, Vec4 *P[6]) {
    int qDir(1);
    double Cth,sqS,y,Phi;
    double Sth,Cph,Sph;
    double beta,x1,x2;
    double kappa3,kappa4,beta3,beta4;
    double S;
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    S = Z[1];
    sqS = sqrt(S);
    y = Z[2];
    Phi = Z[3];
    if (Z[4] < 0.5) qDir = 0;
//--------------------------------------------------------------------------------------------------
    Sth = sqrt(1.-Cth*Cth);
    Cph = cos(Phi);
    Sph = sin(Phi);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    beta  = (x1-x2)/(x1+x2);
//--------------------------------------------------------------------------------------------------
    if (qDir == 0) {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[5] = Vec4();
    } else {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[5] = Vec4();
    }
}

void Process::GetFourMomentaHARD(double *Z, Vec4 *P[6]) {
    int qDir(1);
    double Beta34;
    double kappa3,kappa4,beta3,beta4;
    double beta,x1,x2,y,sqS;
    double Sqspr;
    double S,Spr,Phi4,Phi5;
    double Cth4,Sth4,Cph4,Sph4,Cth5,Sth5,Cph5,Sph5;
//--------------------------------------------------------------------------------------------------
    Spr = Z[0];
    Phi4 = Z[1];
    Cth4 = Z[2];
    Cth5 = Z[3];
    S = Z[4];
    y = Z[5];
    Phi5 = Z[6];
    if (Z[7] < 0.5) qDir = 0;
//--------------------------------------------------------------------------------------------------
    Sth4 = sqrt(1.-Cth4*Cth4);
    Cph4 = cos(Phi4);
    Sph4 = sin(Phi4);
    Sth5 = sqrt(1.-Cth5*Cth5);
    Cph5 = cos(Phi5);
    Sph5 = sin(Phi5);
    sqS = sqrt(S);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    Sqspr = sqrt(Spr);
    kappa3 = 1.+(Mf3_2-Mf4_2)/Spr;
    kappa4 = 1.+(Mf4_2-Mf3_2)/Spr;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/Spr)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/Spr)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/Spr)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/Spr)/kappa4;
    beta  = (x1-x2)/(x1+x2);
    Beta34 = (S-Spr)/(S+Spr);
//--------------------------------------------------------------------------------------------------
    if (qDir == 0) {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).bstZ(beta);
	*P[3] = Vec4(0., 0., -Sqspr/2.*beta3*kappa3, Sqspr/2.*kappa3).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).bstZ(beta);
	*P[4] = Vec4(0., 0., Sqspr/2.*beta4*kappa4, Sqspr/2.*kappa4).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).bstZ(beta);
	*P[5] = Vec4(0., 0., (S-Spr)/2./sqS, (S-Spr)/2./sqS).rot(Sth5,Cth5,Cph5,Sph5).bstZ(beta);
    } else {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[3] = Vec4(0., 0., -Sqspr/2.*beta3*kappa3, Sqspr/2.*kappa3).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[4] = Vec4(0., 0., Sqspr/2.*beta4*kappa4, Sqspr/2.*kappa4).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[5] = Vec4(0., 0., (S-Spr)/2./sqS, (S-Spr)/2./sqS).rot(Sth5,Cth5,Cph5,Sph5).rot(0.,-1.,0.,-1.).bstZ(beta);
    }
}

void Process::GetFourMomentaBRDQ(double *Z, Vec4 *P[6]) {
    int qDir(1);
    double Cth,sqS,y,Phi;
    double Sth,Cph,Sph;
    double beta,x1,x2;
    double kappa3,kappa4,beta3,beta4;
    double S;
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    S = Z[1];
    sqS = sqrt(S);
    y = Z[2];
    Phi = Z[3];
    if (Z[5] < 0.5) qDir = 0;
//--------------------------------------------------------------------------------------------------
    Sth = sqrt(1.-Cth*Cth);
    Cph = cos(Phi);
    Sph = sin(Phi);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    beta  = (x1-x2)/(x1+x2);
//--------------------------------------------------------------------------------------------------
    if (qDir == 0) {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[5] = Vec4();
    } else {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[5] = Vec4();
    }
}

void Process::GetFourMomentaIBAA(double *Z, Vec4 *P[6]) {
    int qDir(1);
    double Cth,sqS,y,Phi;
    double Sth,Cph,Sph;
    double beta,x1,x2;
    double kappa3,kappa4,beta3,beta4;
    double S;
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    sqS = Z[1];
    y = Z[2];
    Phi = Z[3];
    if (Z[4] < 0.5) qDir = 0;
//--------------------------------------------------------------------------------------------------
    S = sqS*sqS;
    Sth = sqrt(1.-Cth*Cth);
    Cph = cos(Phi);
    Sph = sin(Phi);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    beta  = (x1-x2)/(x1+x2);
//--------------------------------------------------------------------------------------------------
    if (qDir == 0) {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[5] = Vec4();
    } else {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[5] = Vec4();
    }
}

void Process::GetFourMomentaIBAQ(double *Z, Vec4 *P[6]) {
    int qDir(1);
    double Beta34;
    double kappa3,kappa4,beta3,beta4;
    double beta,x1,x2,y;
    double sqS,sqSpr;
    double S,Spr,Phi4,Phi5;
    double Cth4,Sth4,Cph4,Sph4,Cth5,Sth5,Cph5,Sph5;
    Vec4 Q[6];
//--------------------------------------------------------------------------------------------------
    Spr = Z[0];
    Phi4 = Z[1];
    Cth4 = Z[2];
    Cth5 = Z[3];
    S = Z[4];
    y = Z[5];
    Phi5 = Z[6];
    if (Z[7] < 0.5) qDir = 0;
//--------------------------------------------------------------------------------------------------
    sqSpr = sqrt(Spr);
    sqS = sqrt(S);
    Sth4 = sqrt(1.-Cth4*Cth4);
    Cph4 = cos(Phi4);
    Sph4 = sin(Phi4);
    Sth5 = sqrt(1.-Cth5*Cth5);
    Cph5 = cos(Phi5);
    Sph5 = sin(Phi5);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    kappa3 = 1.+(Mf3_2-Mf4_2)/Spr;
    kappa4 = 1.+(Mf4_2-Mf3_2)/Spr;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/Spr)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/Spr)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/Spr)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/Spr)/kappa4;
    beta  = (x1-x2)/(x1+x2);
    Beta34 = (S-Spr)/(S+Spr);
//--------------------------------------------------------------------------------------------------
    if (qDir == 0) {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).bstZ(beta);
	*P[3] = Vec4(0., 0., -sqSpr/2.*beta3*kappa3, sqSpr/2.*kappa3).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).bstZ(beta);
	*P[4] = Vec4(0., 0., sqSpr/2.*beta4*kappa4, sqSpr/2.*kappa4).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).bstZ(beta);
	*P[5] = Vec4(0., 0., (S-Spr)/2./sqS, (S-Spr)/2./sqS).rot(Sth5,Cth5,Cph5,Sph5).bstZ(beta);
    } else {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[3] = Vec4(0., 0., -sqSpr/2.*beta3*kappa3, sqSpr/2.*kappa3).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[4] = Vec4(0., 0., sqSpr/2.*beta4*kappa4, sqSpr/2.*kappa4).rot(Sth4,Cth4,-Cph4,Sph4).bstZ(-Beta34).rot(Sth5,Cth5,Cph5,Sph5).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[5] = Vec4(0., 0., (S-Spr)/2./sqS, (S-Spr)/2./sqS).rot(Sth5,Cth5,Cph5,Sph5).rot(0.,-1.,0.,-1.).bstZ(beta);
    }
    Q[1] = *P[1];
    Q[2] = *P[2];
    Q[3] = *P[4];
    Q[4] = *P[3];
    Q[5] = *P[5];
    if (info.Pid < 221) {
	*P[3] = Q[3];
	*P[4] = Q[4];
    }
}

void Process::GetFourMomentaIBS1(double *Z, Vec4 *P[6]) {
    int qDir(1);
    double Cth,sqS,y,Phi;
    double Sth,Cph,Sph;
    double beta,x1,x2,x3;
    double kappa3,kappa4,beta3,beta4;
    double S;
    Vec4 Q[6];
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    sqS = Z[1];
    y = Z[2];
    x3 = Z[3];
    Phi = Z[4];
    if (Z[5] < 0.5) qDir = 0;
//--------------------------------------------------------------------------------------------------
    Sth = sqrt(1.-Cth*Cth);
    Cph = cos(Phi);
    Sph = sin(Phi);
    S = sqS*sqS;
    if (qDir == 0) {
	x1 = sqS/info.Ecm*exp(y)/x3;
	x2 = sqS/info.Ecm*exp(-y);
    } else {
	x1 = sqS/info.Ecm*exp(y);
	x2 = sqS/info.Ecm*exp(-y)/x3;
    }
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    if (qDir == 0) {
	beta  = (x1*x3-x2)/(x1*x3+x2);
    } else {
	beta  = (x1-x2*x3)/(x1+x2*x3);
    }
//--------------------------------------------------------------------------------------------------
    if (qDir == 0) {
	*P[1] = Vec4(0., 0., sqS/2./x3, sqS/2./x3).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[5] = Vec4(0., 0., sqS/2.*(1.-x3)/x3, sqS/2.*(1.-x3)/x3).bstZ(beta);
    } else {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2./x3, sqS/2./x3).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[5] = Vec4(0., 0., sqS/2.*(1.-x3)/x3, sqS/2.*(1.-x3)/x3).rot(0.,-1.,0.,-1.).bstZ(beta);
    }
    Q[1] = *P[1];
    Q[2] = *P[2];
    Q[3] = *P[4];
    Q[4] = *P[3];
    Q[5] = *P[5];
    if (info.Pid <= 203) {
	*P[3] = Q[3];
	*P[4] = Q[4];
    }
}

void Process::GetFourMomentaIBS2(double *Z, Vec4 *P[6]) {
    int qDir(1);
    double Cth,sqS,y,Phi;
    double Sth,Cph,Sph;
    double beta,x1,x2,x3;
    double kappa3,kappa4,beta3,beta4;
    double S;
    Vec4 Q[6];
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    sqS = Z[1];
    y = Z[2];
    x3 = Z[3];
    Phi = Z[4];
    if (Z[5] < 0.5) qDir = 0;
//--------------------------------------------------------------------------------------------------
    Sth = sqrt(1.-Cth*Cth);
    Cph = cos(Phi);
    Sph = sin(Phi);
    S = sqS*sqS;
    if (qDir == 0) {
	x1 = sqS/info.Ecm*exp(y);
	x2 = sqS/info.Ecm*exp(-y)/x3;
    } else {
	x1 = sqS/info.Ecm*exp(y)/x3;
	x2 = sqS/info.Ecm*exp(-y);
    }
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    if (qDir == 0) {
	beta  = (x1-x2*x3)/(x1+x2*x3);
    } else {
	beta  = (x1*x3-x2)/(x1*x3+x2);
    }
//--------------------------------------------------------------------------------------------------
    if (qDir == 0) {
	*P[1] = Vec4(0., 0., sqS/2., sqS/2.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2./x3, sqS/2./x3).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).bstZ(beta);
	*P[5] = Vec4(0., 0., sqS/2.*(1.-x3)/x3, sqS/2.*(1.-x3)/x3).bstZ(beta);
    } else {
	*P[1] = Vec4(0., 0., sqS/2./x3, sqS/2./x3).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[2] = Vec4(0., 0., -sqS/2., sqS/2.).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[3] = Vec4(0., 0., sqS/2.*beta3*kappa3, sqS/2.*kappa3).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[4] = Vec4(0., 0., -sqS/2.*beta4*kappa4, sqS/2.*kappa4).rot(Sth,Cth,Sph,Cph).rot(0.,-1.,0.,-1.).bstZ(beta);
	*P[5] = Vec4(0., 0., sqS/2.*(1.-x3)/x3, sqS/2.*(1.-x3)/x3).rot(0.,-1.,0.,-1.).bstZ(beta);
    }
    Q[1] = *P[1];
    Q[2] = *P[2];
    Q[3] = *P[4];
    Q[4] = *P[3];
    Q[5] = *P[5];
    if (info.Pid <= 203) {
	*P[3] = Q[3];
	*P[4] = Q[4];
    }
}

bool Process::Cuts(Vec4 *P[6]) {
    bool cuts = false;
    Vec4 v3 = *P[3];
    Vec4 v4 = *P[4];
    double Eta3L = v3.eta();
    double Eta4L = v4.eta();
    double Pt3 = v3.pT();
    double Pt4 = v4.pT();
    double M34 = m(v3, v4);
    double Mt34 = sqrt(abs(2.*v3.pT()*v4.pT()*(1.-cosphi(v3, v4))));
    double Y34 = 0.5*log((v3.e()+v4.e()+v3.pz()+v4.pz())/(v3.e()+v4.e()-v3.pz()-v4.pz()));
    if (info.Icuts == 0) {
	if (info.eta3cut && ((Eta3L < info.eta3min) || (Eta3L > info.eta3max))) cuts = true;
	if (info.eta4cut && ((Eta4L < info.eta4min) || (Eta4L > info.eta4max))) cuts = true;
	if (info.pt3cut && ((Pt3 < info.pt3min) || (Pt3 > info.pt3max))) cuts = true;
	if (info.pt4cut && ((Pt4 < info.pt4min) || (Pt4 > info.pt4max))) cuts = true;
	if (info.m34cut && ((M34 < info.m34min) || (M34 > info.m34max))) cuts = true;
	if (info.mt34cut && ((Mt34 < info.mt34min) || (Mt34 > info.mt34max))) cuts = true;
	if (info.y34cut && ((Y34 < info.y34min) || (Y34 > info.y34max))) cuts = true;
    } else if (info.Icuts == 1) {
	if ((abs(Eta3L) > 2.5) && (abs(Eta4L) > 2.5)) cuts = true;
	if ((abs(Eta3L) < 2.5) && (abs(Eta4L) < 2.5)) cuts = true;
	if ((abs(Eta3L) > 4.9) || (abs(Eta4L) > 4.9)) cuts = true;
	if (info.m34cut && ((M34 < info.m34min) || (M34 > info.m34max))) cuts = true;
	if (info.mt34cut && ((Mt34 < info.mt34min) || (Mt34 > info.mt34max))) cuts = true;
    } else if (info.Icuts == 2) {
	if (((Pt3 < 9.) || (Pt4 < 9.))) cuts = true;
	if (((Pt3 < 14.) && (Pt4 < 14.))) cuts = true;
	if (info.m34cut && ((M34 < info.m34min) || (M34 > info.m34max))) cuts = true;
	if (info.mt34cut && ((Mt34 < info.mt34min) || (Mt34 > info.mt34max))) cuts = true;
    }
    return cuts;
}

void Process::PdfConvBORN(double *Z,double *QQ) {
    int iSubProc;
    double FScale,FScale2(1.);
    double Q1(0.),Q2(0.);
    double sqS,S,x1,x2,y;
    S = Z[1];
    y = Z[2];
    iSubProc = (int)(Z[4]*10.) + 1;
    sqS = sqrt(S);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (info.Iscale == 1 || info.Iscale == 3) {
	FScale2 = S;
    } else if (info.Iscale == 2) {
	if (info.Pid <= 203) {
	    FScale2 = info.MZ*info.MZ;
	} else {
	    FScale2 = info.MW*info.MW;
	}
    }
    FScale = sqrt(FScale2);
    consts_.alphas = pdf->AlfasQ(FScale);
    scale = FScale;
    idup[1]=iQ[iSubProc-1];
    idup[2]=iQ[10-iSubProc];
    idup[5]=0;
    if (iQ[iSubProc-1]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[1][0] = 0;
        icolup[1][1] = 901;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[1][0] = 901;
        icolup[1][1] = 0;
    }
    icolup[2][0] = 0;
    icolup[3][0] = 0;
    icolup[4][0] = 0;
    icolup[2][1] = 0;
    icolup[3][1] = 0;
    icolup[4][1] = 0;

    if (SubProcFactor[iSubProc-1] != 0) {
	Q1 = SubProcFactor[iSubProc-1]*pdf->xfA(iQ[iSubProc-1], x1, FScale2)/x1;
	if (info.Ibeams == 1) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(iQ[10-iSubProc], x2, FScale2)/x2;
	} else if (info.Ibeams == 2) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(-iQ[10-iSubProc], x2, FScale2)/x2;
	}
    }
    *QQ = Q1*Q2;
}

void Process::PdfConvHARD(double *Z, double *QQ) {
    int iSubProc;
    double FScale,FScale2(1.);
    double Q1(0.),Q2(0.);
    double sqS,S,Spr,x1,x2,y;
    Spr = Z[0];
    S = Z[4];
    y = Z[5];
    iSubProc = (int)(Z[7]*20.) + 1;
    idup[5]=21;
    if ((iSubProc >= 6) && (iSubProc <= 15)) {
    idup[5]=22;
	iSubProc = iSubProc-5;
    } else if ((iSubProc >= 16) && (iSubProc <= 20)) {
	iSubProc = iSubProc-10;
    }
    sqS = sqrt(S);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (info.Iscale == 1) {
	FScale2 = S;
    } else if (info.Iscale == 2) {
	if (info.Pid <= 203) {
	    FScale2 = info.MZ*info.MZ;
	} else {
	    FScale2 = info.MW*info.MW;
	}
    } else if (info.Iscale == 3) {
	FScale2 = Spr;
    }
    FScale = sqrt(FScale2);
    consts_.alphas = pdf->AlfasQ(FScale);
    idup[1]=iQ[iSubProc-1];
    idup[2]=iQ[10-iSubProc];
    if (iQ[iSubProc-1]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[1][0] = 0;
        icolup[1][1] = 901;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[1][0] = 901;
        icolup[1][1] = 0;
    }
    icolup[2][0] = 0;
    icolup[3][0] = 0;
    icolup[4][0] = 0;
    icolup[2][1] = 0;
    icolup[3][1] = 0;
    icolup[4][1] = 0;
    if (SubProcFactor[iSubProc-1] != 0) {
	Q1 = SubProcFactor[iSubProc-1]*pdf->xfA(iQ[iSubProc-1], x1, FScale2)/x1;
	if (info.Ibeams == 1) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(iQ[10-iSubProc], x2, FScale2)/x2;
	} else if (info.Ibeams == 2) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(-iQ[10-iSubProc], x2, FScale2)/x2;
	}
    }
    *QQ = Q1*Q2;
}

void Process::PdfConvBRDQ(double *Z, double *QQ) {
    int iSubProc;
    double FScale,FScale2(1.);
    double xx1,xx2;
    double AminA,AminB,AmaxA,AmaxB;
    double SubtrA,SubtrB;
    double Ald1,Ard1,Ald2,Ard2;
    double Q1(0.),Q2(0.),Q1S(0.),Q2S(0.);
    double ResA(0.),ResB(0.);
    double D1DA(0.),D1DB(0.);
    double fQED,fQCD;
    double sqS,S,x1,x2,y,z;
    S = Z[1];
    sqS = sqrt(S);
    y = Z[2];
    z = Z[4];
    iSubProc = (int)(Z[5]*10.) + 1;
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (info.Iscale == 1 || info.Iscale == 3) {
	FScale2 = S;
    } else if (info.Iscale == 2) {
	if (info.Pid <= 203) {
	    FScale2 = info.MZ*info.MZ;
	} else {
	    FScale2 = info.MW*info.MW;
	}
    }
    FScale = sqrt(FScale2);
    scale = FScale;
    consts_.alphas = pdf->AlfasQ(FScale);
    if (info.Iqed != 0) fQED = 1.; else fQED = 0.;
    if (info.Iqcd != 0) fQCD = 1.; else fQCD = 0.;
    AminA = x1+1.E-10;
    AmaxA = 1.-1.E-10;
    xx1 = AminA+(AmaxA-AminA)*z;
    AminB = x2+1.E-10;
    AmaxB = 1.-1.E-10;
    xx2 = AminB+(AmaxB-AminB)*z;
    SubtrA = (1.+xx1*xx1)/(1.-xx1)*(log(FScale2/mQ2i[iSubProc-1])-2.*log(1.-xx1)-1.);
    SubtrB = (1.+xx2*xx2)/(1.-xx2)*(log(FScale2/mQ2i[10-iSubProc])-2.*log(1.-xx2)-1.);
    if (info.Imsb == 2) {
	SubtrA=SubtrA+(1.+xx1*xx1)/(1.-xx1)*log((1.-xx1)/xx1)-3./2./(1.-xx1)+2.*xx1+3.;
	SubtrB=SubtrB+(1.+xx2*xx2)/(1.-xx2)*log((1.-xx2)/xx2)-3./2./(1.-xx2)+2.*xx2+3.;
    }
    Ald1 = 1./2.*x1*x1+x1+2.*log(1.-x1);
    Ald2 = 1./2.*x2*x2+x2+2.*log(1.-x2);
    Ard1 = 2.*x1+(1.-2.*x1-x1*x1)*log(1.-x1)-2.*log(1.-x1)*log(1.-x1);
    Ard2 = 2.*x2+(1.-2.*x2-x2*x2)*log(1.-x2)-2.*log(1.-x2)*log(1.-x2);
    if (info.Imsb == 2) {
	Ard1 = Ard1+2.*TMath::DiLog(1.-x1)-consts_.pi*consts_.pi/3.+log(1.-x1)*log(1.-x1)+(-3.+x1+1./2.*x1*x1)*log(1.-x1)-(x1+1./2.*x1*x1)*log(x1)-1./2.*x1;
	Ard2 = Ard2+2.*TMath::DiLog(1.-x2)-consts_.pi*consts_.pi/3.+log(1.-x2)*log(1.-x2)+(-3.+x2+1./2.*x2*x2)*log(1.-x2)-(x2+1./2.*x2*x2)*log(x2)-1./2.*x2;
    }
    idup[1]=iQ[iSubProc-1];
    idup[2]=iQ[10-iSubProc];
    idup[5]=0;
    if (iQ[iSubProc-1]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[1][0] = 0;
        icolup[1][1] = 901;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[1][0] = 901;
        icolup[1][1] = 0;
    }
    icolup[2][0] = 0;
    icolup[3][0] = 0;
    icolup[4][0] = 0;
    icolup[2][1] = 0;
    icolup[3][1] = 0;
    icolup[4][1] = 0;
    if (SubProcFactor[iSubProc-1] != 0) {
	Q1 = SubProcFactor[iSubProc-1]*pdf->xfA(iQ[iSubProc-1], x1, FScale2)/x1;
	if ((AmaxA-AminA) > 0) ResA = SubProcFactor[iSubProc-1]*(AmaxA-AminA)*(pdf->xfA(iQ[iSubProc-1], x1/xx1, FScale2) - pdf->xfA(iQ[iSubProc-1], x1, FScale2))/x1*SubtrA;
	D1DA = Q1*(Ald1*log(FScale2/mQ2i[iSubProc-1])+Ard1);
	Q1S = Q1-(fQED*consts_.alpha*nc_.cfprime*eQ2i[iSubProc-1]+fQCD*consts_.alphas*consts_.cf)/2./consts_.pi*(D1DA+ResA);
	if (info.Ibeams == 1) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(iQ[10-iSubProc], x2, FScale2)/x2;
	    if ((AmaxB-AminB) > 0) ResB = SubProcFactor[iSubProc-1]*(AmaxB-AminB)*(pdf->xfB(iQ[10-iSubProc], x2/xx2, FScale2) - pdf->xfB(iQ[10-iSubProc], x2, FScale2))/x2*SubtrB;
	    D1DB = Q2*(Ald2*log(FScale2/mQ2i[10-iSubProc])+Ard2);
	    Q2S = Q2-(fQED*consts_.alpha*nc_.cfprime*eQ2i[10-iSubProc]+fQCD*consts_.alphas*consts_.cf)/2./consts_.pi*(D1DB+ResB);
	} else if (info.Ibeams == 2) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(-iQ[10-iSubProc], x2, FScale2)/x2;
	    if ((AmaxB-AminB) > 0) ResB = SubProcFactor[iSubProc-1]*(AmaxB-AminB)*(pdf->xfB(-iQ[10-iSubProc], x2/xx2, FScale2) - pdf->xfB(-iQ[10-iSubProc], x2, FScale2))/x2*SubtrB;
	    D1DB = Q2*(Ald2*log(FScale2/mQ2i[10-iSubProc])+Ard2);
	    Q2S = Q2-(fQED*consts_.alpha*nc_.cfprime*eQ2i[10-iSubProc]+fQCD*consts_.alphas*consts_.cf)/2./consts_.pi*(D1DB+ResB);
	}
    }
    *QQ = Q1S*Q2+Q1*Q2S-2.*Q1*Q2;
}

void Process::PdfConvIBAA(double *Z, double *QQ) {
    double FScale,FScale2(1.);
    double Q1(0.),Q2(0.);
    double sqS,S,x1,x2,y;
    sqS = Z[1];
    y = Z[2];
    S = sqS*sqS;
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (info.Iscale == 1 || info.Iscale == 3) {
	FScale2 = S;
    } else if (info.Iscale == 2) {
	if (info.Pid <= 203) {
	    FScale2 = info.MZ*info.MZ;
	} else {
	    FScale2 = info.MW*info.MW;
	}
    }
    idup[1]=22;
    idup[2]=22;
    idup[5]=0;
    icolup[0][0] = 0;
    icolup[0][1] = 0;
    icolup[1][0] = 0;
    icolup[1][1] = 0;
    icolup[2][0] = 0;
    icolup[3][0] = 0;
    icolup[4][0] = 0;
    icolup[2][1] = 0;
    icolup[3][1] = 0;
    icolup[4][1] = 0;
    FScale = sqrt(FScale2);
    scale = FScale;
    consts_.alphas = pdf->AlfasQ(FScale);
    Q1 = pdf->xfA(22, x1, FScale2)/x1;
    Q2 = pdf->xfB(22, x2, FScale2)/x2;
    *QQ = Q1*Q2;
}

void Process::PdfConvIBAQ(double *Z, double *QQ) {
    int iSubProc;
    double FScale,FScale2(1.);
    double Q1(0.),Q2(0.);
    double x1,x2,y,sqS,S,Spr;
    Spr = Z[0];
    S = Z[4];
    y = Z[5];
    iSubProc = (int)(Z[7]*40.) + 1;
    sqS = sqrt(S);
    x1 = sqS/info.Ecm*exp(y);
    x2 = sqS/info.Ecm*exp(-y);
    if (info.Iscale == 1) {
	FScale2 = S;
    } else if (info.Iscale == 2) {
	if (info.Pid <= 203) {
	    FScale2 = info.MZ*info.MZ;
	} else {
	    FScale2 = info.MW*info.MW;
	}
    } else if (info.Iscale == 3) {
	FScale2 = Spr;
    }
    FScale = sqrt(FScale2);
    scale = FScale;
    consts_.alphas = pdf->AlfasQ(FScale);
    if (iSubProc <= 10) {
    idup[1]=21;
    idup[2]=iQ[iSubProc-1];
    idup[5]=-iQ[10-iSubProc];
    if (iQ[iSubProc-1]>0) {
        icolup[1][0] = 901;
        icolup[1][1] = 0;
        icolup[4][0] = 901;
        icolup[4][1] = 0;
    } else {
        icolup[1][0] = 0;
        icolup[1][1] = 901;
        icolup[4][0] = 0;
        icolup[4][1] = 901;
    }
    icolup[0][0] = 0;
    icolup[0][1] = 0;
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[iSubProc-1]*pdf->xfA(21, x1, FScale2)/x1;
	if (info.Ibeams == 1) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(iQ[iSubProc-1], x2, FScale2)/x2;
	} else if (info.Ibeams == 2) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(-iQ[iSubProc-1], x2, FScale2)/x2;
	}
    } else if (iSubProc >= 11 && iSubProc <= 20) {
    idup[1]=22;
    idup[2]=iQ[iSubProc-11];
    idup[5]=-iQ[20-iSubProc];
    if (iQ[iSubProc-11]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 902;
        icolup[1][0] = 902;
        icolup[1][1] = 0;
        icolup[4][0] = 901;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 902;
        icolup[0][1] = 901;
        icolup[1][0] = 0;
        icolup[1][1] = 902;
        icolup[4][0] = 0;
        icolup[4][1] = 901;
    }
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[iSubProc-11]*pdf->xfA(22, x1, FScale2)/x1;
	if (info.Ibeams == 1) {
	    Q2 = SubProcFactor[iSubProc-11]*pdf->xfB(iQ[iSubProc-11], x2, FScale2)/x2;
	} else if (info.Ibeams == 2) {
	    Q2 = SubProcFactor[iSubProc-11]*pdf->xfB(-iQ[iSubProc-11], x2, FScale2)/x2;
	}
    } else if (iSubProc >= 21 && iSubProc <= 30) {
    idup[1]=iQ[30-iSubProc];
    idup[2]=22;
    idup[5]=-iQ[iSubProc-21];
    if (iQ[30-iSubProc]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[1][0] = 902;
        icolup[1][1] = 901;
        icolup[4][0] = 902;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[1][0] = 901;
        icolup[1][1] = 902;
        icolup[4][0] = 0;
        icolup[4][1] = 902;
    }
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[30-iSubProc]*pdf->xfA(iQ[30-iSubProc], x1, FScale2)/x1;
	Q2 = SubProcFactor[30-iSubProc]*pdf->xfB(22, x2, FScale2)/x2;
    } else if (iSubProc >= 31 && iSubProc <= 40) {
    idup[1]=iQ[40-iSubProc];
    idup[2]=21;
    idup[5]=-iQ[iSubProc-21];
    if (iQ[40-iSubProc]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[4][0] = 901;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[4][0] = 0;
        icolup[4][1] = 901;
    }
    icolup[1][0] = 0;
    icolup[1][1] = 0;
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[40-iSubProc]*pdf->xfA(iQ[40-iSubProc], x1, FScale2)/x1;
	Q2 = SubProcFactor[40-iSubProc]*pdf->xfB(21, x2, FScale2)/x2;
    }
    *QQ = Q1*Q2;
}

void Process::PdfConvIBS1(double *Z, double *QQ) {
    int iSubProc;
    double FScale,FScale2(1.);
    double Q1(0.),Q2(0.);
    double SubtrQ(0.);
    double fQED,fQCD;
    double sqS,S,x1,x2,x3,y;
    sqS = Z[1];
    y = Z[2];
    x3 = Z[3];
    iSubProc = (int)(Z[5]*40.) + 1;
    S = sqS*sqS;
    if (iSubProc <= 20) {
	x1 = sqS/info.Ecm*exp(y)/x3;
	x2 = sqS/info.Ecm*exp(-y);
    } else {
	x1 = sqS/info.Ecm*exp(y);
	x2 = sqS/info.Ecm*exp(-y)/x3;
    }
    if (info.Iscale == 1 || info.Iscale == 3) {
	FScale2 = S;
    } else if (info.Iscale == 2) {
	if (info.Pid <= 203) {
	    FScale2 = info.MZ*info.MZ;
	} else {
	    FScale2 = info.MW*info.MW;
	}
    }
    FScale = sqrt(FScale2);
    scale = FScale;
    consts_.alphas = pdf->AlfasQ(FScale);
//--------------------------------------------------------------------------------------------------
    if (info.Iph >= 2) fQED = 1.; else fQED = 0.;
    if (info.Iqcd != 0) fQCD = 1.; else fQCD = 0.;
//--------------------------------------------------------------------------------------------------
    if (iSubProc <= 10 && fQCD == 1 && SubProcFactor[iSubProc-1] != 0) {
    idup[1]=21;
    idup[2]=iQ[iSubProc-1];
    idup[5]=-iQ[10-iSubProc];
    if (iQ[iSubProc-1]>0) {
        icolup[1][0] = 901;
        icolup[1][1] = 0;
        icolup[4][0] = 901;
        icolup[4][1] = 0;
    } else {
        icolup[1][0] = 0;
        icolup[1][1] = 901;
        icolup[4][0] = 0;
        icolup[4][1] = 901;
    }
    icolup[0][0] = 0;
    icolup[0][1] = 0;
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[iSubProc-1]*pdf->xfA(21, x1, FScale2)/x1;
	if (info.Ibeams == 1) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(iQ[iSubProc-1], x2, FScale2)/x2;
	} else if (info.Ibeams == 2) {
	    Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(-iQ[iSubProc-1], x2, FScale2)/x2;
	}
	if (info.Imsb == 1) {
	    SubtrQ = consts_.alphas*consts_.cf/2./consts_.pi*log(100.*FScale2/mQ2f[iSubProc-1])*(x3*x3+(1.-x3)*(1.-x3))*nc_.fc/8.;
	} else if (info.Imsb == 2) {
	    SubtrQ = consts_.alphas*consts_.cf/2./consts_.pi*(log(100.*FScale2*(1.-x3)/mQ2f[iSubProc-1]/x3)*(x3*x3+(1.-x3)*(1.-x3))-16.*x3*x3+16.*x3-2.)*nc_.fc/8.;
	}
    } else if (iSubProc >= 11 && iSubProc <= 20 && fQED == 1 && SubProcFactor[iSubProc-11] != 0) {
    idup[1]=22;
    idup[2]=iQ[iSubProc-11];
    idup[5]=-iQ[20-iSubProc];
    if (iQ[iSubProc-11]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 902;
        icolup[1][0] = 902;
        icolup[1][1] = 0;
        icolup[4][0] = 901;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 902;
        icolup[0][1] = 901;
        icolup[1][0] = 0;
        icolup[1][1] = 902;
        icolup[4][0] = 0;
        icolup[4][1] = 901;
    }
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[iSubProc-11]*pdf->xfA(22, x1, FScale2)/x1;
	if (info.Ibeams == 1) {
	    Q2 = SubProcFactor[iSubProc-11]*pdf->xfB(iQ[iSubProc-11], x2, FScale2)/x2;
	} else if (info.Ibeams == 2) {
	    Q2 = SubProcFactor[iSubProc-11]*pdf->xfB(-iQ[iSubProc-11], x2, FScale2)/x2;
	}
	if (info.Imsb == 1) {
	    SubtrQ = consts_.alpha*nc_.cfprime*eQ2f[iSubProc-11]/2./consts_.pi*log(FScale2/mQ2f[iSubProc-11])*(x3*x3+(1.-x3)*(1.-x3))*nc_.fc;
	} else if (info.Imsb == 2) {
	    SubtrQ = consts_.alpha*nc_.cfprime*eQ2f[iSubProc-11]/2./consts_.pi*(log(FScale2*(1.-x3)/mQ2f[iSubProc-11]/x3)*(x3*x3+(1.-x3)*(1.-x3))-16.*x3*x3+16.*x3-2.)*nc_.fc;
	}
    } else if (iSubProc >= 21 && iSubProc <= 30 && fQED == 1 && SubProcFactor[30-iSubProc] != 0) {
    idup[1]=iQ[30-iSubProc];
    idup[2]=22;
    idup[5]=-iQ[iSubProc-21];
    if (iQ[30-iSubProc]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[1][0] = 902;
        icolup[1][1] = 901;
        icolup[4][0] = 902;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[1][0] = 901;
        icolup[1][1] = 902;
        icolup[4][0] = 0;
        icolup[4][1] = 902;
    }
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[30-iSubProc]*pdf->xfA(iQ[30-iSubProc], x1, FScale2)/x1;
	Q2 = SubProcFactor[30-iSubProc]*pdf->xfB(22, x2, FScale2)/x2;
	if (info.Imsb == 1) {
	    SubtrQ = consts_.alpha*nc_.cfprime*eQ2f[30-iSubProc]/2./consts_.pi*log(FScale2/mQ2f[30-iSubProc])*(x3*x3+(1.-x3)*(1.-x3))*nc_.fc;
	} else if (info.Imsb == 2) {
	    SubtrQ = consts_.alpha*nc_.cfprime*eQ2f[30-iSubProc]/2./consts_.pi*(log(FScale2*(1.-x3)/mQ2f[30-iSubProc]/x3)*(x3*x3+(1.-x3)*(1.-x3))-16.*x3*x3+16.*x3-2.)*nc_.fc;
	}
    } else if (iSubProc >= 31 && iSubProc <= 40 && fQCD == 1 && SubProcFactor[40-iSubProc] != 0) {
    idup[1]=iQ[40-iSubProc];
    idup[2]=21;
    idup[5]=-iQ[iSubProc-21];
    if (iQ[40-iSubProc]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[4][0] = 901;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[4][0] = 0;
        icolup[4][1] = 901;
    }
    icolup[1][0] = 0;
    icolup[1][1] = 0;
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	Q1 = SubProcFactor[40-iSubProc]*pdf->xfA(iQ[40-iSubProc], x1, FScale2)/x1;
	Q2 = SubProcFactor[40-iSubProc]*pdf->xfB(21, x2, FScale2)/x2;
	if (info.Imsb == 1) {
	    SubtrQ = consts_.alphas*consts_.cf/2./consts_.pi*log(100.*FScale2/mQ2f[40-iSubProc])*(x3*x3+(1.-x3)*(1.-x3))*nc_.fc/8.;
	} else if (info.Imsb == 2) {
	    SubtrQ = consts_.alphas*consts_.cf/2./consts_.pi*(log(100.*FScale2*(1.-x3)/mQ2f[40-iSubProc]/x3)*(x3*x3+(1.-x3)*(1.-x3))-16.*x3*x3+16.*x3-2.)*nc_.fc/8.;
	}
    }
    *QQ = -Q1*Q2*SubtrQ;
}

void Process::PdfConvIBS2(double *Z, double *QQ) {
    int iSubProc;
    double FScale,FScale2(1.);
    double Q1(0.),Q2(0.),Q1S(0.),Q2S(0.);
    double SubtrQ(0.),SubtrQS(0.);
    double sqS,S,x1,x2,x3,y,xx1,xx2;
    sqS = Z[1];
    y = Z[2];
    x3 = Z[3];
    iSubProc = (int)(Z[5]*20.) + 1;
    S = sqS*sqS;
    if (iSubProc <= 10) {
	x1 = sqS/info.Ecm*exp(y);
	x2 = sqS/info.Ecm*exp(-y)/x3;
    } else {
	x1 = sqS/info.Ecm*exp(y)/x3;
	x2 = sqS/info.Ecm*exp(-y);
    }
    xx1 = sqS/info.Ecm*exp(y);
    xx2 = sqS/info.Ecm*exp(-y);
    if (info.Iscale == 1 || info.Iscale == 3) {
	FScale2 = S;
    } else if (info.Iscale == 2) {
	if (info.Pid <= 203) {
	    FScale2 = info.MZ*info.MZ;
	} else {
	    FScale2 = info.MW*info.MW;
	}
    }
    FScale = sqrt(FScale2);
    scale = FScale;
    consts_.alphas = pdf->AlfasQ(FScale);
//--------------------------------------------------------------------------------------------------
    if (iSubProc <= 10 && SubProcFactor[iSubProc-1] != 0) {
    idup[1]=22;
    idup[2]=iQ[iSubProc-1];
    idup[5]=-iQ[0-iSubProc];
    if (iQ[iSubProc-1]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 902;
        icolup[1][0] = 902;
        icolup[1][1] = 0;
        icolup[4][0] = 901;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 902;
        icolup[0][1] = 901;
        icolup[1][0] = 0;
        icolup[1][1] = 902;
        icolup[4][0] = 0;
        icolup[4][1] = 901;
    }
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	if (x1 <= info.xmin || x1 >= info.xmax) Q1 = 0.; else Q1 = SubProcFactor[iSubProc-1]*pdf->xfA(22, x1, FScale2)/x1;
	if (info.Ibeams == 1) {
	    if (x2 <= info.xmin || x2 >= info.xmax) Q2 = 0.; else Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(iQ[iSubProc-1], x2, FScale2)/x2;
	    if (xx2 <= info.xmin || xx2 >= info.xmax) Q2S = 0.; else Q2S = SubProcFactor[iSubProc-1]*pdf->xfB(iQ[iSubProc-1], xx2, FScale2)/x2;
	} else if (info.Ibeams == 2) {
	    if (x2 <= info.xmin || x2 >= info.xmax) Q2 = 0.; else Q2 = SubProcFactor[iSubProc-1]*pdf->xfB(-iQ[iSubProc-1], x2, FScale2)/x2;
	    if (xx2 <= info.xmin || xx2 >= info.xmax) Q2S = 0.; else Q2S = SubProcFactor[iSubProc-1]*pdf->xfB(-iQ[iSubProc-1], xx2, FScale2)/x2;
	}
	SubtrQ = consts_.alpha/nc_.cfprime/2./consts_.pi*eQ2i[iSubProc-1]*(1.+(1.-x3)*(1.-x3))/x3*(log(FScale2/mQ2i[iSubProc-1])-2.*log(x3)-1.);
	SubtrQS = consts_.alpha/nc_.cfprime/2./consts_.pi*eQ2i[iSubProc-1]*((1.+x3*x3)/(1.-x3)*(log(x3/(1.-x3))+3./4.)-(9.+5.*x3)/4.);
    } else if (iSubProc >= 10 && iSubProc <= 20 && SubProcFactor[20-iSubProc] != 0) {
    idup[1]=iQ[20-iSubProc];
    idup[2]=22;
    idup[5]=-iQ[iSubProc-11];
    if (iQ[20-iSubProc]>0) {
        icolup[0][0] = 901;
        icolup[0][1] = 0;
        icolup[1][0] = 902;
        icolup[1][1] = 901;
        icolup[4][0] = 902;
        icolup[4][1] = 0;
    } else {
        icolup[0][0] = 0;
        icolup[0][1] = 901;
        icolup[1][0] = 901;
        icolup[1][1] = 902;
        icolup[4][0] = 0;
        icolup[4][1] = 902;
    }
    icolup[2][0] = 0;
    icolup[2][1] = 0;
    icolup[3][0] = 0;
    icolup[3][1] = 0;
	if (x1 <= info.xmin || x1 >= info.xmax) Q1 = 0.; else Q1 = SubProcFactor[20-iSubProc]*pdf->xfA(iQ[20-iSubProc], x1, FScale2)/x1;
	if (xx1 <= info.xmin || xx1 >= info.xmax) Q1S = 0.; else Q1S = SubProcFactor[20-iSubProc]*pdf->xfA(iQ[20-iSubProc], xx1, FScale2)/x1;
	if (x2 <= info.xmin || x2 >= info.xmax) Q2 = 0.; else Q2 = SubProcFactor[20-iSubProc]*pdf->xfB(22, x2, FScale2)/x2;
	SubtrQ = consts_.alpha/nc_.cfprime/2./consts_.pi*eQ2i[20-iSubProc]*(1.+(1.-x3)*(1.-x3))/x3*(log(FScale2/mQ2i[20-iSubProc])-2.*log(x3)-1.);
	SubtrQS = consts_.alpha/nc_.cfprime/2./consts_.pi*eQ2i[20-iSubProc]*((1.+x3*x3)/(1.-x3)*(log(x3/(1.-x3))+3./4.)-(9.+5.*x3)/4.);
    }
    *QQ = -Q1*Q2*SubtrQ;
    if (info.Imsb == 2) {
	*QQ = *QQ-(Q1-Q1S)*(Q2-Q2S)*SubtrQS;
    }
}

void Process::Born(double *Z, double *BornQ) {
    bool SubProcD(false), SubProcU(false);
    int iSubProc;
    double Cth;
    double kappa3,kappa4,beta3,beta4;
    double S,T,U,Qs,Qt,Qu;
    double SoftU(0.),HardU(0.),SoftD(0.),HardD(0.);
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    S = Z[1];
    iSubProc = (int)(Z[4]*10.) + 1;
    SubProcD = ((iSubProc < 6 && iSubProc%2 == 1) || (iSubProc >= 6 && iSubProc%2 == 0));
    SubProcU = ((iSubProc < 6 && iSubProc%2 == 0) || (iSubProc >= 6 && iSubProc%2 == 1));
//--------------------------------------------------------------------------------------------------
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    T = Mf3_2-S/2.*kappa3*(1.-beta3*Cth);
    U = Mf4_2-S/2.*kappa4*(1.+beta4*Cth);
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
    if (info.Pid == 201) {
	if (SubProcD) nc_br_1414_1212_(&Qs,&Qt,&Qu,BornQ,&SoftD,&HardD);
	if (SubProcU) nc_br_1313_1212_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 202) {
	if (SubProcD) nc_br_1414_1616_(&Qs,&Qt,&Qu,BornQ,&SoftD,&HardD);
	if (SubProcU) nc_br_1313_1616_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 203) {
	if (SubProcD) nc_br_1414_2020_(&Qs,&Qt,&Qu,BornQ,&SoftD,&HardD);
	if (SubProcU) nc_br_1313_2020_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 211) {
	cc_br_1314_1112_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 212) {
	cc_br_1314_1516_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 213) {
	cc_br_1314_1920_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 221) {
	cc_br_1413_1211_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 222) {
	cc_br_1413_1615_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    } else if (info.Pid == 223) {
	cc_br_1413_2019_(&Qs,&Qt,&Qu,BornQ,&SoftU,&HardU);
    }
}

void Process::Virt(double *Z, double *VirtQ) {
    bool SubProcD(false), SubProcU(false);
    int iSubProc;
    double Cth,sqS;
    double kappa3,kappa4,beta3,beta4;
    double S,T,U,Qs,Qt,Qu;
    double SoftQqed(0.),SoftQqcd(0.),SoftQho(0.);
    double HardQqed(0.),HardQqcd(0.),HardQho(0.);
    double BornQqed(0.),BornQqcd(0.),BornQho(0.);
    double SigmaQqed(0.);
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    S = Z[1];
    iSubProc = (int)(Z[4]*10.) + 1;
    SubProcD = ((iSubProc < 6 && iSubProc%2 == 1) || (iSubProc >= 6 && iSubProc%2 == 0));
    SubProcU = ((iSubProc < 6 && iSubProc%2 == 0) || (iSubProc >= 6 && iSubProc%2 == 1));
//--------------------------------------------------------------------------------------------------
    sqS = sqrt(S);
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    T = Mf3_2-S/2.*kappa3*(1.-beta3*Cth);
    U = Mf4_2-S/2.*kappa4*(1.+beta4*Cth);
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
    scales_.omega = info.Ome*sqS/2.;
    if (info.Pid == 201) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    if (SubProcU) nc_br_1313_1212_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    if (SubProcD) nc_br_1414_1212_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    if (info.Ifs == 0) {
		if (SubProcU) nc_si_1313_1212_(&S,&T,&U,&SigmaQqed);
		if (SubProcD) nc_si_1414_1212_(&S,&T,&U,&SigmaQqed);
	    } else if (info.Ifs == 1) {
		if (SubProcU) nc_si_1313_1212_fs_(&S,&T,&U,&SigmaQqed);
		if (SubProcD) nc_si_1414_1212_fs_(&S,&T,&U,&SigmaQqed);
	    }
	}
	if (info.Iqcd != 0) {
	    if (SubProcU) nc_qcd_br_1313_1212_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	    if (SubProcD) nc_qcd_br_1414_1212_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
	if (info.Iho != 0) {
	    if (SubProcU) nc_ho_1313_1212_(&Qs,&Qt,&Qu,&BornQho,&SoftQho,&HardQho);
	    if (SubProcD) nc_ho_1414_1212_(&Qs,&Qt,&Qu,&BornQho,&SoftQho,&HardQho);
	}
    } else if (info.Pid == 202) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    if (SubProcU) nc_br_1313_1616_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    if (SubProcD) nc_br_1414_1616_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    if (info.Ifs == 0) {
		if (SubProcU) nc_si_1313_1616_(&S,&T,&U,&SigmaQqed);
		if (SubProcD) nc_si_1414_1616_(&S,&T,&U,&SigmaQqed);
	    } else if (info.Ifs == 1) {
		if (SubProcU) nc_si_1313_1616_fs_(&S,&T,&U,&SigmaQqed);
		if (SubProcD) nc_si_1414_1616_fs_(&S,&T,&U,&SigmaQqed);
	    }
	}
	if (info.Iqcd != 0) {
	    if (SubProcU) nc_qcd_br_1313_1616_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	    if (SubProcD) nc_qcd_br_1414_1616_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
	if (info.Iho != 0) {
	    if (SubProcU) nc_ho_1313_1212_(&Qs,&Qt,&Qu,&BornQho,&SoftQho,&HardQho);
	    if (SubProcD) nc_ho_1414_1212_(&Qs,&Qt,&Qu,&BornQho,&SoftQho,&HardQho);
	}
    } else if (info.Pid == 203) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    if (SubProcU) nc_br_1313_2020_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    if (SubProcD) nc_br_1414_2020_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    if (info.Ifs == 0) {
		if (SubProcU) nc_si_1313_2020_(&S,&T,&U,&SigmaQqed);
		if (SubProcD) nc_si_1414_2020_(&S,&T,&U,&SigmaQqed);
	    } else if (info.Ifs == 1) {
		if (SubProcU) nc_si_1313_2020_fs_(&S,&T,&U,&SigmaQqed);
		if (SubProcD) nc_si_1414_2020_fs_(&S,&T,&U,&SigmaQqed);
	    }
	}
	if (info.Iqcd != 0) {
	    if (SubProcU) nc_qcd_br_1313_2020_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	    if (SubProcD) nc_qcd_br_1414_2020_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
	if (info.Iho != 0) {
	    if (SubProcU) nc_ho_1313_1212_(&Qs,&Qt,&Qu,&BornQho,&SoftQho,&HardQho);
	    if (SubProcD) nc_ho_1414_1212_(&Qs,&Qt,&Qu,&BornQho,&SoftQho,&HardQho);
	}
    } else if (info.Pid == 211) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    cc_br_1314_1112_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    cc_si_1314_1112_(&S,&T,&U,&SigmaQqed);
	}
	if (info.Iqcd != 0) {
	    cc_qcd_br_1314_1112_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
    } else if (info.Pid == 212) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    cc_br_1314_1516_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    cc_si_1314_1516_(&S,&T,&U,&SigmaQqed);
	}
	if (info.Iqcd != 0) {
	    cc_qcd_br_1314_1516_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
    } else if (info.Pid == 213) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    cc_br_1314_1920_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    cc_si_1314_1920_(&S,&T,&U,&SigmaQqed);
	}
	if (info.Iqcd != 0) {
	    cc_qcd_br_1314_1920_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
    } else if (info.Pid == 221) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    cc_br_1413_1211_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    cc_si_1413_1211_(&S,&T,&U,&SigmaQqed);
	}
	if (info.Iqcd != 0) {
	    cc_qcd_br_1413_1211_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
    } else if (info.Pid == 222) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    cc_br_1413_1615_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    cc_si_1413_1615_(&S,&T,&U,&SigmaQqed);
	}
	if (info.Iqcd != 0) {
	    cc_qcd_br_1413_1615_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
    } else if (info.Pid == 223) {
	if (info.Iqed != 0 || info.Iew != 0) {
	    cc_br_1413_2019_(&Qs,&Qt,&Qu,&BornQqed,&SoftQqed,&HardQqed);
	    cc_si_1413_2019_(&S,&T,&U,&SigmaQqed);
	}
	if (info.Iqcd != 0) {
	    cc_qcd_br_1413_2019_(&Qs,&Qt,&Qu,&BornQqcd,&SoftQqcd,&HardQqcd);
	}
    }
    *VirtQ = SigmaQqed+SoftQqed+SoftQqcd+BornQho;
}

void Process::Hard(double *Z, double *HardQ) {
    bool SubProcDqed(false),SubProcUqed(false),SubProcDqcd(false),SubProcUqcd(false);
    int iSubProc;
    double S,Spr,Phi4,Cth4,Cth5,sqS;
    double HardQqed(0.),HardQqcd(0.);
//--------------------------------------------------------------------------------------------------
    Spr= Z[0];
    Phi4 = Z[1];
    Cth4 = Z[2];
    Cth5 = Z[3];
    S = Z[4];
    iSubProc = (int)(Z[7]*20.) + 1;
    SubProcDqed = (((iSubProc < 11 && iSubProc >= 6 && iSubProc%2 == 0) || (iSubProc >= 11 && iSubProc < 16 && iSubProc%2 == 1)) && info.Iqed != 0);
    SubProcUqed = (((iSubProc < 11 && iSubProc >= 6 && iSubProc%2 == 1) || (iSubProc >= 11 && iSubProc < 16 && iSubProc%2 == 0)) && info.Iqed != 0);
    SubProcDqcd = (((iSubProc < 6 && iSubProc%2 == 1) || (iSubProc >= 16 && iSubProc%2 == 0)) && info.Iqcd != 0);
    SubProcUqcd = (((iSubProc < 6 && iSubProc%2 == 0) || (iSubProc >= 16 && iSubProc%2 == 1)) && info.Iqcd != 0);
//--------------------------------------------------------------------------------------------------
    sqS = sqrt(S);
    scales_.omega = info.Ome*sqS/2.;
    if (info.Pid == 201) {
	if (SubProcDqed) nc_ha_1414_1212_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcUqed) nc_ha_1313_1212_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd) nc_qcd_ha_1414_1212_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
	if (SubProcUqcd) nc_qcd_ha_1313_1212_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 202) {
	if (SubProcDqed) nc_ha_1414_1616_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcUqed) nc_ha_1313_1616_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd) nc_qcd_ha_1414_1616_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
	if (SubProcUqcd) nc_qcd_ha_1313_1616_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 203) {
	if (SubProcDqed) nc_ha_1414_2020_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcUqed) nc_ha_1313_2020_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd) nc_qcd_ha_1414_2020_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
	if (SubProcUqcd) nc_qcd_ha_1313_2020_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 211) {
	Cth4 = -Cth4;
	Cth5 = -Cth5;
	if (SubProcDqed || SubProcUqed) cc_ha_1314_1112_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd || SubProcUqcd) cc_qcd_ha_1314_1112_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 212) {
	Cth4 = -Cth4;
	Cth5 = -Cth5;
	if (SubProcDqed || SubProcUqed)cc_ha_1314_1516_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd || SubProcUqcd)cc_qcd_ha_1314_1516_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 213) {
	Cth4 = -Cth4;
	Cth5 = -Cth5;
	if (SubProcDqed || SubProcUqed)cc_ha_1314_1920_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd || SubProcUqcd)cc_qcd_ha_1314_1920_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 221) {
	if (SubProcDqed || SubProcUqed)cc_ha_1413_1211_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd || SubProcUqcd)cc_qcd_ha_1413_1211_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 222) {
	if (SubProcDqed || SubProcUqed)cc_ha_1413_1615_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd || SubProcUqcd)cc_qcd_ha_1413_1615_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    } else if (info.Pid == 223) {
	if (SubProcDqed || SubProcUqed)cc_ha_1413_2019_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqed);
	if (SubProcDqcd || SubProcUqcd)cc_qcd_ha_1413_2019_(&S,&Spr,&Phi4,&Cth4,&Cth5,&HardQqcd);
    }
    *HardQ = HardQqed+HardQqcd;
}

void Process::Brdq(double *Z, double *BrdqQ) {
    bool SubProcD(false), SubProcU(false);
    int iSubProc;
    double Cth;
    double kappa3,kappa4,beta3,beta4;
    double S,T,U,Qs,Qt,Qu;
    double SoftU(0.),HardU(0.),SoftD(0.),HardD(0.);
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    S = Z[1];
    iSubProc = (int)(Z[5]*10.) + 1;
    SubProcD = ((iSubProc < 6 && iSubProc%2 == 1) || (iSubProc >= 6 && iSubProc%2 == 0));
    SubProcU = ((iSubProc < 6 && iSubProc%2 == 0) || (iSubProc >= 6 && iSubProc%2 == 1));
//--------------------------------------------------------------------------------------------------
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    T = Mf3_2-S/2.*kappa3*(1.-beta3*Cth);
    U = Mf4_2-S/2.*kappa4*(1.+beta4*Cth);
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
    if (info.Pid == 201) {
	if (SubProcD) nc_br_1414_1212_(&Qs,&Qt,&Qu,BrdqQ,&SoftD,&HardD);
	if (SubProcU) nc_br_1313_1212_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 202) {
	if (SubProcD) nc_br_1414_1616_(&Qs,&Qt,&Qu,BrdqQ,&SoftD,&HardD);
	if (SubProcU) nc_br_1313_1616_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 203) {
	if (SubProcD) nc_br_1414_2020_(&Qs,&Qt,&Qu,BrdqQ,&SoftD,&HardD);
	if (SubProcU) nc_br_1313_2020_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 211) {
	cc_br_1314_1112_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 212) {
	cc_br_1314_1516_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 213) {
	cc_br_1314_1920_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 221) {
	cc_br_1413_1211_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 222) {
	cc_br_1413_1615_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    } else if (info.Pid == 223) {
	cc_br_1413_2019_(&Qs,&Qt,&Qu,BrdqQ,&SoftU,&HardU);
    }
}

void Process::Ibaa(double *Z, double *IbaaQ) {
    double Cth,sqS;
    double kappa3,kappa4,beta3,beta4;
    double S,T,U,Qs,Qt,Qu;
    double Soft(0.),Hard(0.);
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    sqS = Z[1];
//--------------------------------------------------------------------------------------------------
    S = sqS*sqS;
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    T = Mf3_2-S/2.*kappa3*(1.-beta3*Cth);
    U = Mf4_2-S/2.*kappa4*(1.+beta4*Cth);
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
    if (info.Pid == 201) {
	nc_br_11_1212_(&Qs,&Qt,&Qu,IbaaQ,&Soft,&Hard);
    } else if (info.Pid == 202) {
	nc_br_11_1616_(&Qs,&Qt,&Qu,IbaaQ,&Soft,&Hard);
    } else if (info.Pid == 203) {
	nc_br_11_2020_(&Qs,&Qt,&Qu,IbaaQ,&Soft,&Hard);
    }
}

void Process::Ibaq(double *Z, double *IbaqQ) {
    bool SubProcDqed(false),SubProcUqed(false),SubProcDaqed(false),SubProcUaqed(false);
    bool SubProcDqcd(false),SubProcUqcd(false),SubProcDaqcd(false),SubProcUaqcd(false);
    int iSubProc;
    double S,Spr,Phi4,Cth4,Cth5,sqS;
//--------------------------------------------------------------------------------------------------
    Spr = Z[0];
    Phi4 = Z[1];
    Cth4 = Z[2];
    Cth5 = Z[3];
    S = Z[4];
    iSubProc = (int)(Z[7]*40.) + 1;
    if (info.Pid == 201 || info.Pid == 202 || info.Pid == 203) {
	SubProcUqed  = ((info.Iph >= 2)  && ((iSubProc >= 11 && iSubProc <= 15 && (iSubProc-10)%2 == 0) || ((41-iSubProc) >= 11 && (41-iSubProc) <= 15 && (31-iSubProc)%2 == 0)));
	SubProcUaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 20 && (iSubProc-15)%2 == 0) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 20 && (26-iSubProc)%2 == 0)));
	SubProcDqed  = ((info.Iph >= 2)  && ((iSubProc >= 11 && iSubProc <= 15 && (iSubProc-10)%2 == 1) || ((41-iSubProc) >= 11 && (41-iSubProc) <= 15 && (31-iSubProc)%2 == 1)));
	SubProcDaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 20 && (iSubProc-15)%2 == 1) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 20 && (26-iSubProc)%2 == 1)));
	SubProcUqcd  = ((info.Iqcd == 1) && ((iSubProc >= 1 && iSubProc <= 5 && (iSubProc)%2 == 0) || ((41-iSubProc) >= 1 && (41-iSubProc) <= 5 && (41-iSubProc)%2 == 0)));
	SubProcUaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 10 && (iSubProc-5)%2 == 0) || ((41-iSubProc) >= 6 && (41-iSubProc) <= 10 && (36-iSubProc)%2 == 0)));
	SubProcDqcd  = ((info.Iqcd == 1) && ((iSubProc >= 1 && iSubProc <= 5 && (iSubProc)%2 == 1) || ((41-iSubProc) >= 1 && (41-iSubProc) <= 5 && (41-iSubProc)%2 == 1)));
	SubProcDaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 10 && (iSubProc-5)%2 == 1) || ((41-iSubProc) >= 6 && (41-iSubProc) <= 10 && (36-iSubProc)%2 == 1)));
    } else if (info.Pid == 211 || info.Pid == 212 || info.Pid == 213) {
	SubProcUaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 19) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 19)));
	SubProcDqed  = ((info.Iph >= 2)  && ((iSubProc >= 12 && iSubProc <= 15) || ((41-iSubProc) >= 12 && (41-iSubProc) <= 15)));
	SubProcUaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 9) || ((41-iSubProc) >= 6 && (41-iSubProc) <= 9)));
	SubProcDqcd  = ((info.Iqcd == 1) && ((iSubProc >= 2 && iSubProc <= 5) || ((41-iSubProc) >= 2 && (41-iSubProc) <= 5)));
    } else if (info.Pid == 221 || info.Pid == 222 || info.Pid == 223) {
	SubProcUqed  = ((info.Iph >= 2)  && ((iSubProc >= 12 && iSubProc <= 15) || ((41-iSubProc) >= 12 && (41-iSubProc) <= 15)));
	SubProcDaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 19) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 19)));
	SubProcUqcd  = ((info.Iqcd == 1) && ((iSubProc >= 2 && iSubProc <= 5) || ((41-iSubProc) >= 2 && (41-iSubProc) <= 5)));
	SubProcDaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 9) || ((41-iSubProc) >= 9 && (41-iSubProc) <= 9)));
    }
//--------------------------------------------------------------------------------------------------
    sqS = sqrt(S);
    scales_.omega = info.Ome*sqS/2.;
    Cth4 = -Cth4;
    if (info.Pid == 201) {
	if (SubProcUqed)  nc_ha_1_13__13_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqed) nc_ha_13_1__13_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqed)  nc_ha_1_14__14_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqed) nc_ha_14_1__14_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUqcd)  nc_qcd_ha_23_13__13_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqcd) nc_qcd_ha_13_23__13_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqcd)  nc_qcd_ha_23_14__14_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqcd) nc_qcd_ha_14_23__14_12_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 202) {
	if (SubProcUqed)  nc_ha_1_13__13_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqed) nc_ha_13_1__13_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqed)  nc_ha_1_14__14_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqed) nc_ha_14_1__14_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUqcd)  nc_qcd_ha_23_13__13_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqcd) nc_qcd_ha_13_23__13_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqcd)  nc_qcd_ha_23_14__14_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqcd) nc_qcd_ha_14_23__14_16_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 203) {
	if (SubProcUqed)  nc_ha_1_13__13_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqed) nc_ha_13_1__13_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqed)  nc_ha_1_14__14_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqed) nc_ha_14_1__14_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUqcd)  nc_qcd_ha_23_13__13_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqcd) nc_qcd_ha_13_23__13_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqcd)  nc_qcd_ha_23_14__14_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqcd) nc_qcd_ha_14_23__14_20_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 211) {
	if (SubProcUaqed) cc_ha_13_1__14_11_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqed)  cc_ha_1_14__13_11_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqcd) cc_qcd_ha_13_23__14_11_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqcd)  cc_qcd_ha_23_14__13_11_12_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 212) {
	if (SubProcUaqed) cc_ha_13_1__14_15_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqed)  cc_ha_1_14__13_15_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqcd) cc_qcd_ha_13_23__14_15_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqcd)  cc_qcd_ha_23_14__13_15_16_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 213) {
	if (SubProcUaqed) cc_ha_13_1__14_19_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqed)  cc_ha_1_14__13_19_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUaqcd) cc_qcd_ha_13_23__14_19_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDqcd)  cc_qcd_ha_23_14__13_19_20_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 221) {
	if (SubProcUqed)  cc_ha_1_13__14_12_11_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqed) cc_ha_14_1__13_12_11_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUqcd)  cc_qcd_ha_23_13__14_12_11_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqcd) cc_qcd_ha_14_23__13_12_11_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 222) {
	if (SubProcUqed)  cc_ha_1_13__14_16_15_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqed) cc_ha_14_1__13_16_15_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUqcd)  cc_qcd_ha_23_13__14_16_15_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqcd) cc_qcd_ha_14_23__13_16_15_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    } else if (info.Pid == 223) {
	if (SubProcUqed)  cc_ha_1_13__14_20_19_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqed) cc_ha_14_1__13_20_19_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcUqcd)  cc_qcd_ha_23_13__14_20_19_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
	if (SubProcDaqcd) cc_qcd_ha_14_23__13_20_19_(&S,&Spr,&Phi4,&Cth4,&Cth5,IbaqQ);
    }
}

void Process::Ibs1(double *Z, double *Ibs1Q) {
    bool SubProcDqed(false),SubProcUqed(false),SubProcDaqed(false),SubProcUaqed(false);
    bool SubProcDqcd(false),SubProcUqcd(false),SubProcDaqcd(false),SubProcUaqcd(false);
    int iSubProc;
    double Cth,sqS;
    double kappa3,kappa4,beta3,beta4;
    double S,T,U,Qs,Qt,Qu;
    double SoftQ(0.),HardQ(0.);
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    sqS = Z[1];
    iSubProc = (int)(Z[5]*40.) + 1;
    if (info.Pid == 201 || info.Pid == 202 || info.Pid == 203) {
	SubProcUqed  = ((info.Iph >= 2)  && ((iSubProc >= 11 && iSubProc <= 15 && (iSubProc-10)%2 == 0) || ((41-iSubProc) >= 11 && (41-iSubProc) <= 15 && (31-iSubProc)%2 == 0)));
	SubProcUaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 20 && (iSubProc-15)%2 == 0) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 20 && (26-iSubProc)%2 == 0)));
	SubProcDqed  = ((info.Iph >= 2)  && ((iSubProc >= 11 && iSubProc <= 15 && (iSubProc-10)%2 == 1) || ((41-iSubProc) >= 11 && (41-iSubProc) <= 15 && (31-iSubProc)%2 == 1)));
	SubProcDaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 20 && (iSubProc-15)%2 == 1) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 20 && (26-iSubProc)%2 == 1)));
	SubProcUqcd  = ((info.Iqcd == 1) && ((iSubProc >= 1 && iSubProc <= 5 && (iSubProc)%2 == 0) || ((41-iSubProc) >= 1 && (41-iSubProc) <= 5 && (41-iSubProc)%2 == 0)));
	SubProcUaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 10 && (iSubProc-5)%2 == 0) || ((41-iSubProc) >= 6 && (41-iSubProc) <= 10 && (36-iSubProc)%2 == 0)));
	SubProcDqcd  = ((info.Iqcd == 1) && ((iSubProc >= 1 && iSubProc <= 5 && (iSubProc)%2 == 1) || ((41-iSubProc) >= 1 && (41-iSubProc) <= 5 && (41-iSubProc)%2 == 1)));
	SubProcDaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 10 && (iSubProc-5)%2 == 1) || ((41-iSubProc) >= 6 && (41-iSubProc) <= 10 && (36-iSubProc)%2 == 1)));
    } else if (info.Pid == 211 || info.Pid == 212 || info.Pid == 213) {
	SubProcUaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 19) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 19)));
	SubProcDqed  = ((info.Iph >= 2)  && ((iSubProc >= 12 && iSubProc <= 15) || ((41-iSubProc) >= 12 && (41-iSubProc) <= 15)));
	SubProcUaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 9) || ((41-iSubProc) >= 6 && (41-iSubProc) <= 9)));
	SubProcDqcd  = ((info.Iqcd == 1) && ((iSubProc >= 2 && iSubProc <= 5) || ((41-iSubProc) >= 2 && (41-iSubProc) <= 5)));
    } else if (info.Pid == 221 || info.Pid == 222 || info.Pid == 223) {
	SubProcUqed  = ((info.Iph >= 2)  && ((iSubProc >= 12 && iSubProc <= 15) || ((41-iSubProc) >= 12 && (41-iSubProc) <= 15)));
	SubProcDaqed = ((info.Iph >= 2)  && ((iSubProc >= 16 && iSubProc <= 19) || ((41-iSubProc) >= 16 && (41-iSubProc) <= 19)));
	SubProcUqcd  = ((info.Iqcd == 1) && ((iSubProc >= 2 && iSubProc <= 5) || ((41-iSubProc) >= 2 && (41-iSubProc) <= 5)));
	SubProcDaqcd = ((info.Iqcd == 1) && ((iSubProc >= 6 && iSubProc <= 9) || ((41-iSubProc) >= 9 && (41-iSubProc) <= 9)));
    }
//--------------------------------------------------------------------------------------------------
    S = sqS*sqS;
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    T = Mf3_2-S/2.*kappa3*(1.-beta3*Cth);
    U = Mf4_2-S/2.*kappa4*(1.+beta4*Cth);
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    if (info.Pid == 201) {
	if (SubProcUqed || SubProcUqcd)   nc_br_1313_1212_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcDqed || SubProcDqcd)   nc_br_1414_1212_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd) nc_br_1313_1212_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcDaqed || SubProcDaqcd) nc_br_1414_1212_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 202) {
	if (SubProcUqed || SubProcUqcd)   nc_br_1313_1616_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcDqed || SubProcDqcd)   nc_br_1414_1616_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd) nc_br_1313_1616_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcDaqed || SubProcDaqcd) nc_br_1414_1616_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 203) {
	if (SubProcUqed || SubProcUqcd)   nc_br_1313_2020_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcDqed || SubProcDqcd)   nc_br_1414_2020_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd) nc_br_1313_2020_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcDaqed || SubProcDaqcd) nc_br_1414_2020_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 211) {
	if (SubProcUqed || SubProcUqcd || SubProcDqed || SubProcDqcd)     cc_br_1314_1112_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd || SubProcDaqed || SubProcDaqcd) cc_br_1314_1112_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 212) {
	if (SubProcUqed || SubProcUqcd || SubProcDqed || SubProcDqcd)     cc_br_1314_1516_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd || SubProcDaqed || SubProcDaqcd) cc_br_1314_1516_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 213) {
	if (SubProcUqed || SubProcUqcd || SubProcDqed || SubProcDqcd)     cc_br_1314_1920_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd || SubProcDaqed || SubProcDaqcd) cc_br_1314_1920_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 221) {
	if (SubProcUqed || SubProcUqcd || SubProcDqed || SubProcDqcd)     cc_br_1413_1211_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd || SubProcDaqed || SubProcDaqcd) cc_br_1413_1211_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 222) {
	if (SubProcUqed || SubProcUqcd || SubProcDqed || SubProcDqcd)     cc_br_1413_1615_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd || SubProcDaqed || SubProcDaqcd) cc_br_1413_1615_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
    } else if (info.Pid == 223) {
	if (SubProcUqed || SubProcUqcd || SubProcDqed || SubProcDqcd)     cc_br_1413_2019_(&Qs,&Qu,&Qt,Ibs1Q,&SoftQ,&HardQ);
	if (SubProcUaqed || SubProcUaqcd || SubProcDaqed || SubProcDaqcd) cc_br_1413_2019_(&Qs,&Qt,&Qu,Ibs1Q,&SoftQ,&HardQ);
    }
}

void Process::Ibs2(double *Z, double *Ibs2Q) {
    double Cth,sqS;
    double kappa3,kappa4,beta3,beta4;
    double S,T,U,Qs,Qt,Qu;
    double SoftQ(0.),HardQ(0.);
//--------------------------------------------------------------------------------------------------
    Cth = Z[0];
    sqS = Z[1];
//--------------------------------------------------------------------------------------------------
    S = sqS*sqS;
    kappa3 = 1.+(Mf3_2-Mf4_2)/S;
    kappa4 = 1.+(Mf4_2-Mf3_2)/S;
    beta3 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa3;
    beta4 = sqrt(1.-(Mf3+Mf4)*(Mf3+Mf4)/S)*sqrt(1.-(Mf3-Mf4)*(Mf3-Mf4)/S)/kappa4;
    T = Mf3_2-S/2.*kappa3*(1.-beta3*Cth);
    U = Mf4_2-S/2.*kappa4*(1.+beta4*Cth);
//--------------------------------------------------------------------------------------------------
    Qs = -S;
    Qt = -T;
    Qu = -U;
    flags_.iborn = 0;
//--------------------------------------------------------------------------------------------------
    if (info.Pid == 201) {
	nc_br_11_1212_(&Qs,&Qt,&Qu,Ibs2Q,&SoftQ,&HardQ);
    } else if (info.Pid == 202) {
	nc_br_11_1616_(&Qs,&Qt,&Qu,Ibs2Q,&SoftQ,&HardQ);
    } else if (info.Pid == 203) {
	nc_br_11_2020_(&Qs,&Qt,&Qu,Ibs2Q,&SoftQ,&HardQ);
    }
}

void Process::AFB(Vec4 *P[6], double *XS) {
    double cosCS, XSF(0.), XSB(0.);
    Vec4 v3 = *P[3];
    Vec4 v4 = *P[4];
    if (info.Ibeams == 1) {
	cosCS = (v3.pz()+v4.pz())/abs(v3.pz()+v4.pz())
	*((v3.e()+v3.pz())*(v4.e()-v4.pz())-(v3.e()-v3.pz())*(v4.e()+v4.pz()))
	/m(v3, v4)/sqrt(m(v3, v4)*m(v3, v4)+(v3.px()+v4.px())*(v3.px()+v4.px())+(v3.py()+v4.py())*(v3.py()+v4.py()));
    } else {
	cosCS = 1.
	*((v3.e()+v3.pz())*(v4.e()-v4.pz())-(v3.e()-v3.pz())*(v4.e()+v4.pz()))
	/m(v3, v4)/sqrt(m(v3, v4)*m(v3, v4)+(v3.px()+v4.px())*(v3.px()+v4.px())+(v3.py()+v4.py())*(v3.py()+v4.py()));
    }
    if (cosCS > 0.) XSF = *XS; else XSB = *XS;
    if (info.Iafb == 1) {
	*XS = XSF-XSB;
    } else if (info.Iafb == 2) {
	*XS = XSF+XSB;
    } else if (info.Iafb == 3) {
	*XS = XSF;
    } else if (info.Iafb == 4) {
	*XS = XSB;
    }
}
