#include "ReneSANCe_ee.h"
#include "TRandom3.h"
#include <iostream>
#include <iomanip>
#include <sys/stat.h>
#include <complex>
using namespace std;
typedef complex<double> double_complex;
#include "clooptools.h"
#include "TFoam.h"
#include "Integrand.h"
#include "TFile.h"
#include "TTree.h"
#include "Info.h"
#include "Process.h"
#include "ModulesInterface.h"
#include "Time.h"
#include "DataSaver.h"


extern void LHEInit(void);

using namespace SANC;

#define SP12 setprecision(12) << setw(21)
#define SW18 setw(18)
#define SW3  setw(3)
#define SW6  setw(6)


ReneSANCe_ee::ReneSANCe_ee(Settings & _cfg, Info & _info, Process & _proc, ModulesInterface & _mod, Time & _time, DataSaver & _ds):
cfg(_cfg),timing(_time),info(_info),proc(_proc),modules(_mod),ds(_ds)
{
    PseRan=new TRandom3();
    PseRan->SetSeed(info.RanSeed);
    ffini();
}

void ReneSANCe_ee::generateGrid()
{

}

void ReneSANCe_ee::generateEvents()
{
    TFoam *FoamBorn,*FoamVirt,*FoamHard,*FoamLLI11,*FoamLLI22,*FoamLLI12,*FoamLLF11,*FoamLLF22,*FoamLLF12;
    TFoamIntegrand *Density0,*Density1,*Density2,*Density4,*Density5,*Density6,*Density7,*Density8,*Density9;
    TFile *fileBorn,*fileVirt,*fileHard,*fileLLI11,*fileLLI22,*fileLLI12,*fileLLF11,*fileLLF22,*fileLLF12;
    string bornName, bornNameRoot, bornNameDat;
    string loopName, loopNameRoot, loopNameDat;
    bool bBorn,bVirt,bHard;
    bBorn = true;
    bVirt = !info.Iborn && !info.ForceOffSV && (info.Iew || info.Iqed || info.Iqedlli || info.Iqedllf || info.Ifgg);
    bHard = !info.Iborn && !info.ForceOffH && info.Iqed;
    bornName = "events_LO";
    loopName = "events_NLO";
    if (info.Alr == 1) {
		bornName = bornName+"dif";
		loopName = loopName+"dif";
    } else if (info.Alr == 2) {
		bornName = bornName + "sum";
		loopName = loopName + "sum";
    }
    bornNameRoot = info.outputDir+"/"+bornName+".root";
    bornNameDat = info.outputDir+"/"+bornName+".dat";
    loopNameRoot = info.outputDir+"/"+loopName+".root";
    loopNameDat = info.outputDir+"/"+loopName+".dat";
    timing.startClock();
    if (info.IEXPLBORN) {
	FoamBorn = new TFoam("FoamBorn");   // Create Simulator
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetkDim(proc.kDimBorn);
	FoamBorn->SetnCells(info.nCellsBorn);
	FoamBorn->SetnSampl(info.nSamplBorn);
	FoamBorn->SetnBin(info.nBinBorn);
	FoamBorn->SetEvPerBin(info.EvPerBinBorn);
	FoamBorn->SetOptRej(info.OptRej);
	FoamBorn->SetOptDrive(info.OptDrive);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->SetChat(info.Chat);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetInhiDiv(1, 1);
	FoamBorn->Initialize();
	TFile RootBornFile("FoamBorn.root","RECREATE","Born");
	FoamBorn->Write("FoamBorn");    // Writing Foam on the disk
    } else {
	fileBorn = new TFile("FoamBorn.root"); // connect disk file
	FoamBorn = (TFoam*)fileBorn->Get("FoamBorn"); // find object
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->LinkCells(); // restore pointers of the binary tree of cells
	FoamBorn->GetTotDim();
    }
    timing.stopClock();
    timing.ReportClock("Born initialization");
	FoamBorn->GetPrimary(MCresult);
	if (MCresult<1.e-300) {
        bBorn = false;
        bVirt = false;
    }
    if (bVirt) {
	timing.startClock();
	if (info.IEXPLVIRT) {
	    FoamVirt = new TFoam("FoamVirt");   // Create Simulator
	    Density1 = new Integrand(info, proc, 1);   // Create integrand function
	    FoamVirt->SetkDim(proc.kDimVirt);
	    FoamVirt->SetnCells(info.nCellsVirt);
	    FoamVirt->SetnSampl(info.nSamplVirt);
	    FoamVirt->SetnBin(info.nBinVirt);
	    FoamVirt->SetEvPerBin(info.EvPerBinVirt);
	    FoamVirt->SetOptRej(info.OptRej);
	    FoamVirt->SetOptDrive(info.OptDrive);
	    FoamVirt->SetMaxWtRej(info.MaxWtRejVirt);
	    FoamVirt->SetChat(info.Chat);
	    FoamVirt->SetRho(Density1);
	    FoamVirt->SetPseRan(PseRan);
	    FoamVirt->SetInhiDiv(1, 1);
	    FoamVirt->Initialize();
	    TFile RootVirtFile("FoamVirt.root","RECREATE","Virt");
	    FoamVirt->Write("FoamVirt");    // Writing Foam on the disk
	} else {
	    fileVirt = new TFile("FoamVirt.root"); // connect disk file
	    FoamVirt = (TFoam*)fileVirt->Get("FoamVirt"); // find object
	    Density1 = new Integrand(info, proc, 1);   // Create integrand function
	    FoamVirt->SetPseRan(PseRan);
	    FoamVirt->SetRho(Density1);
	    FoamVirt->SetMaxWtRej(info.MaxWtRejVirt);
	    FoamVirt->LinkCells(); // restore pointers of the binary tree of cells
	    FoamVirt->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("virt initialization");
    }
    if (bHard) {
	timing.startClock();
	if (info.IEXPLHARD) {
	    FoamHard = new TFoam("FoamHard");   // Create Simulator
	    Density2 = new Integrand(info, proc, 2);   // Create integrand function
	    FoamHard->SetkDim(proc.kDimHard);
	    FoamHard->SetnCells(info.nCellsHard);
	    FoamHard->SetnSampl(info.nSamplHard);
	    FoamHard->SetnBin(info.nBinHard);
	    FoamHard->SetEvPerBin(info.EvPerBinHard);
	    FoamHard->SetOptRej(info.OptRej);
	    FoamHard->SetOptDrive(info.OptDrive);
	    FoamHard->SetMaxWtRej(info.MaxWtRejHard);
	    FoamHard->SetChat(info.Chat);
	    FoamHard->SetRho(Density2);
	    FoamHard->SetPseRan(PseRan);
	    FoamHard->SetInhiDiv(4, 1);
	    FoamHard->Initialize();
	    TFile RootHardFile("FoamHard.root","RECREATE","Hard");
	    FoamHard->Write("FoamHard");    // Writing Foam on the disk
	} else {
	    fileHard = new TFile("FoamHard.root"); // connect disk file
	    FoamHard = (TFoam*)fileHard->Get("FoamHard"); // find object
	    Density2 = new Integrand(info, proc, 2);   // Create integrand function
	    FoamHard->SetPseRan(PseRan);
	    FoamHard->SetRho(Density2);
	    FoamHard->SetMaxWtRej(info.MaxWtRejHard);
	    FoamHard->LinkCells(); // restore pointers of the binary tree of cells
	    FoamHard->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("hard initialization");
    FoamHard->GetPrimary(MCresult);
	if (MCresult<1.e-300) { bHard = false; }
    }
    if (!info.Iborn && info.Iqedlli) {
	timing.startClock();
	if (info.IEXPLLLI11) {
	    FoamLLI11 = new TFoam("FoamLLI11");   // Create Simulator
	    Density4 = new Integrand(info, proc, 4);   // Create integrand function
	    FoamLLI11->SetkDim(proc.kDimLL11);
	    FoamLLI11->SetnCells(info.nCellsLL11);
	    FoamLLI11->SetnSampl(info.nSamplLL11);
	    FoamLLI11->SetnBin(info.nBinLL11);
	    FoamLLI11->SetEvPerBin(info.EvPerBinLL11);
	    FoamLLI11->SetOptRej(info.OptRej);
	    FoamLLI11->SetOptDrive(info.OptDrive);
	    FoamLLI11->SetMaxWtRej(info.MaxWtRejLL11);
	    FoamLLI11->SetChat(info.Chat);
	    FoamLLI11->SetRho(Density4);
	    FoamLLI11->SetPseRan(PseRan);
	    FoamLLI11->SetInhiDiv(1, 1);
	    FoamLLI11->Initialize();
	    TFile RootLLI11File("FoamLLI11.root","RECREATE","LLI11");
	    FoamLLI11->Write("FoamLLI11");    // Writing Foam on the disk
	} else {
	    fileLLI11 = new TFile("FoamLLI11.root"); // connect disk file
	    FoamLLI11 = (TFoam*)fileLLI11->Get("FoamLLI11"); // find object
	    Density4 = new Integrand(info, proc, 4);   // Create integrand function
	    FoamLLI11->SetPseRan(PseRan);
	    FoamLLI11->SetRho(Density4);
	    FoamLLI11->SetMaxWtRej(info.MaxWtRejLL11);
	    FoamLLI11->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLLI11->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("lli11 initialization");
    }
    if (!info.Iborn && info.Iqedlli) {
	timing.startClock();
	if (info.IEXPLLLI22) {
	    FoamLLI22 = new TFoam("FoamLLI22");   // Create Simulator
	    Density5 = new Integrand(info, proc, 5);   // Create integrand function
	    FoamLLI22->SetkDim(proc.kDimLL22);
	    FoamLLI22->SetnCells(info.nCellsLL22);
	    FoamLLI22->SetnSampl(info.nSamplLL22);
	    FoamLLI22->SetnBin(info.nBinLL22);
	    FoamLLI22->SetEvPerBin(info.EvPerBinLL22);
	    FoamLLI22->SetOptRej(info.OptRej);
	    FoamLLI22->SetOptDrive(info.OptDrive);
	    FoamLLI22->SetMaxWtRej(info.MaxWtRejLL22);
	    FoamLLI22->SetChat(info.Chat);
	    FoamLLI22->SetRho(Density5);
	    FoamLLI22->SetPseRan(PseRan);
	    FoamLLI22->SetInhiDiv(1, 1);
	    FoamLLI22->Initialize();
	    TFile RootLLI22File("FoamLLI22.root","RECREATE","LLI22");
	    FoamLLI22->Write("FoamLLI22");    // Writing Foam on the disk
	} else {
	    fileLLI22 = new TFile("FoamLLI22.root"); // connect disk file
	    FoamLLI22 = (TFoam*)fileLLI22->Get("FoamLLI22"); // find object
	    Density5 = new Integrand(info, proc, 5);   // Create integrand function
	    FoamLLI22->SetPseRan(PseRan);
	    FoamLLI22->SetRho(Density5);
	    FoamLLI22->SetMaxWtRej(info.MaxWtRejLL22);
	    FoamLLI22->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLLI22->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("lli22 initialization");
    }
    if (!info.Iborn && (info.Iqedlli >1) && (info.Iqedlli !=3) && (info.Iqedlli !=4)) {
	timing.startClock();
	if (info.IEXPLLLI12) {
	    FoamLLI12 = new TFoam("FoamLLI12");   // Create Simulator
	    Density6 = new Integrand(info, proc, 6);   // Create integrand function
	    FoamLLI12->SetkDim(proc.kDimLL12);
	    FoamLLI12->SetnCells(info.nCellsLL12);
	    FoamLLI12->SetnSampl(info.nSamplLL12);
	    FoamLLI12->SetnBin(info.nBinLL12);
	    FoamLLI12->SetEvPerBin(info.EvPerBinLL12);
	    FoamLLI12->SetOptRej(info.OptRej);
	    FoamLLI12->SetOptDrive(info.OptDrive);
	    FoamLLI12->SetMaxWtRej(info.MaxWtRejLL12);
	    FoamLLI12->SetChat(info.Chat);
	    FoamLLI12->SetRho(Density6);
	    FoamLLI12->SetPseRan(PseRan);
//	    FoamLLI12->SetXdivPRD(2, 1, X1Div);
//	    FoamLLI12->SetXdivPRD(3, 1, X2Div);
	    FoamLLI12->SetInhiDiv(1, 1);
	    FoamLLI12->Initialize();
	    TFile RootLLI12File("FoamLLI12.root","RECREATE","LLI12");
	    FoamLLI12->Write("FoamLLI12");    // Writing Foam on the disk
	} else {
	    fileLLI12 = new TFile("FoamLLI12.root"); // connect disk file
	    FoamLLI12 = (TFoam*)fileLLI12->Get("FoamLLI12"); // find object
	    Density6 = new Integrand(info, proc, 6);   // Create integrand function
	    FoamLLI12->SetPseRan(PseRan);
	    FoamLLI12->SetRho(Density6);
	    FoamLLI12->SetMaxWtRej(info.MaxWtRejLL12);
	    FoamLLI12->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLLI12->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("lli12 initialization");
    }
    if (!info.Iborn && info.Iqedllf) {
	timing.startClock();
	if (info.IEXPLLLF11) {
	    FoamLLF11 = new TFoam("FoamLLF11");   // Create Simulator
	    Density7 = new Integrand(info, proc, 7);   // Create integrand function
	    FoamLLF11->SetkDim(proc.kDimLL11);
	    FoamLLF11->SetnCells(info.nCellsLL11);
	    FoamLLF11->SetnSampl(info.nSamplLL11);
	    FoamLLF11->SetnBin(info.nBinLL11);
	    FoamLLF11->SetEvPerBin(info.EvPerBinLL11);
	    FoamLLF11->SetOptRej(info.OptRej);
	    FoamLLF11->SetOptDrive(info.OptDrive);
	    FoamLLF11->SetMaxWtRej(info.MaxWtRejLL11);
	    FoamLLF11->SetChat(info.Chat);
	    FoamLLF11->SetRho(Density7);
	    FoamLLF11->SetPseRan(PseRan);
	    FoamLLF11->SetInhiDiv(1, 1);
	    FoamLLF11->Initialize();
	    TFile RootLLF11File("FoamLLF11.root","RECREATE","LLF11");
	    FoamLLF11->Write("FoamLLF11");    // Writing Foam on the disk
	} else {
	    fileLLF11 = new TFile("FoamLLF11.root"); // connect disk file
	    FoamLLF11 = (TFoam*)fileLLF11->Get("FoamLLF11"); // find object
	    Density7 = new Integrand(info, proc, 7);   // Create integrand function
	    FoamLLF11->SetPseRan(PseRan);
	    FoamLLF11->SetRho(Density7);
	    FoamLLF11->SetMaxWtRej(info.MaxWtRejLL11);
	    FoamLLF11->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLLF11->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("llf11 initialization");
    }
    if (!info.Iborn && info.Iqedllf) {
	timing.startClock();
	if (info.IEXPLLLF22) {
	    FoamLLF22 = new TFoam("FoamLLF22");   // Create Simulator
	    Density8 = new Integrand(info, proc, 8);   // Create integrand function
	    FoamLLF22->SetkDim(proc.kDimLL22);
	    FoamLLF22->SetnCells(info.nCellsLL22);
	    FoamLLF22->SetnSampl(info.nSamplLL22);
	    FoamLLF22->SetnBin(info.nBinLL22);
	    FoamLLF22->SetEvPerBin(info.EvPerBinLL22);
	    FoamLLF22->SetOptRej(info.OptRej);
	    FoamLLF22->SetOptDrive(info.OptDrive);
	    FoamLLF22->SetMaxWtRej(info.MaxWtRejLL22);
	    FoamLLF22->SetChat(info.Chat);
	    FoamLLF22->SetRho(Density8);
	    FoamLLF22->SetPseRan(PseRan);
	    FoamLLF22->SetInhiDiv(1, 1);
	    FoamLLF22->Initialize();
	    TFile RootLLF22File("FoamLLF22.root","RECREATE","LLF22");
	    FoamLLF22->Write("FoamLLF22");    // Writing Foam on the disk
	} else {
	    fileLLF22 = new TFile("FoamLLF22.root"); // connect disk file
	    FoamLLF22 = (TFoam*)fileLLF22->Get("FoamLLF22"); // find object
	    Density8 = new Integrand(info, proc, 8);   // Create integrand function
	    FoamLLF22->SetPseRan(PseRan);
	    FoamLLF22->SetRho(Density8);
	    FoamLLF22->SetMaxWtRej(info.MaxWtRejLL22);
	    FoamLLF22->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLLF22->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("llf22 initialization");
    }
    if (!info.Iborn && (info.Iqedllf >1) && (info.Iqedllf !=3) && (info.Iqedllf !=4)) {
	timing.startClock();
	if (info.IEXPLLLF12) {
	    FoamLLF12 = new TFoam("FoamLLF12");   // Create Simulator
	    Density9 = new Integrand(info, proc, 9);   // Create integrand function
	    FoamLLF12->SetkDim(proc.kDimLL12);
	    FoamLLF12->SetnCells(info.nCellsLL12);
	    FoamLLF12->SetnSampl(info.nSamplLL12);
	    FoamLLF12->SetnBin(info.nBinLL12);
	    FoamLLF12->SetEvPerBin(info.EvPerBinLL12);
	    FoamLLF12->SetOptRej(info.OptRej);
	    FoamLLF12->SetOptDrive(info.OptDrive);
	    FoamLLF12->SetMaxWtRej(info.MaxWtRejLL12);
	    FoamLLF12->SetChat(info.Chat);
	    FoamLLF12->SetRho(Density9);
	    FoamLLF12->SetPseRan(PseRan);
//	    FoamLLF12->SetXdivPRD(2, 1, X1Div);
//	    FoamLLF12->SetXdivPRD(3, 1, X2Div);
	    FoamLLF12->SetInhiDiv(1, 1);
	    FoamLLF12->Initialize();
	    TFile RootLLF12File("FoamLLF12.root","RECREATE","LLF12");
	    FoamLLF12->Write("FoamLLF12");    // Writing Foam on the disk
	} else {
	    fileLLF12 = new TFile("FoamLLF12.root"); // connect disk file
	    FoamLLF12 = (TFoam*)fileLLF12->Get("FoamLLF12"); // find object
	    Density9 = new Integrand(info, proc, 9);   // Create integrand function
	    FoamLLF12->SetPseRan(PseRan);
	    FoamLLF12->SetRho(Density9);
	    FoamLLF12->SetMaxWtRej(info.MaxWtRejLL12);
	    FoamLLF12->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLLF12->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("llf12 initialization");
    }
    timing.startClock();
    cout << "====== Initialization done, entering MC loop" << '\n';
//--------------------------------------------------------------------------------------------------
    double *MCvectBorn = new double[proc.kDimBorn];   // vector generated in the MC run
    double *MCvectVirt = new double[proc.kDimVirt];   // vector generated in the MC run
    double *MCvectHard = new double[proc.kDimHard];   // vector generated in the MC run
    double *MCvectLLI11 = new double[proc.kDimLL11];   // vector generated in the MC run
    double *MCvectLLI22 = new double[proc.kDimLL22];   // vector generated in the MC run
    double *MCvectLLI12 = new double[proc.kDimLL12];   // vector generated in the MC run
    double *MCvectLLF11 = new double[proc.kDimLL11];   // vector generated in the MC run
    double *MCvectLLF22 = new double[proc.kDimLL22];   // vector generated in the MC run
    double *MCvectLLF12 = new double[proc.kDimLL12];   // vector generated in the MC run
//--------------------------------------------------------------------------------------------------
    int iBornPWt1  = 0, iBornNWt1  = 0;
    int iBornPWtN1 = 0, iBornNWtN1 = 0;
    int iVirtPWt1  = 0, iVirtNWt1  = 0;
    int iVirtPWtN1 = 0, iVirtNWtN1 = 0;
    int iHardPWt1  = 0, iHardNWt1  = 0;
    int iHardPWtN1 = 0, iHardNWtN1 = 0;
    int iLLI11PWt1  = 0, iLLI11NWt1  = 0;
    int iLLI11PWtN1 = 0, iLLI11NWtN1 = 0;
    int iLLI22PWt1  = 0, iLLI22NWt1  = 0;
    int iLLI22PWtN1 = 0, iLLI22NWtN1 = 0;
    int iLLI12PWt1  = 0, iLLI12NWt1  = 0;
    int iLLI12PWtN1 = 0, iLLI12NWtN1 = 0;
    int iLLF11PWt1  = 0, iLLF11NWt1  = 0;
    int iLLF11PWtN1 = 0, iLLF11NWtN1 = 0;
    int iLLF22PWt1  = 0, iLLF22NWt1  = 0;
    int iLLF22PWtN1 = 0, iLLF22NWtN1 = 0;
    int iLLF12PWt1  = 0, iLLF12NWt1  = 0;
    int iLLF12PWtN1 = 0, iLLF12NWtN1 = 0;
//--------------------------------------------------------------------------------------------------
    for (int i = 0; i < 9; i++) {
        branch[i] = 0.;
        branch_err[i] = 0.;
    }
//--------------------------------------------------------------------------------------------------
    mkdir(info.outputDir.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
//--------------------------------------------------------------------------------------------------
	if (bBorn) {
		ds.start(info.outputDir, bornName);
//--------------------------------------------------------------------------------------------------
		TFile *fborn = nullptr;
        if (info.PrintROOT) {
            fborn = new TFile(bornNameRoot.c_str(),"recreate");
        }
		TTree *tborn = new TTree("events_born","born events");
		Double_t weight_born[1], px_born[3], py_born[3], pz_born[3], E_born[3];
		tborn->Branch("weight_born", weight_born, "weight_born[1]/D");
		tborn->Branch("px_born", px_born, "px_born[3]/D");
		tborn->Branch("py_born", py_born, "py_born[3]/D");
		tborn->Branch("pz_born", pz_born, "pz_born[3]/D");
		tborn->Branch("E_born", E_born, "E_born[3]/D");
//--------------------------------------------------------------------------------------------------
		for (loop = 0; loop < info.NevBorn; loop++) {
		FoamBorn->MakeEvent(); // generate Born MC event
		FoamBorn->GetMCvect(MCvectBorn);
		MCwt = FoamBorn->GetMCwt();
		xwgtup = proc.event.weight();
		if (MCwt == 1.0) {
			weight_born[0] = xwgtup;
			px_born[0] = proc.event.P_lab(3).px();
			px_born[1] = proc.event.P_lab(4).px();
			px_born[2] = proc.event.P_lab(5).px();
			py_born[0] = proc.event.P_lab(3).py();
			py_born[1] = proc.event.P_lab(4).py();
			py_born[2] = proc.event.P_lab(5).py();
			pz_born[0] = proc.event.P_lab(3).pz();
			pz_born[1] = proc.event.P_lab(4).pz();
			pz_born[2] = proc.event.P_lab(5).pz();
			E_born[0]  = proc.event.P_lab(3).e();
			E_born[1]  = proc.event.P_lab(4).e();
			E_born[2]  = proc.event.P_lab(5).e();
            proc.nup = 4;
			if ( info.PrintLHE ) ds.writeEvent();
			if ( info.PrintROOT ) tborn->Fill();
			if (xwgtup > 0.0) iBornPWt1++;
			if (xwgtup < 0.0) iBornNWt1++;
		} else {
			if (xwgtup > 0.0) iBornPWtN1++;
			if (xwgtup < 0.0) iBornNWtN1++;
            loop--;
		}
		}
		if ( info.PrintROOT ) fborn->Write();
		timing.stopClock();
		timing.ReportClock("Born generation");
    	FoamBorn->GetIntegMC(MCresult, MCerror); // get MC integral
        branch[0] = MCresult;
        branch_err[0] = MCerror;
        info.Sigma = MCresult;
        info.SigmaErr = MCerror;
        ds.finish();
	}
//--------------------------------------------------------------------------------------------------
    if (bVirt) {
        timing.startClock();
        for (loop = 0; loop < info.NevVirt; loop++) {
            FoamVirt->MakeEvent(); // generate Virt MC event
            FoamVirt->GetMCvect(MCvectVirt);
            MCwt = FoamVirt->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("virt generation");
        FoamVirt->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[1] = MCresult;
        branch_err[1] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (bHard) {
        timing.startClock();
        for (loop = 0; loop < info.NevHard; loop++) {
            FoamHard->MakeEvent();   // generate Hard MC event
            FoamHard->GetMCvect(MCvectHard);
            MCwt = FoamHard->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("hard generation");
        FoamHard->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[2] = MCresult;
        branch_err[2] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedlli) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL11; loop++) {
            FoamLLI11->MakeEvent();   // generate LL11 MC event
            FoamLLI11->GetMCvect(MCvectLLI11);
            MCwt = FoamLLI11->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("lli11 generation");
        FoamLLI11->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[3] = MCresult;
        branch_err[3] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedlli) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL22; loop++) {
            FoamLLI22->MakeEvent();   // generate LL22 MC event
            FoamLLI22->GetMCvect(MCvectLLI22);
            MCwt = FoamLLI22->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("lli22 generation");
        FoamLLI22->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[4] = MCresult;
        branch_err[4] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && (info.Iqedlli >1) && (info.Iqedlli !=3) && (info.Iqedlli !=4)) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL12; loop++) {
            FoamLLI12->MakeEvent();   // generate LL12 MC event
            FoamLLI12->GetMCvect(MCvectLLI12);
            MCwt = FoamLLI12->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("lli12 generation");
        FoamLLI12->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[5] = MCresult;
        branch_err[5] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedllf) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL11; loop++) {
            FoamLLF11->MakeEvent();   // generate LLF11 MC event
            FoamLLF11->GetMCvect(MCvectLLF11);
            MCwt = FoamLLF11->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("llf11 generation");
        FoamLLF11->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[6] = MCresult;
        branch_err[6] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedllf) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL22; loop++) {
            FoamLLF22->MakeEvent();   // generate LL22 MC event
            FoamLLF22->GetMCvect(MCvectLLF22);
            MCwt = FoamLLF22->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("llf22 generation");
        FoamLLF22->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[7] = MCresult;
        branch_err[7] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && (info.Iqedllf >1) && (info.Iqedllf !=3) && (info.Iqedllf !=4)) {
        timing.startClock();
        for (loop = 0; loop < info.NevLL12; loop++) {
            FoamLLF12->MakeEvent();   // generate LL12 MC event
            FoamLLF12->GetMCvect(MCvectLLI12);
            MCwt = FoamLLF12->GetMCwt();
        }
        timing.stopClock();
        timing.ReportClock("llf12 generation");
        FoamLLF12->GetIntegMC(MCresult, MCerror);  // get MC integral
        branch[8] = MCresult;
        branch_err[8] = MCerror;
    }
//--------------------------------------------------------------------------------------------------
    info.SigmaBorn = branch[0];
    info.SigmaVirt = branch[1];
    info.SigmaHard = branch[2];
    info.SigmaLLI11 = branch[3];
    info.SigmaLLI22 = branch[4];
    info.SigmaLLI12 = branch[5];
    info.SigmaLLF11 = branch[6];
    info.SigmaLLF22 = branch[7];
    info.SigmaLLF12 = branch[8];
    info.SigmaTot = 0.;
    for (int i = 1; i < 9; i++) {
	    info.SigmaTot += branch[i];
    }
//--------------------------------------------------------------------------------------------------
    cout << "-------------------" << '\n';
    cout << "Branching weights:" << '\n';
    cout << "Born = " << branch[0] << " +- " << branch_err[0] << '\n';
    cout << "Virt = " << branch[1] << " +- " << branch_err[1] << '\n';
    cout << "Hard = " << branch[2] << " +- " << branch_err[2] << '\n';
    cout << "LLI11 = " << branch[3] << " +- " << branch_err[3] << '\n';
    cout << "LLI22 = " << branch[4] << " +- " << branch_err[4] << '\n';
    cout << "LLI12 = " << branch[5] << " +- " << branch_err[5] << '\n';
    cout << "LLF11 = " << branch[6] << " +- " << branch_err[6] << '\n';
    cout << "LLF22 = " << branch[7] << " +- " << branch_err[7] << '\n';
    cout << "LLF12 = " << branch[8] << " +- " << branch_err[8] << '\n';
    cout << "-------------------" << endl;

//--------------------------------------------------------------------------------------------------
	if (info.SigmaTot>1.e-300) {
//--------------------------------------------------------------------------------------------------
	    ds.start(info.outputDir, loopName);
//--------------------------------------------------------------------------------------------------
        double rBr;
        double Br[7];
        Br[0] = branch[1]/info.SigmaTot;
        for (int i = 1; i < 7; i++) {
            Br[i] = Br[i-1]+branch[i+1]/info.SigmaTot;
        }
//--------------------------------------------------------------------------------------------------
        TFile *ftot = nullptr;
        if (info.PrintROOT) {
            ftot = new TFile(loopNameRoot.c_str(),"recreate");
        }
        TTree *ttot = new TTree("events_tot","tot events");
        Double_t weight_tot[1], px_tot[3], py_tot[3], pz_tot[3], E_tot[3];
        ttot->Branch("weight_tot", weight_tot, "weight_tot[1]/D");
        ttot->Branch("px_tot", px_tot, "px_tot[3]/D");
        ttot->Branch("py_tot", py_tot, "py_tot[3]/D");
        ttot->Branch("pz_tot", pz_tot, "pz_tot[3]/D");
        ttot->Branch("E_tot", E_tot, "E_tot[3]/D");
//--------------------------------------------------------------------------------------------------
        TFile *fhardvar = nullptr;
        if (info.PrintHard) {
            fhardvar = new TFile((info.outputDir+"/hardvar.root").c_str(),"recreate");
        }
        TTree *thardvar = new TTree("events_hardvar","tot events");
        Double_t weight_hardvar[1], hardvar[5];
        thardvar->Branch("weight_hardvar", weight_hardvar, "weight_hardvar[1]/D");
        thardvar->Branch("hardvar", hardvar, "hardvar[5]/D");
//--------------------------------------------------------------------------------------------------
        timing.startClock();
        for (loop = 0; loop < info.NevTot; loop++) {
            rBr = (double) random()/RAND_MAX;
//--------------------------------------------------------------------------------------------------
            if (rBr <= Br[0] && !info.Iborn) {
                FoamVirt->MakeEvent();   // generate Virt MC event
                FoamVirt->GetMCvect(MCvectVirt);
                MCwt = FoamVirt->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(3).px();
                px_tot[1] = proc.event.P_lab(4).px();
                px_tot[2] = proc.event.P_lab(5).px();
                py_tot[0] = proc.event.P_lab(3).py();
                py_tot[1] = proc.event.P_lab(4).py();
                py_tot[2] = proc.event.P_lab(5).py();
                pz_tot[0] = proc.event.P_lab(3).pz();
                pz_tot[1] = proc.event.P_lab(4).pz();
                pz_tot[2] = proc.event.P_lab(5).pz();
                E_tot[0]  = proc.event.P_lab(3).e();
                E_tot[1]  = proc.event.P_lab(4).e();
                E_tot[2]  = proc.event.P_lab(5).e();
                proc.nup = 4;
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintLHE) ds.writeEvent();
                if (xwgtup > 0.0) iVirtPWt1++;
                if (xwgtup < 0.0) iVirtNWt1++;
                } else {
                if (xwgtup > 0.0) iVirtPWtN1++;
                if (xwgtup < 0.0) iVirtNWtN1++;
                loop--;
                }
            }
    //--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[1] && !info.Iborn && info.Iqed) {
                FoamHard->MakeEvent();   // generate Hard MC event
                FoamHard->GetMCvect(MCvectHard);
                MCwt = FoamHard->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(3).px();
                px_tot[1] = proc.event.P_lab(4).px();
                px_tot[2] = proc.event.P_lab(5).px();
                py_tot[0] = proc.event.P_lab(3).py();
                py_tot[1] = proc.event.P_lab(4).py();
                py_tot[2] = proc.event.P_lab(5).py();
                pz_tot[0] = proc.event.P_lab(3).pz();
                pz_tot[1] = proc.event.P_lab(4).pz();
                pz_tot[2] = proc.event.P_lab(5).pz();
                E_tot[0]  = proc.event.P_lab(3).e();
                E_tot[1]  = proc.event.P_lab(4).e();
                E_tot[2]  = proc.event.P_lab(5).e();
                weight_hardvar[0] = xwgtup;
                for (int i = 0; i < 5; i++) hardvar[i] = MCvectHard[i];
                proc.nup = 5;
                if (info.PrintLHE) ds.writeEvent();
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintHard) thardvar->Fill();
                if (xwgtup > 0.0) iHardPWt1++;
                if (xwgtup < 0.0) iHardNWt1++;
                } else {
                if (xwgtup > 0.0) iHardPWtN1++;
                if (xwgtup < 0.0) { iHardNWtN1++; /*MCwt = -MCwt;*/ }
                loop--;
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[2] && !info.Iborn && info.Iqedlli) {
                FoamLLI11->MakeEvent();   // generate LL11 MC event
                FoamLLI11->GetMCvect(MCvectLLI11);
                MCwt = FoamLLI11->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(3).px();
                px_tot[1] = proc.event.P_lab(4).px();
                px_tot[2] = proc.event.P_lab(5).px();
                py_tot[0] = proc.event.P_lab(3).py();
                py_tot[1] = proc.event.P_lab(4).py();
                py_tot[2] = proc.event.P_lab(5).py();
                pz_tot[0] = proc.event.P_lab(3).pz();
                pz_tot[1] = proc.event.P_lab(4).pz();
                pz_tot[2] = proc.event.P_lab(5).pz();
                E_tot[0]  = proc.event.P_lab(3).e();
                E_tot[1]  = proc.event.P_lab(4).e();
                E_tot[2]  = proc.event.P_lab(5).e();
                proc.nup = 4;
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintLHE) ds.writeEvent();
                if (xwgtup > 0.0) iLLI11PWt1++;
                if (xwgtup < 0.0) iLLI11NWt1++;
                } else {
                if (xwgtup > 0.0) iLLI11PWtN1++;
                if (xwgtup < 0.0) iLLI11NWtN1++;
                loop--;
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[3] && !info.Iborn && info.Iqedlli) {
                FoamLLI22->MakeEvent();   // generate LL22 MC event
                FoamLLI22->GetMCvect(MCvectLLI22);
                MCwt = FoamLLI22->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(2).px();
                px_tot[1] = proc.event.P_lab(3).px();
                px_tot[2] = proc.event.P_lab(4).px();
                py_tot[0] = proc.event.P_lab(2).py();
                py_tot[1] = proc.event.P_lab(3).py();
                py_tot[2] = proc.event.P_lab(4).py();
                pz_tot[0] = proc.event.P_lab(2).pz();
                pz_tot[1] = proc.event.P_lab(3).pz();
                pz_tot[2] = proc.event.P_lab(4).pz();
                E_tot[0]  = proc.event.P_lab(2).e();
                E_tot[1]  = proc.event.P_lab(3).e();
                E_tot[2]  = proc.event.P_lab(4).e();
                proc.nup = 4;
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintLHE) ds.writeEvent();
                if (xwgtup > 0.0) iLLI22PWt1++;
                if (xwgtup < 0.0) iLLI22NWt1++;
                } else {
                if (xwgtup > 0.0) iLLI22PWtN1++;
                if (xwgtup < 0.0) iLLI22NWtN1++;
                loop--;
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[4] && !info.Iborn && (info.Iqedlli >1) && (info.Iqedlli !=3) && (info.Iqedlli !=4)) {
                FoamLLI12->MakeEvent();   // generate LL12 MC event
                FoamLLI12->GetMCvect(MCvectLLI12);
                MCwt = FoamLLI12->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(2).px();
                px_tot[1] = proc.event.P_lab(3).px();
                px_tot[2] = proc.event.P_lab(4).px();
                py_tot[0] = proc.event.P_lab(2).py();
                py_tot[1] = proc.event.P_lab(3).py();
                py_tot[2] = proc.event.P_lab(4).py();
                pz_tot[0] = proc.event.P_lab(2).pz();
                pz_tot[1] = proc.event.P_lab(3).pz();
                pz_tot[2] = proc.event.P_lab(4).pz();
                E_tot[0]  = proc.event.P_lab(2).e();
                E_tot[1]  = proc.event.P_lab(3).e();
                E_tot[2]  = proc.event.P_lab(4).e();
                proc.nup = 4;
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintLHE) ds.writeEvent();
                if (xwgtup > 0.0) iLLI12PWt1++;
                if (xwgtup < 0.0) iLLI12NWt1++;
                } else {
                if (xwgtup > 0.0) iLLI12PWtN1++;
                if (xwgtup < 0.0) iLLI12NWtN1++;
                loop--;
                }
            }
            else if (rBr <= Br[5] && !info.Iborn && info.Iqedllf) {
                FoamLLF11->MakeEvent();   // generate LLF11 MC event
                FoamLLF11->GetMCvect(MCvectLLF11);
                MCwt = FoamLLF11->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(3).px();
                px_tot[1] = proc.event.P_lab(4).px();
                px_tot[2] = proc.event.P_lab(5).px();
                py_tot[0] = proc.event.P_lab(3).py();
                py_tot[1] = proc.event.P_lab(4).py();
                py_tot[2] = proc.event.P_lab(5).py();
                pz_tot[0] = proc.event.P_lab(3).pz();
                pz_tot[1] = proc.event.P_lab(4).pz();
                pz_tot[2] = proc.event.P_lab(5).pz();
                E_tot[0]  = proc.event.P_lab(3).e();
                E_tot[1]  = proc.event.P_lab(4).e();
                E_tot[2]  = proc.event.P_lab(5).e();
                proc.nup = 4;
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintLHE) ds.writeEvent();
                if (xwgtup > 0.0) iLLF11PWt1++;
                if (xwgtup < 0.0) iLLF11NWt1++;
                } else {
                if (xwgtup > 0.0) iLLF11PWtN1++;
                if (xwgtup < 0.0) iLLF11NWtN1++;
                loop--;
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (rBr <= Br[6] && !info.Iborn && info.Iqedllf) {
                FoamLLF22->MakeEvent();   // generate LLF22 MC event
                FoamLLF22->GetMCvect(MCvectLLF22);
                MCwt = FoamLLF22->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(3).px();
                px_tot[1] = proc.event.P_lab(4).px();
                px_tot[2] = proc.event.P_lab(5).px();
                py_tot[0] = proc.event.P_lab(3).py();
                py_tot[1] = proc.event.P_lab(4).py();
                py_tot[2] = proc.event.P_lab(5).py();
                pz_tot[0] = proc.event.P_lab(3).pz();
                pz_tot[1] = proc.event.P_lab(4).pz();
                pz_tot[2] = proc.event.P_lab(5).pz();
                E_tot[0]  = proc.event.P_lab(3).e();
                E_tot[1]  = proc.event.P_lab(4).e();
                E_tot[2]  = proc.event.P_lab(5).e();
                proc.nup = 4;
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintLHE) ds.writeEvent();
                if (xwgtup > 0.0) iLLF22PWt1++;
                if (xwgtup < 0.0) iLLF22NWt1++;
                } else {
                if (xwgtup > 0.0) iLLF22PWtN1++;
                if (xwgtup < 0.0) iLLF22NWtN1++;
                loop--;
                }
            }
//--------------------------------------------------------------------------------------------------
            else if (!info.Iborn && (info.Iqedllf >1) && (info.Iqedllf !=3) && (info.Iqedllf !=4)) {
                FoamLLF12->MakeEvent();   // generate LLF12 MC event
                FoamLLF12->GetMCvect(MCvectLLF12);
                MCwt = FoamLLF12->GetMCwt();
                xwgtup = proc.event.weight();
                if (MCwt == 1.0) {
                weight_tot[0] = xwgtup;
                px_tot[0] = proc.event.P_lab(3).px();
                px_tot[1] = proc.event.P_lab(4).px();
                px_tot[2] = proc.event.P_lab(5).px();
                py_tot[0] = proc.event.P_lab(3).py();
                py_tot[1] = proc.event.P_lab(4).py();
                py_tot[2] = proc.event.P_lab(5).py();
                pz_tot[0] = proc.event.P_lab(3).pz();
                pz_tot[1] = proc.event.P_lab(4).pz();
                pz_tot[2] = proc.event.P_lab(5).pz();
                E_tot[0]  = proc.event.P_lab(3).e();
                E_tot[1]  = proc.event.P_lab(4).e();
                E_tot[2]  = proc.event.P_lab(5).e();
                proc.nup = 4;
                if (info.PrintROOT) ttot->Fill();
                if (info.PrintLHE) ds.writeEvent();
                if (xwgtup > 0.0) iLLF12PWt1++;
                if (xwgtup < 0.0) iLLF12NWt1++;
                } else {
                if (xwgtup > 0.0) iLLF12PWtN1++;
                if (xwgtup < 0.0) iLLF12NWtN1++;
                loop--;
                }
            }
        }
        if (info.PrintROOT) ftot->Write();
        if (info.PrintHard) fhardvar->Write();
        timing.stopClock();
        timing.ReportClock("Total generation");
    }
//--------------------------------------------------------------------------------------------------
    double BornF = 1.0;
    double VirtF = 1.0;
    double HardF = 1.0;
    double LLI11F = 1.0;
    double LLI22F = 1.0;
    double LLI12F = 1.0;
    double LLF11F = 1.0;
    double LLF22F = 1.0;
    double LLF12F = 1.0;
    if ((iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) > 0) BornF = (double) (iBornPWt1-iBornNWt1+iBornPWtN1-iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1);
    if ((iVirtPWt1+iVirtNWt1+iVirtPWtN1+iVirtNWtN1) > 0) VirtF = (double) (iVirtPWt1-iVirtNWt1+iVirtPWtN1-iVirtNWtN1)/(iVirtPWt1+iVirtNWt1+iVirtPWtN1+iVirtNWtN1);
    if ((iHardPWt1+iHardNWt1+iHardPWtN1+iHardNWtN1) > 0) HardF = (double) (iHardPWt1-iHardNWt1+iHardPWtN1-iHardNWtN1)/(iHardPWt1+iHardNWt1+iHardPWtN1+iHardNWtN1);
    if ((iLLI11PWt1+iLLI11NWt1+iLLI11PWtN1+iLLI11NWtN1) > 0) LLI11F = (double) (iLLI11PWt1-iLLI11NWt1+iLLI11PWtN1-iLLI11NWtN1)/(iLLI11PWt1+iLLI11NWt1+iLLI11PWtN1+iLLI11NWtN1);
    if ((iLLI22PWt1+iLLI22NWt1+iLLI22PWtN1+iLLI22NWtN1) > 0) LLI22F = (double) (iLLI22PWt1-iLLI22NWt1+iLLI22PWtN1-iLLI22NWtN1)/(iLLI22PWt1+iLLI22NWt1+iLLI22PWtN1+iLLI22NWtN1);
    if ((iLLI12PWt1+iLLI12NWt1+iLLI12PWtN1+iLLI12NWtN1) > 0) LLI12F = (double) (iLLI12PWt1-iLLI12NWt1+iLLI12PWtN1-iLLI12NWtN1)/(iLLI12PWt1+iLLI12NWt1+iLLI12PWtN1+iLLI12NWtN1);
    if ((iLLF11PWt1+iLLF11NWt1+iLLF11PWtN1+iLLF11NWtN1) > 0) LLF11F = (double) (iLLF11PWt1-iLLF11NWt1+iLLF11PWtN1-iLLF11NWtN1)/(iLLF11PWt1+iLLF11NWt1+iLLF11PWtN1+iLLF11NWtN1);
    if ((iLLF22PWt1+iLLF22NWt1+iLLF22PWtN1+iLLF22NWtN1) > 0) LLF22F = (double) (iLLF22PWt1-iLLF22NWt1+iLLF22PWtN1-iLLF22NWtN1)/(iLLF22PWt1+iLLF22NWt1+iLLF22PWtN1+iLLF22NWtN1);
    if ((iLLF12PWt1+iLLF12NWt1+iLLF12PWtN1+iLLF12NWtN1) > 0) LLF12F = (double) (iLLF12PWt1-iLLF12NWt1+iLLF12PWtN1-iLLF12NWtN1)/(iLLF12PWt1+iLLF12NWt1+iLLF12PWtN1+iLLF12NWtN1);
//--------------------------------------------------------------------------------------------------
    cout << "====== Events generated, entering Finalize" << '\n';
    Double_t IntNorm, Errel;
    FoamBorn->Finalize(IntNorm, Errel);
    double SigmaBornAbs(0.),SigmaVirtAbs(0.),SigmaHardAbs(0.),SigmaLLI11Abs(0.),SigmaLLI22Abs(0.),SigmaLLI12Abs(0.),SigmaLLF11Abs(0.),SigmaLLF22Abs(0.),SigmaLLF12Abs(0.);
    double ErrBorn(0.),ErrVirt(0.),ErrHard(0.),ErrLLI11(0.),ErrLLI22(0.),ErrLLI12(0.),ErrLLF11(0.),ErrLLF22(0.),ErrLLF12(0.);
    double aveWt,wtMax,sigma;
//--------------------------------------------------------------------------------------------------
    FoamBorn->GetIntegMC(MCresult, MCerror);  // get MC integral
    info.SigmaBorn = BornF*MCresult;
    ErrBorn = MCerror;
    SigmaBornAbs = MCresult;
//--------------------------------------------------------------------------------------------------
    if (bVirt) {
        FoamVirt->Finalize(IntNorm, Errel);
        FoamVirt->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaVirt = VirtF*MCresult;
        ErrVirt = MCerror;
        SigmaVirtAbs = MCresult;
    }
//--------------------------------------------------------------------------------------------------
    if (bHard) {
        FoamHard->Finalize(IntNorm, Errel);
        FoamHard->GetIntegMC(MCresult, MCerror);  // get MC integral
        FoamHard->GetWtParams(0.0001,aveWt,wtMax,sigma);
        info.SigmaHard = HardF*MCresult;
        ErrHard = MCerror;
        SigmaHardAbs = MCresult;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedlli) {
        FoamLLI11->Finalize(IntNorm, Errel);
        FoamLLI11->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLLI11 = LLI11F*MCresult;
        ErrLLI11 = MCerror;
        SigmaLLI11Abs = MCresult;
        delete [] MCvectLLI11;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedlli) {
        FoamLLI22->Finalize(IntNorm, Errel);
        FoamLLI22->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLLI22 = LLI22F*MCresult;
        ErrLLI22 = MCerror;
        SigmaLLI22Abs = MCresult;
        delete [] MCvectLLI22;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && (info.Iqedlli >1) && (info.Iqedlli !=3) && (info.Iqedlli !=4)) {
        FoamLLI12->Finalize(IntNorm, Errel);
        FoamLLI12->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLLI12 = LLI12F*MCresult;
        ErrLLI12 = MCerror;
        SigmaLLI12Abs = MCresult;
        delete [] MCvectLLI12;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedllf) {
        FoamLLF11->Finalize(IntNorm, Errel);
        FoamLLF11->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLLF11 = LLF11F*MCresult;
        ErrLLF11 = MCerror;
        SigmaLLF11Abs = MCresult;
        delete [] MCvectLLF11;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && info.Iqedllf) {
        FoamLLF22->Finalize(IntNorm, Errel);
        FoamLLF22->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLLF22 = LLF22F*MCresult;
        ErrLLF22 = MCerror;
        SigmaLLF22Abs = MCresult;
        delete [] MCvectLLF22;
    }
//--------------------------------------------------------------------------------------------------
    if (!info.Iborn && (info.Iqedllf >1) && (info.Iqedllf !=3) && (info.Iqedllf !=4)) {
        FoamLLF12->Finalize(IntNorm, Errel);
        FoamLLF12->GetIntegMC(MCresult, MCerror);  // get MC integral
        info.SigmaLLF12 = LLF12F*MCresult;
        ErrLLF12 = MCerror;
        SigmaLLF12Abs = MCresult;
        delete [] MCvectLLF12;
    }
//--------------------------------------------------------------------------------------------------
    info.SigmaTot = info.SigmaVirt+info.SigmaHard+info.SigmaLLI11+info.SigmaLLI22+info.SigmaLLI12+info.SigmaLLF11+info.SigmaLLF22+info.SigmaLLF12;
    double SigmaDel = info.SigmaVirt+info.SigmaHard+info.SigmaLLI11+info.SigmaLLI22+info.SigmaLLI12+info.SigmaLLF11+info.SigmaLLF22+info.SigmaLLF12-info.SigmaBorn;
    double SigmaTotAbs = SigmaVirtAbs+SigmaHardAbs+SigmaLLI11Abs+SigmaLLI22Abs+SigmaLLI12Abs+SigmaLLF11Abs+SigmaLLF22Abs+SigmaLLF12Abs;
    double ErrTot = sqrt(ErrVirt*ErrVirt+ErrHard*ErrHard+ErrLLI11*ErrLLI11+ErrLLI22*ErrLLI22+ErrLLI12*ErrLLI12+ErrLLF11*ErrLLF11+ErrLLF22*ErrLLF22+ErrLLF12*ErrLLF12);
    double ErrDel = sqrt(ErrVirt*ErrVirt+ErrHard*ErrHard+ErrLLI11*ErrLLI11+ErrLLI22*ErrLLI22+ErrLLI12*ErrLLI12+ErrLLF11*ErrLLF11+ErrLLF22*ErrLLF22+ErrLLF12*ErrLLF12+ErrBorn*ErrBorn);
    double Delta = SigmaDel/info.SigmaBorn;
    double ErrDelta = abs(Delta)*sqrt(pow2(ErrDel/SigmaDel)+pow2(ErrBorn/info.SigmaBorn));
    info.Sigma = info.SigmaTot;
    info.SigmaErr = ErrTot;
    if (info.PrintLHE) {
        if (info.SigmaTot>1.e-300) {
            ds.finish();
        }
    }
    if (info.PrintROOT) {
        ofstream sigma_brn;
        sigma_brn.open(bornNameDat);
        sigma_brn << setprecision(10) << SigmaBornAbs << '\n';
        sigma_brn.close();
        if (info.SigmaTot>1.e-300) {
            ofstream sigma_tot;
            sigma_tot.open(loopNameDat);
            sigma_tot << setprecision(10) << SigmaTotAbs << '\n';
            sigma_tot.close();
        }
    }
//--------------------------------------------------------------------------------------------------
    delete [] MCvectBorn;
    delete [] MCvectVirt;
    delete [] MCvectHard;
//--------------------------------------------------------------------------------------------------
    std::cout << std::setprecision(14);
    cout << "================================================================" << '\n';
    cout << "        MCresult (Born)  = " << info.SigmaBorn << " +- " << ErrBorn << " RelErr = " << ErrBorn/info.SigmaBorn << '\n';
	cout << "        MCresult (B+S+V) = " << info.SigmaVirt << " +- " << ErrVirt << " RelErr = " << ErrVirt/info.SigmaVirt << '\n';
	cout << "        MCresult (Hard)  = " << info.SigmaHard << " +- " << ErrHard << " RelErr = " << ErrHard/info.SigmaHard << '\n';
    cout << "        MCresult (LL11)  = " << info.SigmaLLI11 << " +- " << ErrLLI11 << " RelErr = " << ErrLLI11/info.SigmaLLI11 << '\n';
    cout << "        MCresult (LL22)  = " << info.SigmaLLI22 << " +- " << ErrLLI22 << " RelErr = " << ErrLLI22/info.SigmaLLI22 << '\n';
    cout << "        MCresult (LL12)  = " << info.SigmaLLI12 << " +- " << ErrLLI12 << " RelErr = " << ErrLLI12/info.SigmaLLI12 << '\n';
    cout << "        MCresult (LLF11)  = " << info.SigmaLLF11 << " +- " << ErrLLF11 << " RelErr = " << ErrLLF11/info.SigmaLLF11 << '\n';
    cout << "        MCresult (LLF22)  = " << info.SigmaLLF22 << " +- " << ErrLLF22 << " RelErr = " << ErrLLF22/info.SigmaLLF22 << '\n';
    cout << "        MCresult (LLF12)  = " << info.SigmaLLF12 << " +- " << ErrLLF12 << " RelErr = " << ErrLLF12/info.SigmaLLF12 << '\n';
    cout << "        MCresult (Tot)   = " << info.SigmaTot << " +- " << ErrTot << " RelErr = " << ErrTot/info.SigmaTot << '\n';
    cout << "        MCresult (Delta) = " << Delta << " +- " << ErrDelta << " RelErr = " << ErrDelta/Delta << '\n';
    cout << "================================================================" << '\n';
    cout << "WtMax(for 1e-4 accuracy goal) = "<<wtMax<< '\n';
	cout << "================================================================" << '\n';
    cout << "Total number of Born events:  " << iBornPWt1 + iBornPWtN1 << " pos. " << iBornNWt1 + iBornNWtN1 << " neg." << '\n';
    cout << "Total number of B+S+V events: " << iVirtPWt1 + iVirtPWtN1 << " pos. " << iVirtNWt1 + iVirtNWtN1 << " neg." << '\n';
    cout << "Total number of Hard events:  " << iHardPWt1 + iHardPWtN1 << " pos. " << iHardNWt1 + iHardNWtN1 << " neg." << '\n';
    cout << "Total number of LLI11 events:  " << iLLI11PWt1 + iLLI11PWtN1 << " pos. " << iLLI11NWt1 + iLLI11NWtN1 << " neg." << '\n';
    cout << "Total number of LLI22 events:  " << iLLI22PWt1 + iLLI22PWtN1 << " pos. " << iLLI22NWt1 + iLLI22NWtN1 << " neg." << '\n';
    cout << "Total number of LLI12 events:  " << iLLI12PWt1 + iLLI12PWtN1 << " pos. " << iLLI12NWt1 + iLLI12NWtN1 << " neg." << '\n';
    cout << "Total number of LLF11 events:  " << iLLF11PWt1 + iLLF11PWtN1 << " pos. " << iLLF11NWt1 + iLLF11NWtN1 << " neg." << '\n';
    cout << "Total number of LLF22 events:  " << iLLF22PWt1 + iLLF22PWtN1 << " pos. " << iLLF22NWt1 + iLLF22NWtN1 << " neg." << '\n';
    cout << "Total number of LLF12 events:  " << iLLF12PWt1 + iLLF12PWtN1 << " pos. " << iLLF12NWt1 + iLLF12NWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Number of Born events with Wt > 1:  " << iBornPWtN1 << " pos. " << iBornNWtN1 << " neg." << '\n';
    cout << "Number of B+S+V events with Wt > 1: " << iVirtPWtN1 << " pos. " << iVirtNWtN1 << " neg." << '\n';
    cout << "Number of Hard events with Wt > 1:  " << iHardPWtN1 << " pos. " << iHardNWtN1 << " neg." << '\n';
    cout << "Number of LL11 events with Wt > 1:  " << iLLI11PWtN1 << " pos. " << iLLI11NWtN1 << " neg." << '\n';
    cout << "Number of LL22 events with Wt > 1:  " << iLLI22PWtN1 << " pos. " << iLLI22NWtN1 << " neg." << '\n';
    cout << "Number of LL12 events with Wt > 1:  " << iLLI12PWtN1 << " pos. " << iLLI12NWtN1 << " neg." << '\n';
    cout << "Number of LLF11 events with Wt > 1:  " << iLLF11PWtN1 << " pos. " << iLLF11NWtN1 << " neg." << '\n';
    cout << "Number of LLF22 events with Wt > 1:  " << iLLF22PWtN1 << " pos. " << iLLF22NWtN1 << " neg." << '\n';
    cout << "Number of LLF12 events with Wt > 1:  " << iLLF12PWtN1 << " pos. " << iLLF12NWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Relative number of Born events with Wt > 1:  " << (double) (iBornPWtN1+iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) << '\n';
    cout << "Relative number of B+S+V events with Wt > 1: " << (double) (iVirtPWtN1+iVirtNWtN1)/(iVirtPWt1+iVirtNWt1+iVirtPWtN1+iVirtNWtN1) << '\n';
    cout << "Relative number of Hard events with Wt > 1:  " << (double) (iHardPWtN1+iHardNWtN1)/(iHardPWt1+iHardNWt1+iHardPWtN1+iHardNWtN1) << '\n';
    cout << "Relative number of LL11 events with Wt > 1:  " << (double) (iLLI11PWtN1+iLLI11NWtN1)/(iLLI11PWt1+iLLI11NWt1+iLLI11PWtN1+iLLI11NWtN1) << '\n';
    cout << "Relative number of LL22 events with Wt > 1:  " << (double) (iLLI22PWtN1+iLLI22NWtN1)/(iLLI22PWt1+iLLI22NWt1+iLLI22PWtN1+iLLI22NWtN1) << '\n';
    cout << "Relative number of LL12 events with Wt > 1:  " << (double) (iLLI12PWtN1+iLLI12NWtN1)/(iLLI12PWt1+iLLI12NWt1+iLLI12PWtN1+iLLI12NWtN1) << '\n';
    cout << "Relative number of LLF11 events with Wt > 1:  " << (double) (iLLF11PWtN1+iLLF11NWtN1)/(iLLF11PWt1+iLLF11NWt1+iLLF11PWtN1+iLLF11NWtN1) << '\n';
    cout << "Relative number of LLF22 events with Wt > 1:  " << (double) (iLLF22PWtN1+iLLF22NWtN1)/(iLLF22PWt1+iLLF22NWt1+iLLF22PWtN1+iLLF22NWtN1) << '\n';
    cout << "Relative number of LLF12 events with Wt > 1:  " << (double) (iLLF12PWtN1+iLLF12NWtN1)/(iLLF12PWt1+iLLF12NWt1+iLLF12PWtN1+iLLF12NWtN1) << '\n';
    cout << "***** End of Program *****" << '\n';
}

void ReneSANCe_ee::generateEventsNB()
{
    TFoam *FoamBorn,*FoamLoop;
    TFoamIntegrand *Density0,*Density3;
    TFile *fileBorn,*fileLoop;
    timing.startClock();
    if (info.IEXPLBORN) {
	FoamBorn = new TFoam("FoamBorn");   // Create Simulator
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetkDim(proc.kDimBorn);
	FoamBorn->SetnCells(info.nCellsBorn);
	FoamBorn->SetnSampl(info.nSamplBorn);
	FoamBorn->SetnBin(info.nBinBorn);
	FoamBorn->SetEvPerBin(info.EvPerBinBorn);
	FoamBorn->SetOptRej(info.OptRej);
	FoamBorn->SetOptDrive(info.OptDrive);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->SetChat(info.Chat);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetInhiDiv(1, 1);
	FoamBorn->Initialize();
	TFile RootBornFile("FoamBorn.root","RECREATE","Born");
	FoamBorn->Write("FoamBorn");    // Writing Foam on the disk
    } else {
	fileBorn = new TFile("FoamBorn.root"); // connect disk file
	FoamBorn = (TFoam*)fileBorn->Get("FoamBorn"); // find object
	Density0 = new Integrand(info, proc, 0);   // Create integrand function
	FoamBorn->SetPseRan(PseRan);
	FoamBorn->SetRho(Density0);
	FoamBorn->SetMaxWtRej(info.MaxWtRejBorn);
	FoamBorn->LinkCells(); // restore pointers of the binary tree of cells
	FoamBorn->GetTotDim();
    }
    timing.stopClock();
    timing.ReportClock("Born initialization");
    if (!info.Iborn && (info.Iew || info.Iqed)) {
	timing.startClock();
	if (info.IEXPLHARD) {
	    FoamLoop = new TFoam("FoamLoop");   // Create Simulator
	    Density3 = new Integrand(info, proc, 3);   // Create integrand function
	    FoamLoop->SetkDim(proc.kDimHard+1);
	    FoamLoop->SetnCells(info.nCellsHard);
	    FoamLoop->SetnSampl(info.nSamplHard);
	    FoamLoop->SetnBin(info.nBinHard);
	    FoamLoop->SetEvPerBin(info.EvPerBinHard);
	    FoamLoop->SetOptRej(info.OptRej);
	    FoamLoop->SetOptDrive(info.OptDrive);
	    FoamLoop->SetMaxWtRej(info.MaxWtRejHard);
	    FoamLoop->SetChat(info.Chat);
	    FoamLoop->SetRho(Density3);
	    FoamLoop->SetPseRan(PseRan);
	    FoamLoop->SetInhiDiv(5,1);
	    double xDiv[1];
	    xDiv[0]=0.5;
	    FoamLoop->SetXdivPRD(5,1,xDiv);
	    FoamLoop->Initialize();
	    TFile RootVirtFile("FoamLoop.root","RECREATE","Loop");
	    FoamLoop->Write("FoamLoop");    // Writing Foam on the disk
	} else {
	    fileLoop = new TFile("FoamLoop.root"); // connect disk file
	    FoamLoop = (TFoam*)fileLoop->Get("FoamLoop"); // find object
	    Density3 = new Integrand(info, proc, 3);   // Create integrand function
	    FoamLoop->SetPseRan(PseRan);
	    FoamLoop->SetRho(Density3);
	    FoamLoop->SetMaxWtRej(info.MaxWtRejHard);
	    FoamLoop->LinkCells(); // restore pointers of the binary tree of cells
	    FoamLoop->GetTotDim();
	}
	timing.stopClock();
	timing.ReportClock("loop initialization");
    }
    timing.startClock();
    cout << "====== Initialization done, entering MC loop" << '\n';
//--------------------------------------------------------------------------------------------------
    double *MCvectBorn = new double[proc.kDimBorn];   // vector generated in the MC run
    double *MCvectLoop = new double[proc.kDimHard+1];   // vector generated in the MC run
//--------------------------------------------------------------------------------------------------
    int iBornPWt1  = 0, iBornNWt1  = 0;
    int iBornPWtN1 = 0, iBornNWtN1 = 0;
    int iLoopPWt1  = 0, iLoopNWt1  = 0;
    int iLoopPWtN1 = 0, iLoopNWtN1 = 0;
//--------------------------------------------------------------------------------------------------
    mkdir(info.outputDir.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
//--------------------------------------------------------------------------------------------------
	ds.start(info.outputDir, "events_LO");
//--------------------------------------------------------------------------------------------------
    TFile *fborn = new TFile((info.outputDir+"/events_LO.root").c_str(),"recreate");
    TTree *tborn = new TTree("events_born","born events");
    Double_t weight_born[1], px_born[3], py_born[3], pz_born[3], E_born[3];
    tborn->Branch("weight_born", weight_born, "weight_born[1]/D");
    tborn->Branch("px_born", px_born, "px_born[3]/D");
    tborn->Branch("py_born", py_born, "py_born[3]/D");
    tborn->Branch("pz_born", pz_born, "pz_born[3]/D");
    tborn->Branch("E_born", E_born, "E_born[3]/D");
//--------------------------------------------------------------------------------------------------
    for (loop = 0; loop < info.NevBorn; loop++) {
	FoamBorn->MakeEvent();   // generate Born MC event
	FoamBorn->GetMCvect(MCvectBorn);
	MCwt = FoamBorn->GetMCwt();
	xwgtup = proc.event.weight();
	if (MCwt == 1.0) {
	    weight_born[0] = xwgtup;
	    px_born[0] = proc.event.P_lab(3).px();
	    px_born[1] = proc.event.P_lab(4).px();
	    px_born[2] = proc.event.P_lab(5).px();
	    py_born[0] = proc.event.P_lab(3).py();
	    py_born[1] = proc.event.P_lab(4).py();
	    py_born[2] = proc.event.P_lab(5).py();
	    pz_born[0] = proc.event.P_lab(3).pz();
	    pz_born[1] = proc.event.P_lab(4).pz();
	    pz_born[2] = proc.event.P_lab(5).pz();
	    E_born[0]  = proc.event.P_lab(3).e();
	    E_born[1]  = proc.event.P_lab(4).e();
	    E_born[2]  = proc.event.P_lab(5).e();
        proc.nup = 4;
	    if (info.PrintLHE) ds.writeEvent();
	    if (info.PrintROOT) tborn->Fill();
	    if (xwgtup > 0.0) iBornPWt1++;
	    if (xwgtup < 0.0) iBornNWt1++;
	} else {
	    if (xwgtup > 0.0) iBornPWtN1++;
	    if (xwgtup < 0.0) iBornNWtN1++;
        loop--;
	}
    }
    fborn->Write();
    FoamBorn->GetIntegMC(MCresult, MCerror); // get MC integral
    info.Sigma = MCresult;
    info.SigmaErr = MCerror;
    ds.finish();
    timing.stopClock();
    timing.ReportClock("Born generation");
//--------------------------------------------------------------------------------------------------
	ds.start(info.outputDir, "events_NLO");
//--------------------------------------------------------------------------------------------------
    TFile *ftot = new TFile((info.outputDir+"/events_NLO.root").c_str(),"recreate");
    TTree *ttot = new TTree("events_tot","tot events");
    Double_t weight_tot[1], px_tot[3], py_tot[3], pz_tot[3], E_tot[3];
    ttot->Branch("weight_tot", weight_tot, "weight_tot[1]/D");
    ttot->Branch("px_tot", px_tot, "px_tot[3]/D");
    ttot->Branch("py_tot", py_tot, "py_tot[3]/D");
    ttot->Branch("pz_tot", pz_tot, "pz_tot[3]/D");
    ttot->Branch("E_tot", E_tot, "E_tot[3]/D");
//--------------------------------------------------------------------------------------------------
    timing.startClock();
    for (loop = 0; loop < info.NevTot; loop++) {
	FoamLoop->MakeEvent();   // generate Hard MC event
	FoamLoop->GetMCvect(MCvectLoop);
	MCwt = FoamLoop->GetMCwt();
	xwgtup = proc.event.weight();
	if (MCwt == 1.0) {
	    weight_tot[0] = xwgtup;
	    px_tot[0] = proc.event.P_lab(3).px();
	    px_tot[1] = proc.event.P_lab(4).px();
	    px_tot[2] = proc.event.P_lab(5).px();
	    py_tot[0] = proc.event.P_lab(3).py();
	    py_tot[1] = proc.event.P_lab(4).py();
	    py_tot[2] = proc.event.P_lab(5).py();
	    pz_tot[0] = proc.event.P_lab(3).pz();
	    pz_tot[1] = proc.event.P_lab(4).pz();
	    pz_tot[2] = proc.event.P_lab(5).pz();
	    E_tot[0]  = proc.event.P_lab(3).e();
	    E_tot[1]  = proc.event.P_lab(4).e();
	    E_tot[2]  = proc.event.P_lab(5).e();
        proc.nup = 5;
	    if (info.PrintLHE == 1) ds.writeEvent();
	    if (info.PrintROOT == 1) ttot->Fill();
	    if (xwgtup > 0.0) iLoopPWt1++;
	    if (xwgtup < 0.0) iLoopNWt1++;
	} else {
	    if (xwgtup > 0.0) iLoopPWtN1++;
	    if (xwgtup < 0.0) iLoopNWtN1++;
        loop--;
        }
    }
    ftot->Write();
    timing.stopClock();
    timing.ReportClock("Total generation");
//--------------------------------------------------------------------------------------------------
    double BornF = 1.;
    double HardF = 1.;
    if ((iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) > 0) BornF = (double) (iBornPWt1-iBornNWt1+iBornPWtN1-iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1);
    if ((iLoopPWt1+iLoopNWt1+iLoopPWtN1+iLoopNWtN1) > 0) HardF = (double) (iLoopPWt1-iLoopNWt1+iLoopPWtN1-iLoopNWtN1)/(iLoopPWt1+iLoopNWt1+iLoopPWtN1+iLoopNWtN1);
//--------------------------------------------------------------------------------------------------
    cout << "====== Events generated, entering Finalize" << '\n';
    Double_t IntNorm, Errel;
    double aveWt,wtMax,sigma;
    FoamBorn->Finalize(IntNorm, Errel);
    FoamLoop->Finalize(IntNorm, Errel);
    FoamLoop->GetWtParams(0.0001,aveWt,wtMax,sigma);
//--------------------------------------------------------------------------------------------------
    FoamBorn->GetIntegMC(MCresult, MCerror);  // get MC integral
    info.SigmaBorn = BornF*MCresult;
    double ErrBorn = MCerror;
    double SigmaBornAbs = MCresult;
    cout << "================================================================" << '\n';
    cout << "        MCresult (Born) = " << info.SigmaBorn << " +- " << ErrBorn << " RelErr = " << ErrBorn/info.SigmaBorn << '\n';
//--------------------------------------------------------------------------------------------------
    FoamLoop->GetIntegMC(MCresult, MCerror);  // get MC integral
    info.SigmaHard = HardF*MCresult;
    double ErrHard = MCerror;
    double SigmaHardAbs = MCresult;
    cout << "        MCresult (Hard) = " << info.SigmaHard << " +- " << ErrHard << " RelErr = " << ErrHard/info.SigmaHard << '\n';
//--------------------------------------------------------------------------------------------------
    info.SigmaTot = info.SigmaHard;
    double SigmaDel = info.SigmaHard-info.SigmaBorn;
    double SigmaTotAbs = SigmaHardAbs;
    double ErrTot = sqrt(ErrHard*ErrHard);
    double ErrDel = sqrt(ErrHard*ErrHard+ErrBorn*ErrBorn);
    double Delta = SigmaDel/info.SigmaBorn;
    double ErrDelta = Delta*sqrt(ErrDel/SigmaDel*ErrDel/SigmaDel+ErrBorn/info.SigmaBorn*ErrBorn/info.SigmaBorn);
    cout << "        MCresult (Tot)  = " << info.SigmaTot << " +- " << ErrTot << " RelErr = " << ErrTot/info.SigmaTot << '\n';
    cout << "        MCresult (Delta)  = " << Delta << " +- " << ErrDelta << " RelErr = " << ErrDelta/Delta << '\n';
    cout << "================================================================" << '\n';
    cout << "WtMax(esp= 0.0001) = "<<wtMax<< '\n';
    ofstream sigma_brn;
    ofstream sigma_tot;
    sigma_brn.open(info.outputDir+"/events_LO.dat");
    sigma_tot.open(info.outputDir+"/events_NLO.dat");
    sigma_brn << setprecision(10) << SigmaBornAbs << '\n';
    sigma_tot << setprecision(10) << SigmaTotAbs << '\n';
    sigma_brn.close();
    sigma_tot.close();
    info.Sigma = info.SigmaTot;
    info.SigmaErr = ErrTot;
    if (info.PrintLHE) {
        if (info.SigmaTot>1.e-300) {
            ds.finish();
        }
    }
//--------------------------------------------------------------------------------------------------
    delete [] MCvectBorn;
    delete [] MCvectLoop;
//--------------------------------------------------------------------------------------------------
    cout << "Total number of Born events: " << iBornPWt1 + iBornPWtN1 << " pos. " << iBornNWt1 + iBornNWtN1 << " neg." << '\n';
    cout << "Total number of Loop events: " << iLoopPWt1 + iLoopPWtN1 << " pos. " << iLoopNWt1 + iLoopNWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Number of Born events with Wt > 1: " << iBornPWtN1 << " pos. " << iBornNWtN1 << " neg." << '\n';
    cout << "Number of Hard events with Wt > 1: " << iLoopPWtN1 << " pos. " << iLoopNWtN1 << " neg." << '\n';
    cout << "================================================================" << '\n';
    cout << "Relative number of Born events with Wt > 1: " << (double) (iBornPWtN1+iBornNWtN1)/(iBornPWt1+iBornNWt1+iBornPWtN1+iBornNWtN1) << '\n';
    cout << "Relative number of Hard events with Wt > 1: " << (double) (iLoopPWtN1+iLoopNWtN1)/(iLoopPWt1+iLoopNWt1+iLoopPWtN1+iLoopNWtN1) << '\n';
    cout << "***** End of Program *****" << '\n';
}
