# This module tries to find the ROOT installation on your system.
# It tries to find the root-config script which gives you all the needed information.
# If the system variable ROOTSYS is set this is straight forward.
# If not the module uses the pathes given in ROOT_CONFIG_SEARCHPATH.
# If you need an other path you should add this path to this varaible.
# The root-config script is then used to detect basically everything else.
# This module defines a number of key variables and macros.

# This module defines
# ROOT_INCLUDE_DIR, where to locate ROOT include files
# ROOT_LIBRARIES, the libraries to link against to use ROOT
# ROOT_FOUND.  If false, you cannot build anything that requires ROOT.
# ROOT_LIBRARY_DIR, where to find the ROOT libraries.
# ROOT_BINARY_DIR
# ROOT_CFLAGS

message(STATUS "Looking for ROOT framework")

set(ROOT_FOUND 0)

#MESSAGE(STATUS "Looking for Root...")
IF( WIN32 OR MSYS OR MINGW)

MESSAGE(STATUS "Looking for Root under Windows environment")
FIND_PROGRAM(ROOT_EXECUTABLE NAMES root PATHS
    $ENV{ROOTSYS}/bin
   )

 IF( NOT ${ROOT_EXECUTABLE} MATCHES "ROOT_EXECUTABLE-NOTFOUND" )
    MESSAGE(STATUS "Looking for Root... - found $ENV{ROOTSYS}/bin/root")
    set(ROOT_FOUND 1)
    SET(ROOT_LIBRARY_DIR $ENV{ROOTSYS}/lib $ENV{ROOTSYS}/lib64)
#    MESSAGE(STATUS "ROOT LIBRARY DIR in..." ${ROOT_LIBRARY_DIR})
    SET(ROOT_INCLUDE_DIR $ENV{ROOTSYS}/include)
#    MESSAGE(STATUS "ROOT Include dir in..." ${ROOT_INCLUDE_DIR})
    SET(ROOT_BINARY_DIR $ENV{ROOTSYS}/bin)

    SET(CMAKE_FIND_LIBRARY_SUFFIXES ".a;.lib;.so;.dll")
     find_library(ROOT_GUI NAMES Gui PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_CORE NAMES Core PATHS ${ROOT_LIBRARY_DIR})
        MESSAGE(STATUS "ROOT_CORE LIBRARY in..." ${ROOT_CORE})
     find_library(ROOT_CINT NAMES Cint PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_RIO NAMES RIO PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_HIST NAMES Hist PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_GRAF NAMES Graf PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_GPAD NAMES Gpad PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_TREE NAMES Tree PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_POSTSCRIPT NAMES Postscript PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_MATRIX NAMES Matrix PATHS ${ROOT_LIBRARY_DIR})
     find_library(ROOT_MATHCORE NAMES MathCore PATHS ${ROOT_LIBRARY_DIR})

     SET(ROOT_LIBRARIES
         ${ROOT_GUI}
         ${ROOT_CORE}
         ${ROOT_CINT}
         ${ROOT_RIO}
         ${ROOT_HIST}
         ${ROOT_GRAF}
         ${ROOT_GPAD}
         ${ROOT_TREE}
         ${ROOT_POSTSCRIPT}
         ${ROOT_MATRIX}
         ${ROOT_MATHCORE}
     )

   MARK_AS_ADVANCED( ROOT_LIBRARY_DIR ROOT_INCLUDE_DIR ROOT_LIBRARIES ROOT_BINARY_DIR)
 ENDIF( NOT ${ROOT_EXECUTABLE} MATCHES "ROOT_EXECUTABLE-NOTFOUND" )


#  # ask root-config for the library varaibles
#  execute_process(COMMAND ${ROOT_CONFIG_EXECUTABLE} --glibs #--noldflags --noauxlibs --libs
#    OUTPUT_VARIABLE root_flags OUTPUT_STRIP_TRAILING_WHITESPACE)
#  SET(ROOT_LIBRARIES ${root_flags})

  # Make variables changeble to the advanced user

ELSE( WIN32 OR MSYS OR MINGW)



SET(ROOT_CONFIG_SEARCHPATH
  $ENV{ROOTSYS}/bin
  ${ROOTSYS}/bin
  /opt/root/bin
#  /usr/local/bin
#  /usr/bin
)

#SET(ROOT_INSTALLED_VERSION_TOO_OLD FALSE)

#SET(ROOT_CONFIG_EXECUTABLE ROOT_CONFIG_EXECUTABLE-NOTFOUND)

FIND_PROGRAM(ROOT_CONFIG_EXECUTABLE NAMES root-config PATHS
   ${ROOT_CONFIG_SEARCHPATH}
#   NO_DEFAULT_PATH
   )

IF (ROOT_CONFIG_EXECUTABLE)

#  MESSAGE(STATUS "find ROOT_CONFIG_EXECUTABLE at ${ROOT_CONFIG_EXECUTABLE}")
  STRING(REGEX REPLACE "(^.*)/bin/root-config" "\\1" ROOTSYS ${ROOT_CONFIG_EXECUTABLE})
  SET( ENV{ROOTSYS} ${ROOTSYS})

  execute_process(COMMAND ${ROOT_CONFIG_EXECUTABLE} --version OUTPUT_VARIABLE ROOTVERSION OUTPUT_STRIP_TRAILING_WHITESPACE)

  MESSAGE(STATUS "Looking for Root... - found ${ROOTSYS}/bin/root")
  MESSAGE(STATUS "Looking for Root... - version ${ROOTVERSION} ")

  # we need at least version 5.00/00
  IF (NOT ROOT_MIN_VERSION)
    SET(ROOT_MIN_VERSION "5.00/00")
  ENDIF (NOT ROOT_MIN_VERSION)

  # now parse the parts of the user given version string into variables
  STRING(REGEX REPLACE "^([0-9]+)\\.[0-9][0-9]+\\/[0-9][0-9]+" "\\1" req_root_major_vers "${ROOT_MIN_VERSION}")
  STRING(REGEX REPLACE "^[0-9]+\\.([0-9][0-9])+\\/[0-9][0-9]+.*" "\\1" req_root_minor_vers "${ROOT_MIN_VERSION}")
  STRING(REGEX REPLACE "^[0-9]+\\.[0-9][0-9]+\\/([0-9][0-9]+)" "\\1" req_root_patch_vers "${ROOT_MIN_VERSION}")

  # and now the version string given by cmake
  STRING(REGEX REPLACE "^([0-9]+)\\.[0-9][0-9]+\\/[0-9][0-9]+.*" "\\1" found_root_major_vers "${ROOTVERSION}")
  STRING(REGEX REPLACE "^[0-9]+\\.([0-9][0-9])+\\/[0-9][0-9]+.*" "\\1" found_root_minor_vers "${ROOTVERSION}")
  STRING(REGEX REPLACE "^[0-9]+\\.[0-9][0-9]+\\/([0-9][0-9]+).*" "\\1" found_root_patch_vers "${ROOTVERSION}")

  IF (found_root_major_vers LESS 5)
    MESSAGE( FATAL_ERROR "Invalid ROOT version \"${ROOTERSION}\", at least major version 4 is required, e.g. \"5.00/00\"")
  ENDIF (found_root_major_vers LESS 5)

  # compute an overall version number which can be compared at once
  MATH(EXPR req_vers "${req_root_major_vers}*10000 + ${req_root_minor_vers}*100 + ${req_root_patch_vers}")
  MATH(EXPR found_vers "${found_root_major_vers}*10000 + ${found_root_minor_vers}*100 + ${found_root_patch_vers}")

  IF (found_vers LESS req_vers)
    SET(ROOT_FOUND FALSE)
    SET(ROOT_INSTALLED_VERSION_TOO_OLD TRUE)
  ELSE (found_vers LESS req_vers)
    SET(ROOT_FOUND TRUE)
  ENDIF (found_vers LESS req_vers)

  # ask root-config for the library dir
  # Set ROOT_LIBRARY_DIR

  execute_process(COMMAND  ${ROOT_CONFIG_EXECUTABLE} --libdir
    OUTPUT_VARIABLE ROOT_LIBRARY_DIR_TMP OUTPUT_STRIP_TRAILING_WHITESPACE)

  IF(EXISTS "${ROOT_LIBRARY_DIR_TMP}")
    SET(ROOT_LIBRARY_DIR ${ROOT_LIBRARY_DIR_TMP} )
  ELSE(EXISTS "${ROOT_LIBRARY_DIR_TMP}")
    MESSAGE("Warning: ROOT_CONFIG_EXECUTABLE reported ${ROOT_LIBRARY_DIR_TMP} as library path,")
    MESSAGE("Warning: but ${ROOT_LIBRARY_DIR_TMP} does NOT exist, ROOT must NOT be installed correctly.")
  ENDIF(EXISTS "${ROOT_LIBRARY_DIR_TMP}")

  # ask root-config for the binary dir
  execute_process(COMMAND ${ROOT_CONFIG_EXECUTABLE} --bindir
    OUTPUT_VARIABLE root_bins OUTPUT_STRIP_TRAILING_WHITESPACE)
  SET(ROOT_BINARY_DIR ${root_bins})

  # ask root-config for the include dir
  execute_process(COMMAND ${ROOT_CONFIG_EXECUTABLE} --incdir
    OUTPUT_VARIABLE root_headers OUTPUT_STRIP_TRAILING_WHITESPACE)
  SET(ROOT_INCLUDE_DIR ${root_headers})
      # CACHE INTERNAL "")

  # ask root-config for the library varaibles
  execute_process(COMMAND ${ROOT_CONFIG_EXECUTABLE} --glibs #--noldflags --noauxlibs --libs
    OUTPUT_VARIABLE root_flags OUTPUT_STRIP_TRAILING_WHITESPACE)

  # ask root-config for the cpp varaibles
  execute_process(COMMAND ${ROOT_CONFIG_EXECUTABLE} --cflags
    OUTPUT_VARIABLE ROOT_CFLAGS OUTPUT_STRIP_TRAILING_WHITESPACE)

#  STRING(REGEX MATCHALL "([^ ])+"  root_libs_all ${root_flags})
#  STRING(REGEX MATCHALL "-L([^ ])+"  root_library ${root_flags})
#  REMOVE_FROM_LIST(root_flags "${root_libs_all}" "${root_library}")

  SET(ROOT_LIBRARIES ${root_flags})

  # Make variables changeble to the advanced user
  MARK_AS_ADVANCED( ROOT_LIBRARY_DIR ROOT_INCLUDE_DIR ROOT_LIBRARIES ROOT_BINARY_DIR ROOT_CFLAGS)

  # Set ROOT_INCLUDES
#  SET( ROOT_INCLUDES ${ROOT_INCLUDE_DIR})

#  SET(LD_LIBRARY_PATH ${LD_LIBRARY_PATH} ${ROOT_LIBRARY_DIR})

  #######################################
  #
  #       Check the executables of ROOT
  #          ( rootcint )
  #
  #######################################

#  FIND_PROGRAM(ROOT_CINT_EXECUTABLE
#    NAMES rootcint
#    PATHS ${ROOT_BINARY_DIR}
#    NO_DEFAULT_PATH
#    )

ENDIF (ROOT_CONFIG_EXECUTABLE)
ENDIF(WIN32 OR MSYS OR MINGW)
